@@ -17,20 +17,20 @@
  */
 #include "angband.h"
 
+#include "cave.h"
+#include "cmds.h"
+#include "game-cmd.h"
+#include "monster/monster.h"
 #include "object/object.h"
 #include "object/tvalsval.h"
-#include "game-cmd.h"
+#include "spells.h"
+#include "target.h"
 
-/**
- * Determines how likely an object is to break on throwing or shooting.
- *
- * \returns percentage change of breaking
- */
+/* Returns percent chance of an object breaking after throwing or shooting. */
 int breakage_chance(const object_type *o_ptr)
 {
 	/* Artifacts never break */
-	if (artifact_p(o_ptr))
-		return 0;
+	if (artifact_p(o_ptr)) return 0;
 
 	switch (o_ptr->tval)
 	{
@@ -41,7 +41,7 @@ int breakage_chance(const object_type *o_ptr)
 		case TV_JUNK:
 			return 100;
 
-		case TV_LITE:
+		case TV_LIGHT:
 		case TV_SCROLL:
 		case TV_SKELETON:
 			return 50;
@@ -92,7 +92,7 @@ bool test_hit(int chance, int ac, int vis)
  * Critical hits (from objects thrown by player)
  * Factor in item weight, total plusses, and player level.
  */
-static int critical_shot(int weight, int plus, int dam)
+static int critical_shot(int weight, int plus, int dam, u32b *msg_type)
 {
 	int i, k;
 
@@ -106,20 +106,22 @@ static int critical_shot(int weight, int plus, int dam)
 
 		if (k < 500)
 		{
-			msg_print("It was a good hit!");
+			*msg_type = MSG_HIT_GOOD;
 			dam = 2 * dam + 5;
 		}
 		else if (k < 1000)
 		{
-			msg_print("It was a great hit!");
+			*msg_type = MSG_HIT_GREAT;
 			dam = 2 * dam + 10;
 		}
 		else
 		{
-			msg_print("It was a superb hit!");
+			*msg_type = MSG_HIT_SUPERB;
 			dam = 3 * dam + 15;
 		}
 	}
+	else
+		*msg_type = MSG_SHOOT_HIT;
 
 	return (dam);
 }
@@ -131,7 +133,7 @@ static int critical_shot(int weight, int plus, int dam)
  *
  * Factor in weapon weight, total plusses, player level.
  */
-static int critical_norm(int weight, int plus, int dam)
+static int critical_norm(int weight, int plus, int dam, u32b *msg_type)
 {
 	int i, k;
 
@@ -145,41 +147,34 @@ static int critical_norm(int weight, int plus, int dam)
 
 		if (k < 400)
 		{
-			sound(MSG_HIT_GOOD);
-			msg_print("It was a good hit!");
+			*msg_type = MSG_HIT_GOOD;
 			dam = 2 * dam + 5;
 		}
 		else if (k < 700)
 		{
-			sound(MSG_HIT_GREAT);
-			msg_print("It was a great hit!");
+			*msg_type = MSG_HIT_GREAT;
 			dam = 2 * dam + 10;
 		}
 		else if (k < 900)
 		{
-			sound(MSG_HIT_SUPERB);
-			msg_print("It was a superb hit!");
+			*msg_type = MSG_HIT_SUPERB;
 			dam = 3 * dam + 15;
 		}
 		else if (k < 1300)
 		{
-			sound(MSG_HIT_HI_GREAT);
-			msg_print("It was a *GREAT* hit!");
+			*msg_type = MSG_HIT_HI_GREAT;
 			dam = 3 * dam + 20;
 		}
 		else
 		{
-			sound(MSG_HIT_HI_SUPERB);
-			msg_print("It was a *SUPERB* hit!");
+			*msg_type = MSG_HIT_HI_SUPERB;
 			dam = ((7 * dam) / 2) + 25;
 		}
 	}
 	else
-	{
-		sound(MSG_HIT);
-	}
+		*msg_type = MSG_HIT;
 
-	return (dam);
+	return dam;
 }
 
 
@@ -193,71 +188,57 @@ static int critical_norm(int weight, int plus, int dam)
  *
  * \param o_ptr is the object being used to attack
  * \param m_ptr is the monster being attacked
- * \param hit_verb is where a new verb is returned
- * \param is_ranged should be true for ranged attacks
+ * \param best_s_ptr is the best applicable slay_table entry, or NULL if no
+ *  slay already known
  *
- * \returns attack multiplier
  */
-static int get_brand_mult(object_type *o_ptr, const monster_type *m_ptr,
-		const char **hit_verb, bool is_ranged, bool secondary)
+void improve_attack_modifier(object_type *o_ptr, const monster_type *m_ptr, const slay_t **best_s_ptr)
 {
-	int mult = 1;
 	const slay_t *s_ptr;
 
 	monster_race *r_ptr = &r_info[m_ptr->r_idx];
 	monster_lore *l_ptr = &l_list[m_ptr->r_idx];
 
-	u32b f[OBJ_FLAG_N];
+	bitflag f[OF_SIZE], known_f[OF_SIZE];
 	object_flags(o_ptr, f);
+	object_flags_known(o_ptr, known_f);
 
 	for (s_ptr = slay_table; s_ptr->slay_flag; s_ptr++)
 	{
-		if (!(f[0] & s_ptr->slay_flag)) continue;
+		if (!of_has(f, s_ptr->slay_flag)) continue;
 
-		/* notice any brand or slay that would affect the monster */
-		if ((s_ptr->resist_flag) || (r_ptr->flags[2] & s_ptr->monster_flag))
+		/* Learn about monster resistance/vulnerability IF:
+		 * 1) The slay flag on the object is known OR
+		 * 2) The monster does not possess the appropriate resistance flag OR
+		 * 3) The monster does possess the appropriate vulnerability flag
+		 */
+		if (of_has(known_f, s_ptr->slay_flag) ||
+		    (s_ptr->monster_flag && rf_has(r_ptr->flags, s_ptr->monster_flag)) ||
+		    (s_ptr->resist_flag && !rf_has(r_ptr->flags, s_ptr->resist_flag)))
 		{
-			object_notice_slays(o_ptr, s_ptr->slay_flag);
-			wieldeds_notice_slays(s_ptr->slay_flag);
-		}
-
-		/* If the monster doesn't match or the slay flag does */
-		if ((s_ptr->brand && !(r_ptr->flags[2] & s_ptr->resist_flag)) || 
-			(r_ptr->flags[2] & s_ptr->monster_flag))
-		{
-			/* Learn the flag */
-			if (m_ptr->ml)
-				l_ptr->flags[2] |= s_ptr->monster_flag;
-
-			if (mult < s_ptr->mult)
-				mult = s_ptr->mult;
-
-			/* Set the hit verb appropriately */
-			if (is_ranged)
-				*hit_verb = s_ptr->range_verb;
-			else
-				*hit_verb = s_ptr->melee_verb;
-
-			/* Print a cool message for branded rings et al */
-			if (s_ptr->active_verb && secondary)
+			if (m_ptr->ml && s_ptr->monster_flag)
 			{
-				char o_name[40];
-				object_desc(o_name, sizeof(o_name), o_ptr,
-						FALSE, ODESC_BASE);
-				msg_format("Your %s %s!", o_name,
-						s_ptr->active_verb);
+				rf_on(l_ptr->flags, s_ptr->monster_flag);
+			}
+
+			if (m_ptr->ml && s_ptr->resist_flag)
+			{
+				rf_on(l_ptr->flags, s_ptr->resist_flag);
 			}
 		}
 
-		/* If the monster resisted, add to the monster lore */
-		if (r_ptr->flags[2] & s_ptr->resist_flag)
+		/* If the monster doesn't match or the slay flag does */
+		if ((s_ptr->brand && !rf_has(r_ptr->flags, s_ptr->resist_flag)) || 
+		    rf_has(r_ptr->flags, s_ptr->monster_flag))
 		{
-			if (m_ptr->ml)
-				l_ptr->flags[2] |= s_ptr->resist_flag;
+			/* notice any brand or slay that would affect the monster */
+			object_notice_slay(o_ptr, s_ptr->slay_flag);
+
+			/* compare multipliers to determine best attack, would be more complex for O-style brands */
+			if ((*best_s_ptr == NULL) || ((*best_s_ptr)->mult < s_ptr->mult))
+				*best_s_ptr = s_ptr;
 		}
 	}
-	
-	return mult;
 }
 
 
@@ -266,14 +247,19 @@ static int get_brand_mult(object_type *o_ptr, const monster_type *m_ptr,
  * Attack the monster at the given location
  *
  * If no "weapon" is available, then "punch" the monster one time.
+ *
+ * We get blows until energy drops below that required for another blow, or
+ * until the target monster dies. We use a wrapper to work out the number of
+ * blows. We don't allow @ to spend more than 100 energy in one go, to avoid
+ * slower monsters getting double moves.
  */
-void py_attack(int y, int x)
+bool py_attack_real(int y, int x)
 {
-	int num = 0, bonus, chance;
+	int bonus, chance;
 
-	monster_type *m_ptr;
-	monster_race *r_ptr;
-	monster_lore *l_ptr;
+	monster_type *m_ptr = &mon_list[cave_m_idx[y][x]];
+	monster_race *r_ptr = &r_info[m_ptr->r_idx];
+	monster_lore *l_ptr = &l_list[m_ptr->r_idx];
 
 	object_type *o_ptr;
 
@@ -282,177 +268,194 @@ void py_attack(int y, int x)
 	bool fear = FALSE;
 
 	bool do_quake = FALSE;
+	bool dead = FALSE;
 
+	u32b msg_type = 0;
+	bool success = FALSE;
 
-	/* Get the monster */
-	m_ptr = &mon_list[cave_m_idx[y][x]];
-	r_ptr = &r_info[m_ptr->r_idx];
-	l_ptr = &l_list[m_ptr->r_idx];
-
-
-	/* Disturb the player */
-	disturb(0, 0);
-
+	/* Default to punching for one damage */
+	const char *hit_verb = "punch";
+	int dmg = 1;
+	msg_type = MSG_HIT;
 
 	/* Extract monster name (or "it") */
 	monster_desc(m_name, sizeof(m_name), m_ptr, 0);
 
-
 	/* Auto-Recall if possible and visible */
 	if (m_ptr->ml) monster_race_track(m_ptr->r_idx);
 
 	/* Track a new monster */
 	if (m_ptr->ml) health_track(cave_m_idx[y][x]);
 
-
 	/* Handle player fear (only for invisible monsters) */
 	if (p_ptr->state.afraid)
 	{
 		message_format(MSG_AFRAID, 0,
 				"You are too afraid to attack %s!",
 				m_name);
-		return;
+		return (FALSE);
 	}
 
-
 	/* Disturb the monster */
 	wake_monster(m_ptr);
 
-
 	/* Get the weapon */
-	o_ptr = &inventory[INVEN_WIELD];
+	o_ptr = &p_ptr->inventory[INVEN_WIELD];
 
 	/* Calculate the "attack quality" */
 	bonus = p_ptr->state.to_h + o_ptr->to_h;
 	chance = (p_ptr->state.skills[SKILL_TO_HIT_MELEE] + (bonus * BTH_PLUS_ADJ));
 
+	/* See if the player hit */
+	success = test_hit(chance, r_ptr->ac, m_ptr->ml);
 
-	/* Attack once for each legal blow */
-	while (num++ < p_ptr->state.num_blow)
+	/* If a miss, skip this hit */
+	if (!success)
 	{
-		/* Test for hit */
-		if (test_hit(chance, r_ptr->ac, m_ptr->ml))
-		{
-			/* Default to punching for one damage */
-			const char *hit_verb = "punch";
-			int k = 1;
+		message_format(MSG_MISS, m_ptr->r_idx, "You miss %s.", m_name);
+		return (FALSE);
+	}
 
-			/* Handle normal weapon */
-			if (o_ptr->k_idx)
-			{
-				int weapon_brand_mult, ring_brand_mult[2];
-				int use_mult = 1;
+	/* Handle normal weapon */
+	if (o_ptr->k_idx)
+	{
+		int i;
+		const slay_t *best_s_ptr = NULL;
 
-				hit_verb = "hit";
+		hit_verb = "hit";
 
-				/* Hack-- put rings first, because they can
-				 * only be brands right now */
-				ring_brand_mult[0] = get_brand_mult(
-						&inventory[INVEN_LEFT],
-						m_ptr, &hit_verb, FALSE, TRUE);
-				ring_brand_mult[1] = get_brand_mult(
-						&inventory[INVEN_RIGHT],
-						m_ptr, &hit_verb, FALSE, TRUE);
-				weapon_brand_mult = get_brand_mult(
-						o_ptr,
-						m_ptr, &hit_verb, FALSE, FALSE);
+		/* Get the best attack from all slays or
+		 * brands on all non-launcher equipment */
+		for (i = INVEN_LEFT; i < INVEN_TOTAL; i++)
+			improve_attack_modifier(&p_ptr->inventory[i], m_ptr,
+			&best_s_ptr);
 
-				/* Message. Need to do this after tot_dam_aux, which sets hit_verb, but before critical_norm, which may print further messages. */
-				message_format(MSG_GENERIC, m_ptr->r_idx, "You %s %s.", hit_verb, m_name);
-						
-				if (ring_brand_mult[0] > use_mult)
-					use_mult = ring_brand_mult[0];
-				if (ring_brand_mult[1] > use_mult)
-					use_mult = ring_brand_mult[1];
-				if (weapon_brand_mult > use_mult)
-					use_mult = weapon_brand_mult;
+		improve_attack_modifier(o_ptr, m_ptr, &best_s_ptr);
+		if (best_s_ptr != NULL)
+			hit_verb = best_s_ptr->melee_verb;
 
-				k = damroll(o_ptr->dd, o_ptr->ds);
-				k *= use_mult;
+		dmg = damroll(o_ptr->dd, o_ptr->ds);
+		dmg *= (best_s_ptr == NULL) ? 1 : best_s_ptr->mult;
 
-				if (p_ptr->state.impact && (k > 50))
-					do_quake = TRUE;
+		if (p_ptr->state.impact && (dmg > 50))
+			do_quake = TRUE;
 
-				k += o_ptr->to_d;
-				k = critical_norm(o_ptr->weight, o_ptr->to_h, k);
+		dmg += o_ptr->to_d;
+		dmg = critical_norm(o_ptr->weight, o_ptr->to_h, dmg, &msg_type);
 
-				/* Learn by use */
-				object_notice_attack_plusses(o_ptr);
-				wieldeds_notice_on_attack();
+		/* Learn by use for the weapon */
+		object_notice_attack_plusses(o_ptr);
 
-				if (do_quake)
-					wieldeds_notice_flag(2, TR2_IMPACT);
-			}
-			else
-			{
-				message_format(MSG_GENERIC, m_ptr->r_idx, "You %s %s.", hit_verb, m_name);
-			}
+		if (do_quake)
+			wieldeds_notice_flag(OF_IMPACT);
+	}
 
-			/* Apply the player damage bonuses */
-			k += p_ptr->state.to_d;
+	/* Learn by use for other equipped items */
+	wieldeds_notice_on_attack();
 
-			/* No negative damage */
-			if (k < 0) k = 0;
+	/* Apply the player damage bonuses */
+	dmg += p_ptr->state.to_d;
 
-			/* Complex message */
-			if (p_ptr->wizard)
-				msg_format("You do %d (out of %d) damage.", k, m_ptr->hp);
+	/* No negative damage */
+	if (dmg <= 0) dmg = 0;
 
-			/* Damage, check for fear and death */
-			if (mon_take_hit(cave_m_idx[y][x], k, &fear, NULL)) break;
+	/* Tell the player what happened */
+	if (dmg <= 0)
+		message_format(MSG_MISS, m_ptr->r_idx,
+					   "You fail to harm %s.", m_name);
+	else if (msg_type == MSG_HIT)
+		message_format(MSG_HIT, m_ptr->r_idx, "You %s %s.",
+					   hit_verb, m_name);
+	else if (msg_type == MSG_HIT_GOOD)
+		message_format(MSG_HIT_GOOD, m_ptr->r_idx, "You %s %s. %s",
+					   hit_verb, m_name, "It was a good hit!");
+	else if (msg_type == MSG_HIT_GREAT)
+		message_format(MSG_HIT_GREAT, m_ptr->r_idx, "You %s %s. %s",
+					   hit_verb, m_name, "It was a great hit!");
+	else if (msg_type == MSG_HIT_SUPERB)
+		message_format(MSG_HIT_SUPERB, m_ptr->r_idx, "You %s %s. %s",
+					   hit_verb, m_name, "It was a superb hit!");
+	else if (msg_type == MSG_HIT_HI_GREAT)
+		message_format(MSG_HIT_HI_GREAT, m_ptr->r_idx, "You %s %s. %s",
+					   hit_verb, m_name, "It was a *GREAT* hit!");
+	else if (msg_type == MSG_HIT_HI_SUPERB)
+		message_format(MSG_HIT_HI_SUPERB, m_ptr->r_idx, "You %s %s. %s",
+					   hit_verb, m_name, "It was a *SUPERB* hit!");
 
-			/* Confusion attack */
-			if (p_ptr->confusing)
-			{
-				/* Cancel glowing hands */
-				p_ptr->confusing = FALSE;
+	/* Complex message */
+	if (p_ptr->wizard)
+		msg_format("You do %d (out of %d) damage.", dmg, m_ptr->hp);
 
-				/* Message */
-				msg_print("Your hands stop glowing.");
+	/* Confusion attack */
+	if (p_ptr->confusing)
+	{
+		/* Cancel glowing hands */
+		p_ptr->confusing = FALSE;
 
-				/* Confuse the monster */
-				if (r_ptr->flags[2] & (RF2_NO_CONF))
-				{
-					if (m_ptr->ml)
-					{
-						l_ptr->flags[2] |= (RF2_NO_CONF);
-					}
+		/* Message */
+		msg_print("Your hands stop glowing.");
 
-					msg_format("%^s is unaffected.", m_name);
-				}
-				else if (randint0(100) < r_ptr->level)
-				{
-					msg_format("%^s is unaffected.", m_name);
-				}
-				else
-				{
-					msg_format("%^s appears confused.", m_name);
-					m_ptr->confused += 10 + randint0(p_ptr->lev) / 5;
-				}
-			}
-		}
+		/* Update the lore */
+		if (m_ptr->ml)
+			rf_on(l_ptr->flags, RF_NO_CONF);
 
-		/* Player misses */
+		/* Confuse the monster */
+		if (rf_has(r_ptr->flags, RF_NO_CONF))
+			msg_format("%^s is unaffected.", m_name);
+		else if (randint0(100) < r_ptr->level)
+			msg_format("%^s is unaffected.", m_name);
 		else
 		{
-			/* Message */
-			message_format(MSG_MISS, m_ptr->r_idx, "You miss %s.", m_name);
+			msg_format("%^s appears confused.", m_name);
+			m_ptr->confused += 10 + randint0(p_ptr->lev) / 5;
 		}
 	}
 
+	/* Damage, check for fear and death */
+	dead = mon_take_hit(cave_m_idx[y][x], dmg, &fear, NULL);
 
 	/* Hack -- delay fear messages */
 	if (fear && m_ptr->ml)
-	{
-		/* Message */
-		message_format(MSG_FLEE, m_ptr->r_idx, "%^s flees in terror!", m_name);
-	}
-
+		message_format(MSG_FLEE, m_ptr->r_idx, "%^s flees in terror!",
+		m_name);
 
 	/* Mega-Hack -- apply earthquake brand */
 	if (do_quake) earthquake(p_ptr->py, p_ptr->px, 10);
+
+	return (dead);
 }
 
+void py_attack(int y, int x)
+{
+	int blow_energy;
+	bool stop = FALSE;
+	int blows = 0;
+
+	/* disturb the player */
+	disturb(0,0);
+
+	/* calculate energy per blow */
+	blow_energy = 10000 / p_ptr->state.num_blow;
+
+	/*
+	 * set energy use to zero, overriding whatever was set before we
+	 * got here
+	 */
+	p_ptr->energy_use = 0;
+
+	/*
+	 * take blows until energy runs out or monster dies -
+	 * note that we limit energy use to 100, even if an extra blow would
+	 * be possible, to avoid monster double moves
+	 */
+	while ((p_ptr->energy >= (blow_energy * (blows + 1))) && !stop)
+	{
+		stop = py_attack_real(y, x);
+		p_ptr->energy_use += blow_energy;
+		if ((p_ptr->energy_use + blow_energy) > 100) stop = TRUE;
+		blows++;
+	}
+}
 
 
 /*
@@ -499,13 +502,15 @@ void do_cmd_fire(cmd_code code, cmd_arg args[])
 
 	char o_name[80];
 
+	u32b msg_type = 0;
+
 	int path_n;
 	u16b path_g[256];
 
 	int msec = op_ptr->delay_factor * op_ptr->delay_factor;
 
 	/* Get the "bow" */
-	j_ptr = &inventory[INVEN_BOW];
+	j_ptr = &p_ptr->inventory[INVEN_BOW];
 
 	/* Require a usable launcher */
 	if (!j_ptr->tval || !p_ptr->state.ammo_tval)
@@ -519,7 +524,7 @@ void do_cmd_fire(cmd_code code, cmd_arg args[])
 	dir = args[1].direction;
 
 	/* Check the item being fired is usable by the player. */
-	if (!item_is_available(item, NULL, (USE_INVEN | USE_FLOOR)))
+	if (!item_is_available(item, NULL, (USE_EQUIP | USE_INVEN | USE_FLOOR)))
 	{
 		msg_format("That item is not within your reach.");
 		return;
@@ -560,9 +565,10 @@ void do_cmd_fire(cmd_code code, cmd_arg args[])
 	/* Sound */
 	sound(MSG_SHOOT);
 
+	object_notice_on_firing(o_ptr);
+
 	/* Describe the object */
-	object_desc(o_name, sizeof(o_name), o_ptr, FALSE,
-			ODESC_FULL | ODESC_SINGULAR);
+	object_desc(o_name, sizeof(o_name), o_ptr, ODESC_FULL | ODESC_SINGULAR);
 
 	/* Find the color and symbol for the object for throwing */
 	missile_attr = object_attr(o_ptr);
@@ -609,7 +615,7 @@ void do_cmd_fire(cmd_code code, cmd_arg args[])
 			if (p_ptr->redraw) redraw_stuff();
 
 			Term_xtra(TERM_XTRA_DELAY, msec);
-			lite_spot(y, x);
+			light_spot(y, x);
 
 			Term_fresh();
 			if (p_ptr->redraw) redraw_stuff();
@@ -630,13 +636,10 @@ void do_cmd_fire(cmd_code code, cmd_arg args[])
 
 			int chance2 = chance - distance(p_ptr->py, p_ptr->px, y, x);
 			int visible = m_ptr->ml;
+			int multiplier = 1;
 
 			const char *hit_verb = "hits";
-
-			int ammo_mult = get_brand_mult(o_ptr, m_ptr,
-					&hit_verb, TRUE, FALSE);
-			int shoot_mult = get_brand_mult(j_ptr, m_ptr,
-					&hit_verb, TRUE, FALSE);
+			const slay_t *best_s_ptr = NULL;
 
 			/* Note the collision */
 			hit_body = TRUE;
@@ -649,16 +652,37 @@ void do_cmd_fire(cmd_code code, cmd_arg args[])
 				/* Assume a default death */
 				cptr note_dies = " dies.";
 
+				improve_attack_modifier(o_ptr, m_ptr, &best_s_ptr);
+				improve_attack_modifier(j_ptr, m_ptr, &best_s_ptr);
+				if (best_s_ptr != NULL)
+					hit_verb = best_s_ptr->range_verb;
+
 				/* Some monsters get "destroyed" */
-				if ((r_ptr->flags[2] & RF2_DEMON) ||
-						(r_ptr->flags[2] & RF2_UNDEAD) ||
-						(r_ptr->flags[1] & RF1_STUPID) ||
-						strchr("Evg", r_ptr->d_char))
+				if (monster_is_unusual(r_ptr))
 				{
 					/* Special note at death */
 					note_dies = " is destroyed.";
 				}
 
+				/* Calculate multiplier */
+				multiplier = p_ptr->state.ammo_mult;
+				if (best_s_ptr != NULL) multiplier += best_s_ptr->mult;
+
+				/* Apply damage: multiplier, slays, criticals, bonuses */
+				tdam = damroll(o_ptr->dd, o_ptr->ds);
+				tdam += o_ptr->to_d + j_ptr->to_d;
+				tdam *= multiplier;
+				tdam = critical_shot(o_ptr->weight, o_ptr->to_h, tdam, &msg_type);
+
+				object_notice_attack_plusses(o_ptr);
+				object_notice_attack_plusses(&p_ptr->inventory[INVEN_BOW]);
+
+				/* No negative damage; change verb if no damage done */
+				if (tdam <= 0)
+				{
+					tdam = 0;
+					hit_verb = "fail to harm";
+				}
 
 				/* Handle unseen monster */
 				if (!visible)
@@ -675,29 +699,35 @@ void do_cmd_fire(cmd_code code, cmd_arg args[])
 					/* Get "the monster" or "it" */
 					monster_desc(m_name, sizeof(m_name), m_ptr, 0);
 
-					/* Message */
-					message_format(MSG_SHOOT_HIT, 0, "The %s %s %s.", o_name, hit_verb, m_name);
-
+					/* Tell the player what happened */
+					if (msg_type == MSG_SHOOT_HIT)
+						message_format(MSG_SHOOT_HIT, 0, "The %s %s %s.", o_name, hit_verb, m_name);
+					else
+					{
+						if (msg_type == MSG_HIT_GOOD)
+						{
+							message_format(MSG_HIT_GOOD, 0, "The %s %s %s. %s", o_name, hit_verb, m_name,
+										   "It was a good hit!");
+						}
+						else if (msg_type == MSG_HIT_GREAT)
+						{
+							message_format(MSG_HIT_GREAT, 0, "The %s %s %s. %s", o_name, hit_verb, m_name,
+										   "It was a great hit!");
+						}
+						else if (msg_type == MSG_HIT_SUPERB)
+						{
+							message_format(MSG_HIT_SUPERB, 0, "The %s %s %s. %s", o_name, hit_verb, m_name,
+										   "It was a superb hit!");
+						}
+					}
 					/* Hack -- Track this monster race */
 					if (m_ptr->ml) monster_race_track(m_ptr->r_idx);
 
 					/* Hack -- Track this monster */
 					if (m_ptr->ml) health_track(cave_m_idx[y][x]);
+
 				}
 
-				/* Apply damage: multiplier, slays, criticals, bonuses */
-				tdam = damroll(o_ptr->dd, o_ptr->ds);
-				tdam += o_ptr->to_d + j_ptr->to_d;
-				tdam *= p_ptr->state.ammo_mult;
-				tdam *= MAX(ammo_mult, shoot_mult);
-				tdam = critical_shot(o_ptr->weight, o_ptr->to_h, tdam);
-
-				object_notice_attack_plusses(o_ptr);
-				object_notice_attack_plusses(&inventory[INVEN_BOW]);
-
-				/* No negative damage */
-				if (tdam < 0) tdam = 0;
-
 				/* Complex message */
 				if (p_ptr->wizard)
 				{
@@ -748,8 +778,6 @@ void do_cmd_fire(cmd_code code, cmd_arg args[])
 	/* Single object */
 	i_ptr->number = 1;
 
-
-	/* Reduce and describe inventory */
 	if (item >= 0)
 	{
 		inven_item_increase(item, -1);
@@ -769,42 +797,51 @@ void do_cmd_fire(cmd_code code, cmd_arg args[])
 	j = (hit_body ? breakage_chance(i_ptr) : 0);
 
 	/* Drop (or break) near that location */
-	drop_near(i_ptr, j, y, x);
+	drop_near(i_ptr, j, y, x, TRUE);
 }
 
-void textui_cmd_fire(void)
+void textui_cmd_fire_at_nearest(void)
 {
-	object_type *j_ptr, *o_ptr;
-	int item;
-	int dir;
-	cptr q, s;
-
-
-	/* Get the "bow" (if any) */
-	j_ptr = &inventory[INVEN_BOW];
+	/* the direction '5' means 'use the target' */
+	int i, dir = 5, item = -1;
 
 	/* Require a usable launcher */
-	if (!j_ptr->tval || !p_ptr->state.ammo_tval)
+	if (!p_ptr->inventory[INVEN_BOW].tval || !p_ptr->state.ammo_tval)
 	{
 		msg_print("You have nothing to fire with.");
 		return;
 	}
 
-	/* Require proper missile */
-	item_tester_tval = p_ptr->state.ammo_tval;
+	/* Find first eligible ammo in the quiver */
+	for (i = QUIVER_START; i < QUIVER_END; i++)
+	{
+		if (p_ptr->inventory[i].tval != p_ptr->state.ammo_tval) continue;
+		item = i;
+		break;
+	}
 
-	/* Get an item */
-	q = "Fire which item? ";
-	s = "You have nothing to fire.";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
+	/* Require usable ammo */
+	if (item < 0)
+	{
+		msg_print("You have no ammunition in the quiver to fire");
+		return;
+	}
 
-	/* Get the object */
-	o_ptr = object_from_item_idx(item);
+	/* Require foe */
+	if (!target_set_closest(TARGET_KILL | TARGET_QUIET))
+		return;
 
-	/* Get a direction (or cancel) */
-	if (!get_aim_dir(&dir)) return;
+	/* Check for confusion */
+	if (p_ptr->timed[TMD_CONFUSED])
+	{
+		msg_print("You are confused.");
+		dir = ddd[randint0(8)];
+	}
 
-	cmd_insert(CMD_FIRE, item, dir);
+	/* Fire! */
+	cmd_insert(CMD_FIRE);
+	cmd_set_arg_item(cmd_get_top(), 0, item);
+	cmd_set_arg_target(cmd_get_top(), 1, dir);
 }
 
 /*
@@ -836,6 +873,8 @@ void do_cmd_throw(cmd_code code, cmd_arg args[])
 
 	char o_name[80];
 
+	u32b msg_type = 0;
+
 	int path_n;
 	u16b path_g[256];
 
@@ -845,8 +884,15 @@ void do_cmd_throw(cmd_code code, cmd_arg args[])
 	item = args[0].item;
 	dir = args[1].direction;
 
+	/* Make sure the player isn't throwing wielded items */
+	if (item >= INVEN_WIELD && item < QUIVER_START)
+	{
+		msg_print("You have cannot throw wielded items.");
+		return;
+	}
+
 	/* Check the item being thrown is usable by the player. */
-	if (!item_is_available(item, NULL, (USE_INVEN | USE_FLOOR)))
+	if (!item_is_available(item, NULL, (USE_EQUIP | USE_INVEN | USE_FLOOR)))
 	{
 		msg_format("That item is not within your reach.");
 		return;
@@ -854,6 +900,7 @@ void do_cmd_throw(cmd_code code, cmd_arg args[])
 
 	/* Get the object */
 	o_ptr = object_from_item_idx(item);
+	object_notice_on_firing(o_ptr);
 
 	/* Get local object */
 	i_ptr = &object_type_body;
@@ -884,7 +931,7 @@ void do_cmd_throw(cmd_code code, cmd_arg args[])
 
 
 	/* Description */
-	object_desc(o_name, sizeof(o_name), i_ptr, FALSE, ODESC_FULL);
+	object_desc(o_name, sizeof(o_name), i_ptr, ODESC_FULL);
 
 	/* Find the color and symbol for the object for throwing */
 	missile_attr = object_attr(i_ptr);
@@ -958,7 +1005,7 @@ void do_cmd_throw(cmd_code code, cmd_arg args[])
 			if (p_ptr->redraw) redraw_stuff();
 
 			Term_xtra(TERM_XTRA_DELAY, msec);
-			lite_spot(y, x);
+			light_spot(y, x);
 
 			Term_fresh();
 			if (p_ptr->redraw) redraw_stuff();
@@ -989,23 +1036,34 @@ void do_cmd_throw(cmd_code code, cmd_arg args[])
 			{
 				const char *hit_verb = "hits";
 				bool fear = FALSE;
+				const slay_t *best_s_ptr = NULL;
 
 				/* Assume a default death */
 				cptr note_dies = " dies.";
 
 				/* Some monsters get "destroyed" */
-				if ((r_ptr->flags[2] & (RF2_DEMON)) ||
-				    (r_ptr->flags[2] & (RF2_UNDEAD)) ||
-				    (r_ptr->flags[1] & (RF1_STUPID)) ||
-				    (strchr("Evg", r_ptr->d_char)))
+				if (monster_is_unusual(r_ptr))
 				{
 					/* Special note at death */
 					note_dies = " is destroyed.";
 				}
 
 				/* Apply special damage  - brought forward to fill in hit_verb XXX XXX XXX */
-				tdam *= get_brand_mult(i_ptr, m_ptr,
-						&hit_verb, TRUE, FALSE);
+				improve_attack_modifier(i_ptr, m_ptr, &best_s_ptr);
+				if (best_s_ptr != NULL)
+				{
+					tdam *= best_s_ptr->mult;
+					hit_verb = best_s_ptr->range_verb;
+				}
+				/* Apply special damage XXX XXX XXX */
+				tdam = critical_shot(i_ptr->weight, i_ptr->to_h, tdam, &msg_type);
+
+				/* No negative damage; change verb if no damage done */
+				if (tdam <= 0)
+				{
+					tdam = 0;
+					hit_verb = "fail to harm";
+				}
 
 				/* Handle unseen monster */
 				if (!visible)
@@ -1022,9 +1080,27 @@ void do_cmd_throw(cmd_code code, cmd_arg args[])
 					/* Get "the monster" or "it" */
 					monster_desc(m_name, sizeof(m_name), m_ptr, 0);
 
-					/* Message */
-					msg_format("The %s %s %s.", o_name, hit_verb, m_name);
-
+					/* Tell the player what happened */
+					if (msg_type == MSG_SHOOT_HIT)
+						message_format(MSG_SHOOT_HIT, 0, "The %s %s %s.", o_name, hit_verb, m_name);
+					else
+					{
+						if (msg_type == MSG_HIT_GOOD)
+						{
+							message_format(MSG_HIT_GOOD, 0, "The %s %s %s. %s", o_name, hit_verb, m_name,
+										   "It was a good hit!");
+						}
+						else if (msg_type == MSG_HIT_GREAT)
+						{
+							message_format(MSG_HIT_GREAT, 0, "The %s %s %s. %s", o_name, hit_verb, m_name,
+										   "It was a great hit!");
+						}
+						else if (msg_type == MSG_HIT_SUPERB)
+						{
+							message_format(MSG_HIT_SUPERB, 0, "The %s %s %s. %s", o_name, hit_verb, m_name,
+										   "It was a superb hit!");
+						}
+					}
 					/* Hack -- Track this monster race */
 					if (m_ptr->ml) monster_race_track(m_ptr->r_idx);
 
@@ -1032,25 +1108,18 @@ void do_cmd_throw(cmd_code code, cmd_arg args[])
 					if (m_ptr->ml) health_track(cave_m_idx[y][x]);
 				}
 
-				/* Apply special damage XXX XXX XXX */
-				tdam = critical_shot(i_ptr->weight, i_ptr->to_h, tdam);
-
-				/* No negative damage */
-				if (tdam < 0) tdam = 0;
-
 				/* Learn the bonuses */
-				/* XXX Eddie This is messed up, better done for firing, should use that method [split last] instead */
+				/* XXX Eddie This is messed up, better done for firing, */
+				/* should use that method [split last] instead */
 				/* check if inven_optimize removed what o_ptr referenced */
-				if (object_similar(i_ptr, o_ptr))
+				if (object_similar(i_ptr, o_ptr, OSTACK_PACK))
 					object_notice_attack_plusses(o_ptr);
 				object_notice_attack_plusses(i_ptr);
 
 				/* Complex message */
 				if (p_ptr->wizard)
-				{
 					msg_format("You do %d (out of %d) damage.",
-					           tdam, m_ptr->hp);
-				}
+							   tdam, m_ptr->hp);
 
 				/* Hit the monster, check for death */
 				if (mon_take_hit(cave_m_idx[y][x], tdam, &fear, note_dies))
@@ -1088,7 +1157,7 @@ void do_cmd_throw(cmd_code code, cmd_arg args[])
 	j = (hit_body ? breakage_chance(i_ptr) : 0);
 
 	/* Drop (or break) near that location */
-	drop_near(i_ptr, j, y, x);
+	drop_near(i_ptr, j, y, x, TRUE);
 }
 
 void textui_cmd_throw(void)
@@ -1099,10 +1168,18 @@ void textui_cmd_throw(void)
 	/* Get an item */
 	q = "Throw which item? ";
 	s = "You have nothing to throw.";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
+	if (!get_item(&item, q, s, CMD_THROW, (USE_EQUIP | USE_INVEN | USE_FLOOR))) return;
+
+	if (item >= INVEN_WIELD && item < QUIVER_START)
+	{
+		msg_print("You cannot throw wielded items.");
+		return;
+	}
 
 	/* Get a direction (or cancel) */
 	if (!get_aim_dir(&dir)) return;
 
-	cmd_insert(CMD_THROW, item, dir);
+	cmd_insert(CMD_THROW);
+	cmd_set_arg_item(cmd_get_top(), 0, item);
+	cmd_set_arg_target(cmd_get_top(), 1, dir);
 }
