@@ -3,7 +3,7 @@
  * Purpose: Handle objects in various ways
  *
  * Copyright (c) 1997 Ben Harrison, James E. Wilson, Robert A. Koeneke
- * Copyright (c) 2007 Andrew Sidwell
+ * Copyright (c) 2007-9 Andrew Sidwell, Chris Carr, Ed Graham, Erik Osheim
  *
  * This work is free software; you can redistribute it and/or modify it
  * under the terms of either:
@@ -16,20 +16,25 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
+
 #include "angband.h"
-#include "object/tvalsval.h"
-#include "object/object.h"
-#include "game-cmd.h"
+#include "cave.h"
 #include "cmds.h"
 #include "effects.h"
+#include "game-cmd.h"
+#include "object/object.h"
+#include "object/tvalsval.h"
+#include "spells.h"
+#include "target.h"
 
 /*** Utility bits and bobs ***/
+
 /*
  * Check to see if the player can use a rod/wand/staff/activatable object.
  */
 static int check_devices(object_type *o_ptr)
 {
-	int lev, chance;
+	int fail;
 	const char *msg;
 	const char *what = NULL;
 
@@ -42,30 +47,11 @@ static int check_devices(object_type *o_ptr)
 		default:       msg = "activate it";  break;
 	}
 
-	/* Extract the item level */
-	if (artifact_p(o_ptr))
-		lev = a_info[o_ptr->name1].level;
-	else
-		lev = k_info[o_ptr->k_idx].level;
-
-	/* Base chance of success */
-	chance = p_ptr->state.skills[SKILL_DEVICE];
-
-	/* Confusion hurts skill */
-	if (p_ptr->timed[TMD_CONFUSED] || p_ptr->timed[TMD_AMNESIA])
-		chance = chance / 2;
-
-	/* High level objects are harder */
-	chance -= MIN(lev, 50);
-
-	/* Give everyone a (slight) chance */
-	if ((chance < USE_DEVICE) && one_in_(USE_DEVICE - chance + 1))
-	{
-		chance = USE_DEVICE;
-	}
+	/* Figure out how hard the item is to use */
+	fail = get_use_device_chance(o_ptr);
 
 	/* Roll for usage */
-	if ((chance < USE_DEVICE) || (randint1(chance) < USE_DEVICE))
+	if (randint1(1000) < fail)
 	{
 		if (OPT(flush_failure)) flush();
 		msg_format("You failed to %s properly.", msg);
@@ -84,6 +70,7 @@ static int check_devices(object_type *o_ptr)
 	return TRUE;
 }
 
+
 /*
  * Return the chance of an effect beaming, given a tval.
  */
@@ -99,12 +86,12 @@ static int beam_chance(int tval)
 }
 
 
-typedef enum { 
-	ART_TAG_NONE, 
-	ART_TAG_NAME, 
-	ART_TAG_KIND, 
-	ART_TAG_VERB, 
-	ART_TAG_VERB_IS 
+typedef enum {
+	ART_TAG_NONE,
+	ART_TAG_NAME,
+	ART_TAG_KIND,
+	ART_TAG_VERB,
+	ART_TAG_VERB_IS
 } art_tag_t;
 
 static art_tag_t art_tag_lookup(const char *tag)
@@ -123,13 +110,13 @@ static art_tag_t art_tag_lookup(const char *tag)
 
 /*
  * Print an artifact activation message.
- * 
+ *
  * In order to support randarts, with scrambled names, we re-write
  * the message to replace instances of {name} with the artifact name
  * and instances of {kind} with the type of object.
  *
- * This code deals with plural and singular forms of verbs correctly 
- * when encountering {s}, though in fact both names and kinds are 
+ * This code deals with plural and singular forms of verbs correctly
+ * when encountering {s}, though in fact both names and kinds are
  * always singular in the current code (gloves are "Set of" and boots
  * are "Pair of")
  */
@@ -141,9 +128,9 @@ static void activation_message(object_type *o_ptr, const char *message)
 	const char *tag;
 	const char *in_cursor;
 	size_t end = 0;
- 
+
 	in_cursor = message;
- 
+
 	next = strchr(in_cursor, '{');
 	while (next)
 	{
@@ -157,11 +144,11 @@ static void activation_message(object_type *o_ptr, const char *message)
 		{
 			tag = next + 1; /* Start the tag after the { */
 			in_cursor = s + 1;
- 
+
 			switch(art_tag_lookup(tag))
 			{
 			case ART_TAG_NAME:
-				end += object_desc(buf, 1024, o_ptr, TRUE, ODESC_BASE); 
+				end += object_desc(buf, 1024, o_ptr, ODESC_PREFIX | ODESC_BASE); 
 				break;
 			case ART_TAG_KIND:
 				object_kind_name(&buf[end], 1024-end, o_ptr->k_idx, TRUE);
@@ -182,14 +169,14 @@ static void activation_message(object_type *o_ptr, const char *message)
 		else    /* An invalid tag, skip it */
 		{
 			in_cursor = next + 1;
-		} 
+		}
 
 		next = strchr(in_cursor, '{');
 	}
 	strnfcat(buf, 1024, &end, in_cursor);
- 
+
 	msg_print(buf);
-} 
+}
 
 
 
@@ -205,7 +192,7 @@ void do_cmd_uninscribe(cmd_code code, cmd_arg args[])
 
 	o_ptr->note = 0;
 
-	p_ptr->notice |= (PN_COMBINE | PN_SQUELCH);
+	p_ptr->notice |= (PN_COMBINE | PN_SQUELCH | PN_SORT_QUIVER);
 	p_ptr->redraw |= (PR_INVEN | PR_EQUIP);
 }
 
@@ -213,19 +200,19 @@ void do_cmd_uninscribe(cmd_code code, cmd_arg args[])
 void do_cmd_inscribe(cmd_code code, cmd_arg args[])
 {
 	object_type *o_ptr = object_from_item_idx(args[0].item);
-	
+
 	o_ptr->note = quark_add(args[1].string);
 
-	p_ptr->notice |= (PN_COMBINE | PN_SQUELCH);
+	p_ptr->notice |= (PN_COMBINE | PN_SQUELCH | PN_SORT_QUIVER);
 	p_ptr->redraw |= (PR_INVEN | PR_EQUIP);
 }
 
-static void obj_inscribe(object_type *o_ptr, int item)
+void textui_obj_inscribe(object_type *o_ptr, int item)
 {
 	char o_name[80];
 	char tmp[80] = "";
 
-	object_desc(o_name, sizeof(o_name), o_ptr, TRUE, ODESC_FULL);
+	object_desc(o_name, sizeof(o_name), o_ptr, ODESC_PREFIX | ODESC_FULL);
 	msg_format("Inscribing %s.", o_name);
 	message_flush();
 
@@ -236,27 +223,28 @@ static void obj_inscribe(object_type *o_ptr, int item)
 	/* Get a new inscription (possibly empty) */
 	if (get_string("Inscription: ", tmp, sizeof(tmp)))
 	{
-		cmd_insert(CMD_INSCRIBE, item, tmp);
+		cmd_insert(CMD_INSCRIBE);
+		cmd_set_arg_item(cmd_get_top(), 0, item);
+		cmd_set_arg_string(cmd_get_top(), 1, tmp);
 	}
 }
 
 
 /*** Examination ***/
-static void obj_examine(object_type *o_ptr, int item)
+void textui_obj_examine(object_type *o_ptr, int item)
 {
+	char header[120];
+
+	textblock *tb;
+	region area = { 0, 0, 0, 0 };
+
 	track_object(item);
 
-	text_out_hook = text_out_to_screen;
-	screen_save();
+	tb = object_info(o_ptr, OINFO_NONE);
+	object_desc(header, sizeof(header), o_ptr, ODESC_PREFIX | ODESC_FULL);
 
-	object_info_header(o_ptr);
-	if (!object_info(o_ptr, FALSE))
-		text_out("\n\nThis item does not seem to possess any special abilities.");
-
-	text_out_c(TERM_L_BLUE, "\n\n[Press any key to continue]\n");
-	(void)anykey();
-
-	screen_load();
+	textui_textblock_show(tb, area, format("%^s", header));
+	textblock_free(tb);
 }
 
 
@@ -281,6 +269,7 @@ void do_cmd_takeoff(cmd_code code, cmd_arg args[])
 	}
 
 	(void)inven_takeoff(item, 255);
+	pack_overflow();
 	p_ptr->energy_use = 50;
 }
 
@@ -300,7 +289,7 @@ void do_cmd_wield(cmd_code code, cmd_arg args[])
 	{
 		msg_print("You do not have that item to wield.");
 		return;
-	}	
+	}
 
 	/* Check the slot */
 	if (!slot_can_wield_item(slot, o_ptr))
@@ -309,32 +298,42 @@ void do_cmd_wield(cmd_code code, cmd_arg args[])
 		return;
 	}
 
-	equip_o_ptr = &inventory[slot];
+	equip_o_ptr = &p_ptr->inventory[slot];
 
-	/* Check for existing wielded item */
-	if (equip_o_ptr)
+	/* If the slot is open, wield and be done */
+	if (!equip_o_ptr->k_idx) 
 	{
-		/* Prevent wielding into a cursed slot */
-		if (cursed_p(equip_o_ptr))
-		{
-			/* Message */
-			object_desc(o_name, sizeof(o_name), equip_o_ptr, FALSE, ODESC_BASE);
-			msg_format("The %s you are %s appears to be cursed.",
-			           o_name, describe_use(slot));
+		wield_item(o_ptr, item, slot);
+		return;
+	}
 
-			return;
-		}
+	/* If the slot is in the quiver and objects can be combined */
+	if (obj_is_ammo(equip_o_ptr) && object_similar(equip_o_ptr, o_ptr,
+		OSTACK_QUIVER))
+	{
+		wield_item(o_ptr, item, slot);
+		return;
+	}
 
-		/* "!t" checks for taking off */
-		n = check_for_inscrip(equip_o_ptr, "!t");
-		while (n--)
-		{
-			/* Prompt */
-			object_desc(o_name, sizeof(o_name), equip_o_ptr, TRUE, ODESC_FULL);
+	/* Prevent wielding into a cursed slot */
+	if (cursed_p(equip_o_ptr))
+	{
+		object_desc(o_name, sizeof(o_name), equip_o_ptr, ODESC_BASE);
+		msg_format("The %s you are %s appears to be cursed.", o_name,
+				   describe_use(slot));
+		return;
+	}
 
-			/* Forget it */
-			if (!get_check(format("Really take off %s? ", o_name))) return;
-		}
+	/* "!t" checks for taking off */
+	n = check_for_inscrip(equip_o_ptr, "!t");
+	while (n--)
+	{
+		/* Prompt */
+		object_desc(o_name, sizeof(o_name), equip_o_ptr,
+					ODESC_PREFIX | ODESC_FULL);
+		
+		/* Forget it */
+		if (!get_check(format("Really take off %s? ", o_name))) return;
 	}
 
 	wield_item(o_ptr, item, slot);
@@ -364,86 +363,39 @@ void do_cmd_drop(cmd_code code, cmd_arg args[])
 	p_ptr->energy_use = 50;
 }
 
-static void obj_drop(object_type *o_ptr, int item)
-{
-	int amt;
-
-	amt = get_quantity(NULL, o_ptr->number);
-	if (amt <= 0) return;
-
-	cmd_insert(CMD_DROP, item, amt);
-}
-
-static void obj_wield(object_type *o_ptr, int item)
+void textui_obj_wield(object_type *o_ptr, int item)
 {
 	int slot = wield_slot(o_ptr);
 
-	if (o_ptr->tval == TV_RING &&
-		(inventory[INVEN_LEFT].k_idx && inventory[INVEN_RIGHT].k_idx))
+	/* Usually if the slot is taken we'll just replace the item in the slot,
+	 * but in some cases we need to ask the user which slot they actually
+	 * want to replace */
+	if (p_ptr->inventory[slot].k_idx)
 	{
-		cptr q = "Replace which ring? ";
-		cptr s = "Error in obj_wield, please report";
+		if (o_ptr->tval == TV_RING)
+		{
+			cptr q = "Replace which ring? ";
+			cptr s = "Error in obj_wield, please report";
+			item_tester_hook = obj_is_ring;
+			if (!get_item(&slot, q, s, CMD_WIELD, USE_EQUIP)) return;
+		}
 
-		item_tester_hook = obj_is_ring;
-		if (!get_item(&slot, q, s, USE_EQUIP)) return;
+		if (obj_is_ammo(o_ptr) && !object_similar(&p_ptr->inventory[slot],
+			o_ptr, OSTACK_QUIVER))
+		{
+			cptr q = "Replace which ammunition? ";
+			cptr s = "Error in obj_wield, please report";
+			item_tester_hook = obj_is_ammo;
+			if (!get_item(&slot, q, s, CMD_WIELD, USE_EQUIP)) return;
+		}
 	}
 
-	cmd_insert(CMD_WIELD, item, slot);
+	cmd_insert(CMD_WIELD);
+	cmd_set_arg_item(cmd_get_top(), 0, item);
+	cmd_set_arg_number(cmd_get_top(), 1, slot);
 }
 
 
-/*** Casting and browsing ***/
-
-/* Peruse spells in a book */
-static void obj_browse(object_type *o_ptr, int item)
-{
-	do_cmd_browse_aux(o_ptr, item);
-}
-
-/* Study a book to gain a new spell */
-static void obj_study(object_type *o_ptr, int item)
-{
-	/* Track the object kind */
-	track_object(item);
-
-	/* Mage -- Choose a spell to study */
-	if (cp_ptr->flags & CF_CHOOSE_SPELLS)
-	{
-		int spell = get_spell(o_ptr, "study", FALSE, FALSE);
-		if (spell >= 0)
-			cmd_insert(CMD_STUDY_SPELL, spell);
-	}
-	/* Priest -- Choose a book to study */
-	else
-	{
-		cmd_insert(CMD_STUDY_BOOK, item);
-	}
-}
-
-static void obj_cast(object_type *o_ptr, int item)
-{
-	int spell, dir = DIR_UNKNOWN;
-
-	cptr verb = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "cast" : "recite");
-	cptr noun = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
-
-	/* Track the object kind */
-	track_object(item);
-
-	/* Ask for a spell */
-	spell = get_spell(o_ptr, verb, TRUE, FALSE);
-	if (spell < 0)
-	{
-		if (spell == -2) msg_format("You don't know any %ss in that book.", noun);
-		return;
-	}
-
-	if (spell_needs_aim(cp_ptr->spell_book, spell) && !get_aim_dir(&dir))
-		return;
-
-	cmd_insert(CMD_CAST, spell, dir);
-}
-
 
 /*** Using items the traditional way ***/
 
@@ -468,11 +420,11 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	int item = args[0].item;
 	object_type *o_ptr = object_from_item_idx(item);
 	int effect;
-	bool ident = FALSE, used;
+	bool ident = FALSE, used = FALSE;
 	bool was_aware = object_flavor_is_aware(o_ptr);
 	int dir = 5;
 	int px = p_ptr->px, py = p_ptr->py;
-	int snd;
+	int snd, boost, level;
 	use_type use;
 	int items_allowed = 0;
 
@@ -481,7 +433,7 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	{
 		if (!obj_can_zap(o_ptr))
 		{
-			msg_print("The rod is not yet recharged.");
+			msg_print("That rod is still charging.");
 			return;
 		}
 
@@ -491,26 +443,38 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	}
 	else if (obj_is_wand(o_ptr))
 	{
+		if (!obj_has_charges(o_ptr))
+		{
+			msg_print("That wand has no charges.");
+			return;
+		}
+
 		use = USE_CHARGE;
 		snd = MSG_ZAP_ROD;
 		items_allowed = USE_INVEN | USE_FLOOR;
 	}
 	else if (obj_is_staff(o_ptr))
-	{	
+	{
+		if (!obj_has_charges(o_ptr))
+		{
+			msg_print("That staff has no charges.");
+			return;
+		}
+
 		use = USE_CHARGE;
-		snd = MSG_ZAP_ROD;
+		snd = MSG_USE_STAFF;
 		items_allowed = USE_INVEN | USE_FLOOR;
 	}
 	else if (obj_is_food(o_ptr))
 	{
 		use = USE_SINGLE;
-		snd = MSG_EAT;		
+		snd = MSG_EAT;
 		items_allowed = USE_INVEN | USE_FLOOR;
 	}
 	else if (obj_is_potion(o_ptr))
 	{
 		use = USE_SINGLE;
-		snd = MSG_QUAFF;		
+		snd = MSG_QUAFF;
 		items_allowed = USE_INVEN | USE_FLOOR;
 	}
 	else if (obj_is_scroll(o_ptr))
@@ -520,17 +484,17 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 			return;
 
 		use = USE_SINGLE;
-		snd = MSG_GENERIC;		
+		snd = MSG_GENERIC;
 		items_allowed = USE_INVEN | USE_FLOOR;
 	}
 	else if (obj_is_activatable(o_ptr))
 	{
 		if (!obj_can_activate(o_ptr))
 		{
-			msg_print("The item is not ready to activate");
+			msg_print("That item is still charging.");
 			return;
 		}
-		
+
 		use = USE_TIMEOUT;
 		snd = MSG_ACT_ARTIFACT;
 		items_allowed = USE_EQUIP;
@@ -553,45 +517,56 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	/* Figure out effect to use */
 	effect = object_effect(o_ptr);
 
-	/* If the item requires a direction, get one (allow cancelling) */	
+	/* If the item requires a direction, get one (allow cancelling) */
 	if (obj_needs_aim(o_ptr))
 		dir = args[1].direction;
 
-	/* Use energy regardless of failure */
-	p_ptr->energy_use = 100;
-
-	/* Check for use */
-	if (use == USE_CHARGE || use == USE_TIMEOUT)
+	/* Check for use if necessary, and execute the effect */
+	if ((use != USE_CHARGE && use != USE_TIMEOUT) ||
+	    check_devices(o_ptr))
 	{
-		if (!check_devices(o_ptr))
-			return;
+		/* Special message for artifacts */
+		if (artifact_p(o_ptr))
+		{
+			message(snd, 0, "You activate it.");
+			if (a_info[o_ptr->name1].effect_msg)
+				activation_message(o_ptr, a_info[o_ptr->name1].effect_msg);
+			level = a_info[o_ptr->name1].level;
+		}
+		else
+		{
+			/* Make a noise! */
+			sound(snd);
+			level = k_info[o_ptr->k_idx].level;
+		}
+
+		/* A bit of a hack to make ID work better.
+			-- Check for "obvious" effects beforehand. */
+		if (effect_obvious(effect)) object_flavor_aware(o_ptr);
+
+		/* Boost damage effects if skill > difficulty */
+		boost = p_ptr->state.skills[SKILL_DEVICE] - level;
+		if (boost < 0) boost = 0;
+
+		/* Do effect */
+		used = effect_do(effect, &ident, was_aware, dir,
+			beam_chance(o_ptr->tval), boost);
+
+		/* Quit if the item wasn't used and no knowledge was gained */
+		if (!used && (was_aware || !ident)) return;
 	}
 
-	/* Special message for artifacts */
-	if (artifact_p(o_ptr))
-	{
-		message(snd, 0, "You activate it.");
-		activation_message(o_ptr, a_text + a_info[o_ptr->name1].effect_msg);
-	}
-	else
-	{
-		/* Make a noise! */
-		sound(snd);
-	}
+	/* If the item is a null pointer or has been wiped, be done now */
+	if (!o_ptr || o_ptr->k_idx <= 1) return;
 
-	/* A bit of a hack to make ID work better.
-	   -- Check for "obvious" effects beforehand. */
-	if (effect_obvious(effect)) object_flavor_aware(o_ptr);
-
-	/* Do effect */
-	used = effect_do(effect, &ident, was_aware, dir, beam_chance(o_ptr->tval));
 	if (ident) object_notice_effect(o_ptr);
 
 	/* Food feeds the player */
 	if (o_ptr->tval == TV_FOOD || o_ptr->tval == TV_POTION)
 		(void)set_food(p_ptr->food + o_ptr->pval);
 
-	if (!used && !ident) return;
+	/* Use the turn */
+	p_ptr->energy_use = 100;
 
 	/* Mark as tried and redisplay */
 	p_ptr->notice |= (PN_COMBINE | PN_REORDER);
@@ -612,11 +587,14 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 		gain_exp((lev + (p_ptr->lev / 2)) / p_ptr->lev);
 		p_ptr->notice |= PN_SQUELCH;
 	}
-	else
+	else if (used)
 	{
 		object_flavor_tried(o_ptr);
 	}
 
+	/* If there are no more of the item left, then we're done. */
+	if (!o_ptr->number) return;
+
 	/* Chargeables act differently to single-used items when not used up */
 	if (used && use == USE_CHARGE)
 	{
@@ -635,12 +613,12 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 		if (o_ptr->name1)
 		{
 			const artifact_type *a_ptr = &a_info[o_ptr->name1];
-			o_ptr->timeout = a_ptr->time_base + damroll(a_ptr->time_dice, a_ptr->time_sides);
+			o_ptr->timeout = randcalc(a_ptr->time, 0, RANDOMISE);
 		}
 		else
 		{
 			const object_kind *k_ptr = &k_info[o_ptr->k_idx];
-			o_ptr->timeout += k_ptr->time_base + damroll(k_ptr->time_dice, k_ptr->time_sides);
+			o_ptr->timeout += randcalc(k_ptr->time, 0, RANDOMISE);
 		}
 	}
 	else if (used && use == USE_SINGLE)
@@ -666,24 +644,25 @@ void do_cmd_use(cmd_code code, cmd_arg args[])
 	if (cave_feat[py][px] == FEAT_GLYPH)
 	{
 		/* Shift any objects to further away */
-		for (o_ptr = get_first_object(py, px); o_ptr; o_ptr = get_next_object(o_ptr))
+		for (o_ptr = get_first_object(py, px); o_ptr; o_ptr =
+			get_next_object(o_ptr))
 		{
-			drop_near(o_ptr, 0, py, px);
+			drop_near(o_ptr, 0, py, px, FALSE);
 		}
-		
+
 		/* Delete the "moved" objects from their original position */
 		delete_object(py, px);
 	}
 
-	
+
 }
 
 
 /*** Refuelling ***/
 void do_cmd_refill(cmd_code code, cmd_arg args[])
 {
-	object_type *j_ptr = &inventory[INVEN_LITE];
-	u32b f[OBJ_FLAG_N];
+	object_type *j_ptr = &p_ptr->inventory[INVEN_LIGHT];
+	bitflag f[OF_SIZE];
 
 	int item = args[0].item;
 	object_type *o_ptr = object_from_item_idx(item);
@@ -697,24 +676,24 @@ void do_cmd_refill(cmd_code code, cmd_arg args[])
 	/* Check what we're wielding. */
 	object_flags(j_ptr, f);
 
-	if (j_ptr->tval != TV_LITE)
+	if (j_ptr->tval != TV_LIGHT)
 	{
 		msg_print("You are not wielding a light.");
 		return;
 	}
 
-	else if (f[2] & TR2_NO_FUEL)
+	else if (of_has(f, OF_NO_FUEL))
 	{
 		msg_print("Your light cannot be refilled.");
 		return;
 	}
 
 	/* It's a lamp */
-	if (j_ptr->sval == SV_LITE_LANTERN)
+	if (j_ptr->sval == SV_LIGHT_LANTERN)
 		refill_lamp(j_ptr, o_ptr, item);
 
 	/* It's a torch */
-	else if (j_ptr->sval == SV_LITE_TORCH)
+	else if (j_ptr->sval == SV_LIGHT_TORCH)
 		refuel_torch(j_ptr, o_ptr, item);
 
 	p_ptr->energy_use = 50;
@@ -722,196 +701,153 @@ void do_cmd_refill(cmd_code code, cmd_arg args[])
 
 
 
-/*** Handling bits ***/
+/*** Spell casting ***/
 
-/* Item "action" type */
-typedef struct
+/* Gain a specific spell, specified by spell number (for mages). */
+void do_cmd_study_spell(cmd_code code, cmd_arg args[])
 {
-	void (*action)(object_type *, int);
-	cmd_code command;
-	const char *desc;
+	int spell = args[0].choice;
 
-	const char *prompt;
-	const char *noop;
+	int item_list[INVEN_TOTAL + MAX_FLOOR_STACK];
+	int item_num;
+	int i;
 
-	bool (*filter)(const object_type *o_ptr);
-	int mode;
-	bool (*prereq)(void);
-} item_act_t;
+	/* Check the player can study at all atm */
+	if (!player_can_study())
+		return;
 
+	/* Check that the player can actually learn the nominated spell. */
+	item_tester_hook = obj_can_browse;
+	item_num = scan_items(item_list, N_ELEMENTS(item_list), (USE_INVEN | USE_FLOOR));
 
-/* All possible item actions */
-static item_act_t item_actions[] =
-{
-	/* Not setting IS_HARMLESS for this one because it could cause a true
-	 * dangerous command to not be prompted, later.
-	 */
-	{ NULL, CMD_UNINSCRIBE, "uninscribe",
-	  "Un-inscribe which item? ", "You have nothing to un-inscribe.",
-	  obj_has_inscrip, (USE_EQUIP | USE_INVEN | USE_FLOOR), NULL },
-
-	{ obj_inscribe, CMD_NULL, "inscribe",
-	  "Inscribe which item? ", "You have nothing to inscribe.",
-	  NULL, (USE_EQUIP | USE_INVEN | USE_FLOOR | IS_HARMLESS), NULL },
-
-	{ obj_examine, CMD_NULL, "examine",
-	  "Examine which item? ", "You have nothing to examine.",
-	  NULL, (USE_EQUIP | USE_INVEN | USE_FLOOR | IS_HARMLESS), NULL },
-
-	/*** Takeoff/drop/wear ***/
-	{ NULL, CMD_TAKEOFF, "takeoff",
-	  "Take off which item? ", "You are not wearing anything you can take off.",
-	  obj_can_takeoff, USE_EQUIP, NULL },
-
-	{ obj_wield, CMD_WIELD, "wield",
-	  "Wear/Wield which item? ", "You have nothing you can wear or wield.",
-	  obj_can_wear, (USE_INVEN | USE_FLOOR), NULL },
-
-	{ obj_drop, CMD_NULL, "drop",
-	  "Drop which item? ", "You have nothing to drop.",
-	  NULL, (USE_EQUIP | USE_INVEN), NULL },
-
-	/*** Spellbooks ***/
-	{ obj_browse, CMD_NULL, "browse",
-	  "Browse which book? ", "You have no books that you can read.",
-	  obj_can_browse, (USE_INVEN | USE_FLOOR | IS_HARMLESS), NULL },
-
-	{ obj_study, CMD_NULL, "study",
-	  "Study which book? ", "You have no books that you can read.",
-	  obj_can_browse, (USE_INVEN | USE_FLOOR), player_can_study },
-
-	{ obj_cast, CMD_NULL, "cast",
-	  "Use which book? ", "You have no books that you can read.",
-	  obj_can_browse, (USE_INVEN | USE_FLOOR), player_can_cast },
-
-	/*** Item usage ***/
-	{ NULL, CMD_USE_STAFF, "use",
-	  "Use which staff? ", "You have no staff to use.",
-	  obj_is_staff, (USE_INVEN | USE_FLOOR), NULL },
-
-	{ NULL, CMD_USE_WAND, "aim",
-      "Aim which wand? ", "You have no wand to aim.",
-	  obj_is_wand, (USE_INVEN | USE_FLOOR), NULL },
-
-	{ NULL, CMD_USE_ROD, "zap",
-      "Zap which rod? ", "You have no charged rods to zap.",
-	  obj_can_zap, (USE_INVEN | USE_FLOOR), NULL },
-
-	{ NULL, CMD_ACTIVATE, "activate",
-      "Activate which item? ", "You have nothing to activate.",
-	  obj_can_activate, USE_EQUIP, NULL },
-
-	{ NULL, CMD_EAT, "eat",
-      "Eat which item? ", "You have nothing to eat.",
-	  obj_is_food, (USE_INVEN | USE_FLOOR), NULL },
-
-	{ NULL, CMD_QUAFF, "quaff",
-      "Quaff which potion? ", "You have no potions to quaff.",
-	  obj_is_potion, (USE_INVEN | USE_FLOOR), NULL },
-
-	{ NULL, CMD_READ_SCROLL, "read",
-      "Read which scroll? ", "You have no scrolls to read.",
-	  obj_is_scroll, (USE_INVEN | USE_FLOOR), player_can_read },
-
-	{ NULL, CMD_REFILL, "refill",
-      "Refuel with what fuel source? ", "You have nothing to refuel with.",
-	  obj_can_refill, (USE_INVEN | USE_FLOOR), NULL },
-};
-
-
-/* List matching up to item_actions[] */
-typedef enum
-{
-	ACTION_UNINSCRIBE = 0,
-	ACTION_INSCRIBE,
-	ACTION_EXAMINE,
-	ACTION_TAKEOFF,
-	ACTION_WIELD,
-	ACTION_DROP,
-
-	ACTION_BROWSE,
-	ACTION_STUDY,
-	ACTION_CAST,
-
-	ACTION_USE_STAFF,
-	ACTION_AIM_WAND,
-	ACTION_ZAP_ROD,
-	ACTION_ACTIVATE,
-	ACTION_EAT_FOOD,
-	ACTION_QUAFF_POTION,
-	ACTION_READ_SCROLL,
-	ACTION_REFILL
-} item_act;
-
-
-
-/*** Old-style noun-verb functions ***/
-
-/* Generic "do item action" function */
-static void do_item(item_act act)
-{
-	int item;
-	object_type *o_ptr;
-	bool cmd_needs_aim = FALSE;
-
-	cptr q, s;
-
-	if (item_actions[act].prereq)
+	/* Check through all available books */
+	for (i = 0; i < item_num; i++)
 	{
-		if (!item_actions[act].prereq())
+		if (spell_in_book(spell, item_list[i]))
+		{
+			if (spell_okay_to_study(spell))
+			{
+				/* Spell is in an available book, and player is capable. */
+				spell_learn(spell);
+				p_ptr->energy_use = 100;
+			}
+			else
+			{
+				/* Spell is present, but player incapable. */
+				msg_format("You cannot learn that spell.");
+			}
+
 			return;
+		}
 	}
-
-	/* Get item */
-	q = item_actions[act].prompt;
-	s = item_actions[act].noop;
-	item_tester_hook = item_actions[act].filter;
-	if (!get_item(&item, q, s, item_actions[act].mode)) return;
-
-	/* Get the item */
-	o_ptr = object_from_item_idx(item);
-
-	/* These commands need an aim */
-	if (item_actions[act].command == CMD_QUAFF ||
-		item_actions[act].command == CMD_ACTIVATE ||
-		item_actions[act].command == CMD_USE_WAND ||
-		item_actions[act].command == CMD_USE_ROD ||
-		item_actions[act].command == CMD_USE_STAFF ||
-		item_actions[act].command == CMD_READ_SCROLL)
-	{
-		cmd_needs_aim = TRUE;
-	}
-
-	if (item_actions[act].action != NULL)
-		item_actions[act].action(o_ptr, item);
-	else if (cmd_needs_aim && obj_needs_aim(o_ptr))
-	{
-		int dir;
-		if (!get_aim_dir(&dir))
-			return;
-
-		cmd_insert(item_actions[act].command, item, dir);
-	}
-	else
-		cmd_insert(item_actions[act].command, item);
 }
 
-/* Wrappers */
-void textui_cmd_uninscribe(void) { do_item(ACTION_UNINSCRIBE); }
-void textui_cmd_inscribe(void) { do_item(ACTION_INSCRIBE); }
-void do_cmd_observe(void) { do_item(ACTION_EXAMINE); }
-void textui_cmd_takeoff(void) { do_item(ACTION_TAKEOFF); }
-void textui_cmd_wield(void) { do_item(ACTION_WIELD); }
-void textui_cmd_drop(void) { do_item(ACTION_DROP); }
-void do_cmd_browse(void) { do_item(ACTION_BROWSE); }
-void textui_cmd_study(void) { do_item(ACTION_STUDY); }
-void textui_cmd_cast(void) { do_item(ACTION_CAST); }
-void textui_cmd_pray(void) { do_item(ACTION_CAST); }
-void textui_cmd_use_staff(void) { do_item(ACTION_USE_STAFF); }
-void textui_cmd_aim_wand(void) { do_item(ACTION_AIM_WAND); }
-void textui_cmd_zap_rod(void) { do_item(ACTION_ZAP_ROD); }
-void textui_cmd_activate(void) { do_item(ACTION_ACTIVATE); }
-void textui_cmd_eat_food(void) { do_item(ACTION_EAT_FOOD); }
-void textui_cmd_quaff_potion(void) { do_item(ACTION_QUAFF_POTION); }
-void textui_cmd_read_scroll(void) { do_item(ACTION_READ_SCROLL); }
-void textui_cmd_refill(void) { do_item(ACTION_REFILL); }
+/* Cast a spell from a book */
+void do_cmd_cast(cmd_code code, cmd_arg args[])
+{
+	int spell = args[0].choice;
+	int dir = args[1].direction;
+
+	int item_list[INVEN_TOTAL + MAX_FLOOR_STACK];
+	int item_num;
+	int i;
+
+	cptr verb = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "cast" : "recite");
+	cptr noun = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
+
+	/* Check the player can cast spells at all */
+	if (!player_can_cast())
+		return;
+
+	/* Check spell is in a book they can access */
+	item_tester_hook = obj_can_browse;
+	item_num = scan_items(item_list, N_ELEMENTS(item_list), (USE_INVEN | USE_FLOOR));
+
+	/* Check through all available books */
+	for (i = 0; i < item_num; i++)
+	{
+		if (spell_in_book(spell, item_list[i]))
+		{
+			if (spell_okay_to_cast(spell))
+			{
+				/* Get the spell */
+				const magic_type *s_ptr = &mp_ptr->info[spell];	
+				
+				/* Verify "dangerous" spells */
+				if (s_ptr->smana > p_ptr->csp)
+				{
+					/* Warning */
+					msg_format("You do not have enough mana to %s this %s.", verb, noun);
+					
+					/* Flush input */
+					flush();
+					
+					/* Verify */
+					if (!get_check("Attempt it anyway? ")) return;
+				}
+
+				/* Cast a spell */
+				if (spell_cast(spell, dir))
+					p_ptr->energy_use = 100;
+			}
+			else
+			{
+				/* Spell is present, but player incapable. */
+				msg_format("You cannot %s that %s.", verb, noun);
+			}
+
+			return;
+		}
+	}
+
+}
+
+
+/* Gain a random spell from the given book (for priests) */
+void do_cmd_study_book(cmd_code code, cmd_arg args[])
+{
+	int book = args[0].item;
+	object_type *o_ptr = object_from_item_idx(book);
+
+	int spell = -1;
+	int i, k = 0;
+
+	cptr p = ((cp_ptr->spell_book == TV_MAGIC_BOOK) ? "spell" : "prayer");
+
+	/* Check the player can study at all atm */
+	if (!player_can_study())
+		return;
+
+	/* Check that the player has access to the nominated spell book. */
+	if (!item_is_available(book, obj_can_browse, (USE_INVEN | USE_FLOOR)))
+	{
+		msg_format("That item is not within your reach.");
+		return;
+	}
+
+	/* Extract spells */
+	for (i = 0; i < SPELLS_PER_BOOK; i++)
+	{
+		int s = get_spell_index(o_ptr, i);
+		
+		/* Skip non-OK spells */
+		if (s == -1) continue;
+		if (!spell_okay_to_study(s)) continue;
+		
+		/* Apply the randomizer */
+		if ((++k > 1) && (randint0(k) != 0)) continue;
+		
+		/* Track it */
+		spell = s;
+	}
+
+	if (spell < 0)
+	{
+		msg_format("You cannot learn any %ss in that book.", p);
+	}
+	else
+	{
+		spell_learn(spell);
+		p_ptr->energy_use = 100;	
+	}
+}
