@@ -16,36 +16,58 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
+
 #include "angband.h"
-#include "object/tvalsval.h"
+#include "attack.h"
+#include "cave.h"
 #include "cmds.h"
-
-
-
+#include "generate.h"
+#include "history.h"
+#include "monster/monster.h"
+#include "object/inventory.h"
+#include "object/tvalsval.h"
+#include "object/object.h"
+#include "squelch.h"
+#include "trap.h"
 
 /*
- * Search for hidden things
+ * Search for hidden things.  Returns true if a search was attempted, returns
+ * false when the player has a 0% chance of finding anything.  Prints messages
+ * for negative confirmation when verbose mode is requested.
  */
-void search(void)
+bool search(bool verbose)
 {
 	int py = p_ptr->py;
 	int px = p_ptr->px;
 
 	int y, x, chance;
 
+	bool found = FALSE;
+
 	object_type *o_ptr;
 
 
 	/* Start with base search ability */
 	chance = p_ptr->state.skills[SKILL_SEARCH];
 
-	/* Notice object flags */
-	wieldeds_notice_flag(0, TR0_SEARCH);
-
 	/* Penalize various conditions */
-	if (p_ptr->timed[TMD_BLIND] || no_lite()) chance = chance / 10;
+	if (p_ptr->timed[TMD_BLIND] || no_light()) chance = chance / 10;
 	if (p_ptr->timed[TMD_CONFUSED] || p_ptr->timed[TMD_IMAGE]) chance = chance / 10;
 
+	/* Prevent fruitless searches */
+	if (chance <= 0)
+	{
+		if (verbose)
+		{
+			msg_print("You can't make out your surroundings well enough to search.");
+
+			/* Cancel repeat */
+			disturb(0, 0);
+		}
+
+		return FALSE;
+	}
+
 	/* Search the nearby grids, which are always in bounds */
 	for (y = (py - 1); y <= (py + 1); y++)
 	{
@@ -57,6 +79,8 @@ void search(void)
 				/* Invisible trap */
 				if (cave_feat[y][x] == FEAT_INVIS)
 				{
+					found = TRUE;
+
 					/* Pick a trap */
 					pick_trap(y, x);
 
@@ -70,6 +94,8 @@ void search(void)
 				/* Secret door */
 				if (cave_feat[y][x] == FEAT_SECRET)
 				{
+					found = TRUE;
+
 					/* Message */
 					msg_print("You have found a secret door.");
 
@@ -95,6 +121,8 @@ void search(void)
 					/* Identify once */
 					if (!object_is_known(o_ptr))
 					{
+						found = TRUE;
+
 						/* Message */
 						msg_print("You have discovered a trap on the chest!");
 
@@ -108,6 +136,16 @@ void search(void)
 			}
 		}
 	}
+
+	if (verbose && !found)
+	{
+		if (chance >= 100)
+			msg_print("There are no secrets here.");
+		else
+			msg_print("You found nothing.");
+	}
+
+	return TRUE;
 }
 
 
@@ -129,6 +167,7 @@ static void py_pickup_gold(void)
 	object_type *o_ptr;
 
 	int sound_msg;
+	bool verbal = FALSE;
 
 	/* Allocate an array of ordinary gold objects */
 	treasure = C_ZNEW(SV_GOLD_MAX, byte);
@@ -150,6 +189,10 @@ static void py_pickup_gold(void)
 		/* Note that we have this kind of treasure */
 		treasure[o_ptr->sval]++;
 
+		/* Remember whether feedback message is in order */
+		if (!squelch_item_ok(o_ptr))
+			verbal = TRUE;
+
 		/* Increment total value */
 		total_gold += (s32b)o_ptr->pval;
 
@@ -211,7 +254,8 @@ static void py_pickup_gold(void)
 		else                       sound_msg = MSG_MONEY3;
 
 		/* Display the message */
-		message(sound_msg, 0, buf);
+		if (verbal)
+			message(sound_msg, 0, buf);
 
 		/* Add gold to purse */
 		p_ptr->au += total_gold;
@@ -232,9 +276,8 @@ static void py_pickup_gold(void)
 static bool auto_pickup_okay(const object_type *o_ptr)
 {
 	if (!inven_carry_okay(o_ptr)) return FALSE;
-
-	if (OPT(pickup_inven) && inven_stack_okay(o_ptr)) return TRUE;
 	if (OPT(pickup_always) || check_for_inscrip(o_ptr, "=g")) return TRUE;
+	if (OPT(pickup_inven) && inven_stack_okay(o_ptr)) return TRUE;
 
 	return FALSE;
 }
@@ -245,39 +288,61 @@ static bool auto_pickup_okay(const object_type *o_ptr)
  */
 static void py_pickup_aux(int o_idx, bool msg)
 {
-	int slot;
+	int slot, quiver_slot = 0;
 
 	char o_name[80];
 	object_type *o_ptr = &o_list[o_idx];
 
 	/* Carry the object */
-	slot = inven_carry(o_ptr);
+	slot = inven_carry(p_ptr, o_ptr);
 
 	/* Handle errors (paranoia) */
 	if (slot < 0) return;
 
+	/* If we have picked up ammo which matches something in the quiver, note
+	 * that it so that we can wield it later (and suppress pick up message) */
+	if (obj_is_ammo(o_ptr)) 
+	{
+		int i;
+		for (i = QUIVER_START; i < QUIVER_END; i++) 
+		{
+			if (!p_ptr->inventory[i].k_idx) continue;
+			if (!object_similar(&p_ptr->inventory[i], o_ptr,
+				OSTACK_QUIVER)) continue;
+			quiver_slot = i;
+			break;
+		}
+	}
+
 	/* Get the new object */
-	o_ptr = &inventory[slot];
+	o_ptr = &p_ptr->inventory[slot];
 
 	/* Set squelch status */
 	p_ptr->notice |= PN_SQUELCH;
 
+	/* Automatically sense artifacts */
+	object_sense_artifact(o_ptr);
+
+	/* Log artifacts if found */
+	if (artifact_p(o_ptr))
+		history_add_artifact(o_ptr->name1, object_is_known(o_ptr), TRUE);
+
 	/* Optionally, display a message */
-	if (msg)
+	if (msg && !quiver_slot)
 	{
 		/* Describe the object */
-		object_desc(o_name, sizeof(o_name), o_ptr, TRUE, ODESC_FULL);
+		object_desc(o_name, sizeof(o_name), o_ptr, ODESC_PREFIX | ODESC_FULL);
 
 		/* Message */
 		msg_format("You have %s (%c).", o_name, index_to_label(slot));
 	}
 
-	/* Log if picking up an artifact. */
-	if (artifact_p(o_ptr))
-		history_add_artifact(o_ptr->name1, object_is_known(o_ptr));
 
 	/* Delete the object */
 	delete_object_idx(o_idx);
+
+	/* If we have a quiver slot that this ammo matches, use it */
+	if (quiver_slot) wield_item(o_ptr, slot, quiver_slot);
 }
 
 
@@ -336,7 +401,7 @@ byte py_pickup(int pickup)
 	int can_pickup = 0;
 	bool call_function_again = FALSE;
 
-	bool blind = ((p_ptr->timed[TMD_BLIND]) || (no_lite()));
+	bool blind = ((p_ptr->timed[TMD_BLIND]) || (no_light()));
 	bool msg = TRUE;
 
 
@@ -415,8 +480,12 @@ byte py_pickup(int pickup)
 			o_ptr = &o_list[floor_o_idx];
 
 			/* Describe the object.  Less detail if blind. */
-			if (blind) object_desc(o_name, sizeof(o_name), o_ptr, TRUE, ODESC_BASE);
-			else       object_desc(o_name, sizeof(o_name), o_ptr, TRUE, ODESC_FULL);
+			if (blind)
+				object_desc(o_name, sizeof(o_name), o_ptr,
+							ODESC_PREFIX | ODESC_BASE);
+			else
+				object_desc(o_name, sizeof(o_name), o_ptr,
+							ODESC_PREFIX | ODESC_FULL);
 
 			/* Message */
 			message_flush();
@@ -427,6 +496,8 @@ byte py_pickup(int pickup)
 			/* Optionally, display more information about floor items */
 			if (OPT(pickup_detail))
 			{
+				ui_event_data e;
+
 				if (!can_pickup)	p = "have no room for the following objects";
 				else if (blind)     p = "feel something on the floor";
 
@@ -437,16 +508,17 @@ byte py_pickup(int pickup)
 				screen_save();
 
 				/* Display objects on the floor */
-				show_floor(floor_list, floor_num, FALSE);
+				show_floor(floor_list, floor_num, (OLIST_WEIGHT));
 
 				/* Display prompt */
 				prt(format("You %s: ", p), 0, 0);
 
 				/* Move cursor back to character, if needed */
-				if (OPT(hilite_player)) move_cursor_relative(p_ptr->py, p_ptr->px);
+				if (OPT(highlight_player)) move_cursor_relative(p_ptr->py, p_ptr->px);
 
 				/* Wait for it.  Use key as next command. */
-				p_ptr->command_new = inkey();
+				e = inkey_ex();
+				Term_event_push(&e);
 
 				/* Restore screen */
 				screen_load();
@@ -495,7 +567,7 @@ byte py_pickup(int pickup)
 		/* Get an object or exit. */
 		q = "Get which item?";
 		s = "You see nothing there.";
-		if (!get_item(&item, q, s, USE_FLOOR))
+		if (!get_item(&item, q, s, CMD_PICKUP, USE_FLOOR))
 			return (objs_picked_up);
 
 		this_o_idx = 0 - item;
@@ -561,13 +633,8 @@ void move_player(int dir)
 			(cave_feat[y][x] <= FEAT_DOOR_TAIL))
 	{
 		/* Auto-repeat if not already repeating */
-		if (!p_ptr->command_rep && (p_ptr->command_arg <= 0))
-		{
-			p_ptr->command_rep = 99;
-
-			/* Reset the command count */
-			p_ptr->command_arg = 0;
-		}
+		if (cmd_get_nrepeats() == 0)
+			cmd_set_repeat(99);
 
 		do_cmd_alter_aux(dir);
 	}
@@ -586,7 +653,7 @@ void move_player(int dir)
 			{
 				message(MSG_HITWALL, 0, "You feel a pile of rubble blocking your way.");
 				cave_info[y][x] |= (CAVE_MARK);
-				lite_spot(y, x);
+				light_spot(y, x);
 			}
 
 			/* Closed door */
@@ -594,7 +661,7 @@ void move_player(int dir)
 			{
 				message(MSG_HITWALL, 0, "You feel a door blocking your way.");
 				cave_info[y][x] |= (CAVE_MARK);
-				lite_spot(y, x);
+				light_spot(y, x);
 			}
 
 			/* Wall (or secret door) */
@@ -602,7 +669,7 @@ void move_player(int dir)
 			{
 				message(MSG_HITWALL, 0, "You feel a wall blocking your way.");
 				cave_info[y][x] |= (CAVE_MARK);
-				lite_spot(y, x);
+				light_spot(y, x);
 			}
 		}
 
@@ -653,26 +720,25 @@ void move_player(int dir)
 		if ((p_ptr->state.skills[SKILL_SEARCH_FREQUENCY] >= 50) ||
 		    one_in_(50 - p_ptr->state.skills[SKILL_SEARCH_FREQUENCY]))
 		{
-			search();
+			search(FALSE);
 		}
 
 		/* Continuous Searching */
 		if (p_ptr->searching)
 		{
-			search();
+			search(FALSE);
 		}
 
 
 		/* Handle "store doors" */
-		if ((cave_feat[y][x] >= FEAT_SHOP_HEAD) &&
-		    (cave_feat[y][x] <= FEAT_SHOP_TAIL))
+		if ((cave_feat[p_ptr->py][p_ptr->px] >= FEAT_SHOP_HEAD) &&
+			(cave_feat[p_ptr->py][p_ptr->px] <= FEAT_SHOP_TAIL))
 		{
 			/* Disturb */
 			disturb(0, 0);
 			cmd_insert(CMD_ENTER_STORE);
 		}
 
-
 		/* All other grids (including traps) */
 		else
 		{
