@@ -15,11 +15,19 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-#include "angband.h"
-#include "object/tvalsval.h"
 
+#include "angband.h"
+#include "attack.h"
+#include "cave.h"
 #include "cmds.h"
+#include "files.h"
 #include "game-cmd.h"
+#include "generate.h"
+#include "monster/monster.h"
+#include "object/tvalsval.h"
+#include "spells.h"
+#include "squelch.h"
+#include "trap.h"
 
 /*
  * Go up one level
@@ -88,11 +96,9 @@ void do_cmd_go_down(cmd_code code, cmd_arg args[])
  */
 void do_cmd_search(cmd_code code, cmd_arg args[])
 {
-	/* Take a turn */
-	p_ptr->energy_use = 100;
-
-	/* Search */
-	search();
+	/* Only take a turn if attempted */
+	if (search(TRUE))
+		p_ptr->energy_use = 100;
 }
 
 
@@ -169,8 +175,7 @@ static s16b chest_check(int y, int x)
  * Small chests often contain "gold", while Large chests always contain
  * items.  Wooden chests contain 2 items, Iron chests contain 4 items,
  * and Steel chests contain 6 items.  The "value" of the items in a
- * chest is based on the "power" of the chest, which is in turn based
- * on the level on which the chest is generated.
+ * chest is based on the level on which the chest is generated.
  */
 static void chest_death(int y, int x, s16b o_idx)
 {
@@ -200,7 +205,9 @@ static void chest_death(int y, int x, s16b o_idx)
 	opening_chest = TRUE;
 
 	/* Determine the "value" of the items */
-	value = ABS(o_ptr->pval) + 10;
+	value = o_ptr->origin_depth - 10 + 2 * o_ptr->sval;
+	if (value < 1)
+		value = 1;
 
 	/* Drop some objects (non-chests) */
 	for (; number > 0; --number)
@@ -215,15 +222,19 @@ static void chest_death(int y, int x, s16b o_idx)
 		if (tiny && (randint0(100) < 75))
 			make_gold(i_ptr, value, SV_GOLD_ANY);
 
-		/* Otherwise drop an item */
+		/* Otherwise drop an item, as long as it isn't a chest */
 		else
 		{
 			if (!make_object(i_ptr, value, FALSE, FALSE))
 				continue;
+			if (i_ptr->tval == TV_CHEST)
+				continue;
+			i_ptr->origin = ORIGIN_CHEST;
+			i_ptr->origin_depth = o_ptr->origin_depth;
 		}
 
 		/* Drop it in the dungeon */
-		drop_near(i_ptr, -1, y, x);
+		drop_near(i_ptr, 0, y, x, TRUE);
 	}
 
 	/* No longer opening a chest */
@@ -279,7 +290,7 @@ static void chest_trap(int y, int x, s16b o_idx)
 		if (!(p_ptr->state.resist_pois || p_ptr->timed[TMD_OPP_POIS]))
 			(void)inc_timed(TMD_POISONED, 10 + randint1(20), TRUE);
 		else if (p_ptr->state.resist_pois)
-			wieldeds_notice_flag(1, TR1_RES_POIS);
+			wieldeds_notice_flag(OF_RES_POIS);
 	}
 
 	/* Paralyze */
@@ -289,7 +300,7 @@ static void chest_trap(int y, int x, s16b o_idx)
 		if (!p_ptr->state.free_act)
 			(void)inc_timed(TMD_PARALYZED, 10 + randint1(20), TRUE);
 		else
-			wieldeds_notice_flag(2, TR2_FREE_ACT);
+			wieldeds_notice_flag(OF_FREE_ACT);
 	}
 
 	/* Summon monsters */
@@ -343,7 +354,7 @@ static bool do_cmd_open_chest(int y, int x, s16b o_idx)
 		i = p_ptr->state.skills[SKILL_DISARM];
 
 		/* Penalize some conditions */
-		if (p_ptr->timed[TMD_BLIND] || no_lite()) i = i / 10;
+		if (p_ptr->timed[TMD_BLIND] || no_light()) i = i / 10;
 		if (p_ptr->timed[TMD_CONFUSED] || p_ptr->timed[TMD_IMAGE]) i = i / 10;
 
 		/* Extract the difficulty */
@@ -383,7 +394,7 @@ static bool do_cmd_open_chest(int y, int x, s16b o_idx)
 		p_ptr->notice |= PN_SQUELCH;
 
 		/* Redraw chest, to be on the safe side (it may have been squelched) */
-		lite_spot(y, x);
+		light_spot(y, x);
 	}
 
 	/* Result */
@@ -411,7 +422,7 @@ static bool do_cmd_disarm_chest(int y, int x, s16b o_idx)
 	i = p_ptr->state.skills[SKILL_DISARM];
 
 	/* Penalize some conditions */
-	if (p_ptr->timed[TMD_BLIND] || no_lite()) i = i / 10;
+	if (p_ptr->timed[TMD_BLIND] || no_light()) i = i / 10;
 	if (p_ptr->timed[TMD_CONFUSED] || p_ptr->timed[TMD_IMAGE]) i = i / 10;
 
 	/* Extract the difficulty */
@@ -470,7 +481,7 @@ static bool do_cmd_disarm_chest(int y, int x, s16b o_idx)
 /*
  * Return TRUE if the given feature is an open door
  */
-static bool is_open(int feat)
+bool is_open(int feat)
 {
 	return (feat == FEAT_OPEN);
 }
@@ -479,7 +490,7 @@ static bool is_open(int feat)
 /*
  * Return TRUE if the given feature is a closed door
  */
-static bool is_closed(int feat)
+bool is_closed(int feat)
 {
 	return ((feat >= FEAT_DOOR_HEAD) &&
 	        (feat <= FEAT_DOOR_TAIL));
@@ -489,7 +500,7 @@ static bool is_closed(int feat)
 /*
  * Return TRUE if the given feature is a trap
  */
-static bool is_trap(int feat)
+bool is_trap(int feat)
 {
 	return ((feat >= FEAT_TRAP_HEAD) &&
 	        (feat <= FEAT_TRAP_TAIL));
@@ -499,7 +510,7 @@ static bool is_trap(int feat)
 /*
  * Return the number of doors/traps around (or under) the character.
  */
-static int count_feats(int *y, int *x, bool (*test)(int feat), bool under)
+int count_feats(int *y, int *x, bool (*test)(int feat), bool under)
 {
 	int d;
 	int xx, yy;
@@ -541,7 +552,7 @@ static int count_feats(int *y, int *x, bool (*test)(int feat), bool under)
  * Return the number of chests around (or under) the character.
  * If requested, count only trapped chests.
  */
-static int count_chests(int *y, int *x, bool trapped)
+int count_chests(int *y, int *x, bool trapped)
 {
 	int d, count, o_idx;
 
@@ -592,7 +603,7 @@ static int count_chests(int *y, int *x, bool trapped)
  * Extract a "direction" which will move one step from the player location
  * towards the given "target" location (or "5" if no motion necessary).
  */
-static int coords_to_dir(int y, int x)
+int coords_to_dir(int y, int x)
 {
 	return (motion_dir(p_ptr->py, p_ptr->px, y, x));
 }
@@ -661,7 +672,7 @@ static bool do_cmd_open_aux(int y, int x)
 		i = p_ptr->state.skills[SKILL_DISARM];
 
 		/* Penalize some conditions */
-		if (p_ptr->timed[TMD_BLIND] || no_lite()) i = i / 10;
+		if (p_ptr->timed[TMD_BLIND] || no_light()) i = i / 10;
 		if (p_ptr->timed[TMD_CONFUSED] || p_ptr->timed[TMD_IMAGE]) i = i / 10;
 
 		/* Extract the lock power */
@@ -746,8 +757,12 @@ void do_cmd_open(cmd_code code, cmd_arg args[])
 
 
 	/* Verify legality */
-	if (!o_idx && !do_cmd_open_test(y, x)) return;
-
+	if (!o_idx && !do_cmd_open_test(y, x))
+	{
+		/* Cancel repeat */
+		disturb(0, 0);
+		return;
+	}
 
 	/* Take a turn */
 	p_ptr->energy_use = 100;
@@ -792,31 +807,6 @@ void do_cmd_open(cmd_code code, cmd_arg args[])
 	if (!more) disturb(0, 0);
 }
 
-void textui_cmd_open(void)
-{
-	int y, x, dir = DIR_UNKNOWN;
-
-	/* Easy Open */
-	if (OPT(easy_open))
-	{
-		int num_doors, num_chests;
-
-		/* Count closed doors */
-		num_doors = count_feats(&y, &x, is_closed, FALSE);
-
-		/* Count chests (locked) */
-		num_chests = count_chests(&y, &x, FALSE);
-
-		/* See if only one target */
-		if ((num_doors + num_chests) == 1)
-		{
-			dir = coords_to_dir(y, x);
-		}
-	}
-
-	cmd_insert(CMD_OPEN, dir);
-}
-
 
 /*
  * Determine if a given grid may be "closed"
@@ -904,7 +894,12 @@ void do_cmd_close(cmd_code code, cmd_arg args[])
 	x = p_ptr->px + ddx[dir];
 
 	/* Verify legality */
-	if (!do_cmd_close_test(y, x)) return;
+	if (!do_cmd_close_test(y, x))
+	{
+		/* Cancel repeat */
+		disturb(0, 0);
+		return;
+	}
 
 	/* Take a turn */
 	p_ptr->energy_use = 100;
@@ -939,28 +934,6 @@ void do_cmd_close(cmd_code code, cmd_arg args[])
 	if (!more) disturb(0, 0);
 }
 
-void textui_cmd_close(void)
-{
-	int y, x, dir = DIR_UNKNOWN;
-
-	/* Easy Close */
-	if (OPT(easy_open))
-	{
-		/* Count open doors */
-		if (count_feats(&y, &x, is_open, FALSE) == 1)
-		{
-			dir = coords_to_dir(y, x);
-		}
-	}
-	else
-	{
-		if (!get_rep_dir(&dir))
-			return;
-	}
-
-	cmd_insert(CMD_CLOSE, dir);
-}
-
 
 /*
  * Determine if a given grid may be "tunneled"
@@ -1190,7 +1163,7 @@ static bool do_cmd_tunnel_aux(int y, int x)
 			more = TRUE;
 
 			/* Occasional Search XXX XXX */
-			if (randint0(100) < 25) search();
+			if (randint0(100) < 25) search(FALSE);
 		}
 	}
 
@@ -1236,8 +1209,12 @@ void do_cmd_tunnel(cmd_code code, cmd_arg args[])
 
 
 	/* Oops */
-	if (!do_cmd_tunnel_test(y, x)) return;
-
+	if (!do_cmd_tunnel_test(y, x))
+	{
+		/* Cancel repeat */
+		disturb(0, 0);
+		return;
+	}
 
 	/* Take a turn */
 	p_ptr->energy_use = 100;
@@ -1272,14 +1249,6 @@ void do_cmd_tunnel(cmd_code code, cmd_arg args[])
 	if (!more) disturb(0, 0);
 }
 
-void textui_cmd_tunnel(void)
-{
-	int dir;
-	if (!get_rep_dir(&dir)) return;
-	cmd_insert(CMD_TUNNEL, dir);
-}
-
-
 /*
  * Determine if a given grid may be "disarmed"
  */
@@ -1332,13 +1301,13 @@ static bool do_cmd_disarm_aux(int y, int x)
 
 
 	/* Get the trap name */
-	name = (f_name + f_info[cave_feat[y][x]].name);
+	name = f_info[cave_feat[y][x]].name;
 
 	/* Get the "disarm" factor */
 	i = p_ptr->state.skills[SKILL_DISARM];
 
 	/* Penalize some conditions */
-	if (p_ptr->timed[TMD_BLIND] || no_lite()) i = i / 10;
+	if (p_ptr->timed[TMD_BLIND] || no_light()) i = i / 10;
 	if (p_ptr->timed[TMD_CONFUSED] || p_ptr->timed[TMD_IMAGE]) i = i / 10;
 
 	/* XXX XXX XXX Variable power? */
@@ -1418,8 +1387,12 @@ void do_cmd_disarm(cmd_code code, cmd_arg args[])
 
 
 	/* Verify legality */
-	if (!o_idx && !do_cmd_disarm_test(y, x)) return;
-
+	if (!o_idx && !do_cmd_disarm_test(y, x))
+	{
+		/* Cancel repeat */
+		disturb(0, 0);
+		return;
+	}
 
 	/* Take a turn */
 	p_ptr->energy_use = 100;
@@ -1464,38 +1437,6 @@ void do_cmd_disarm(cmd_code code, cmd_arg args[])
 	if (!more) disturb(0, 0);
 }
 
-void textui_cmd_disarm(void)
-{
-	int y, x, dir;
-
-	dir = DIR_UNKNOWN;
-
-	/* Easy Disarm */
-	if (OPT(easy_open))
-	{
-		int num_traps, num_chests;
-
-		/* Count visible traps */
-		num_traps = count_feats(&y, &x, is_trap, TRUE);
-
-		/* Count chests (trapped) */
-		num_chests = count_chests(&y, &x, TRUE);
-
-		/* See if only one target */
-		if (num_traps || num_chests)
-		{
-			if (num_traps + num_chests <= 1)
-				dir = coords_to_dir(y, x);
-		}
-	}
-	else
-	{
-		if (!get_rep_dir(&dir))
-			return;
-	}
-
-	cmd_insert(CMD_DISARM, dir);
-}
 
 /*
  * Determine if a given grid may be "bashed"
@@ -1627,6 +1568,7 @@ static bool do_cmd_bash_aux(int y, int x)
 void do_cmd_bash(cmd_code code, cmd_arg args[])
 {
 	int y, x, dir;
+	bool more = FALSE;
 
 	dir = args[0].direction;
 
@@ -1636,8 +1578,12 @@ void do_cmd_bash(cmd_code code, cmd_arg args[])
 
 
 	/* Verify legality */
-	if (!do_cmd_bash_test(y, x)) return;
-
+	if (!do_cmd_bash_test(y, x))
+	{
+		/* Cancel repeat */
+		disturb(0, 0);
+		return;
+	}
 
 	/* Take a turn */
 	p_ptr->energy_use = 100;
@@ -1665,24 +1611,13 @@ void do_cmd_bash(cmd_code code, cmd_arg args[])
 	else
 	{
 		/* Bash the door */
-		if (!do_cmd_bash_aux(y, x))
-		{
-			/* Cancel repeat */
-			disturb(0, 0);
-		}
+		more = do_cmd_bash_aux(y, x);
 	}
+
+	/* Cancel repeat unless we may continue */
+	if (!more) disturb(0, 0);
 }
 
-void textui_cmd_bash(void)
-{
-	int dir;
-	if (!get_rep_dir(&dir))
-		return;
-
-	cmd_insert(CMD_BASH, dir);
-}
-
-
 
 /*
  * Manipulate an adjacent grid in some way
@@ -1730,53 +1665,46 @@ void do_cmd_alter_aux(int dir)
 	/* Attack monsters */
 	if (cave_m_idx[y][x] > 0)
 	{
-		/* Attack */
 		py_attack(y, x);
 	}
 
 	/* Tunnel through walls */
 	else if (feat >= FEAT_SECRET)
 	{
-		/* Tunnel */
 		more = do_cmd_tunnel_aux(y, x);
 	}
+
 #if 0
 	/* Bash jammed doors */
 	else if (feat >= FEAT_DOOR_HEAD + 0x08)
 	{
-		/* Tunnel */
 		more = do_cmd_bash_aux(y, x);
 	}
-#endif /* 0 */
+#endif
+
 	/* Open closed doors */
 	else if (feat >= FEAT_DOOR_HEAD)
 	{
-		/* Tunnel */
 		more = do_cmd_open_aux(y, x);
 	}
 
 	/* Disarm traps */
 	else if (feat >= FEAT_TRAP_HEAD)
 	{
-		/* Tunnel */
 		more = do_cmd_disarm_aux(y, x);
 	}
 
 #if 0
-
 	/* Close open doors */
 	else if (feat == FEAT_OPEN)
 	{
-		/* Close */
 		more = do_cmd_close_aux(y, x);
 	}
-
 #endif
 
 	/* Oops */
 	else
 	{
-		/* Oops */
 		msg_print("You spin around.");
 	}
 
@@ -1789,15 +1717,6 @@ void do_cmd_alter(cmd_code code, cmd_arg args[])
 	do_cmd_alter_aux(args[0].direction);
 }
 
-void textui_cmd_alter(void)
-{
-	int dir;
-
-	if (!get_rep_dir(&dir))
-		return;
-
-	cmd_insert(CMD_ALTER, dir);
-}
 
 /*
  * Find the index of some "spikes", if possible.
@@ -1811,7 +1730,7 @@ static bool get_spike(int *ip)
 	/* Check every item in the pack */
 	for (i = 0; i < INVEN_PACK; i++)
 	{
-		object_type *o_ptr = &inventory[i];
+		object_type *o_ptr = &p_ptr->inventory[i];
 
 		/* Skip non-objects */
 		if (!o_ptr->k_idx) continue;
@@ -1943,15 +1862,6 @@ void do_cmd_spike(cmd_code code, cmd_arg args[])
 	}
 }
 
-void textui_cmd_spike(void)
-{
-	int dir;
-	if (!get_rep_dir(&dir))
-		return;
-
-	cmd_insert(CMD_JAM, dir);
-}
-
 
 /*
  * Determine if a given grid may be "walked"
@@ -2012,6 +1922,9 @@ static bool do_cmd_walk_test(int y, int x)
 			message(MSG_HITWALL, 0, "There is a wall in the way!");
 		}
 
+		/* Cancel repeat */
+		disturb(0, 0);
+
 		/* Nope */
 		return (FALSE);
 	}
@@ -2059,19 +1972,6 @@ void do_cmd_walk(cmd_code code, cmd_arg args[])
 	move_player(dir);
 }
 
-/*
- * Tell the game we want to walk - in future we might want to supply
- * directions here rather than rely on keymap/macro things.
- */
-void textui_cmd_walk(void)
-{
-	int dir;
-	if (!get_rep_dir(&dir))
-		return;
-
-	cmd_insert(CMD_WALK, dir);
-}
-
 
 /*
  * Jump into a trap, turn off pickup (does not work).
@@ -2092,15 +1992,6 @@ void do_cmd_jump(cmd_code code, cmd_arg args[])
 	OPT(easy_alter) = old_easy_alter;
 }
 
-void textui_cmd_jump(void)
-{
-	int dir;
-	if (!get_rep_dir(&dir))
-		return;
-
-	cmd_insert(CMD_JUMP, dir);
-}
-
 
 /*
  * Start running.
@@ -2133,14 +2024,6 @@ void do_cmd_run(cmd_code code, cmd_arg args[])
 	run_step(dir);
 }
 
-void textui_cmd_run(void)
-{
-	int dir;
-	if (!get_rep_dir(&dir))
-		return;
-
-	cmd_insert(CMD_RUN, dir);
-}
 
 /*
  * Start running with pathfinder.
@@ -2181,13 +2064,13 @@ void do_cmd_hold(cmd_code code, cmd_arg args[])
 	if ((p_ptr->state.skills[SKILL_SEARCH_FREQUENCY] >= 50) ||
 	    one_in_(50 - p_ptr->state.skills[SKILL_SEARCH_FREQUENCY]))
 	{
-		search();
+		search(FALSE);
 	}
 
 	/* Continuous Searching */
 	if (p_ptr->searching)
 	{
-		search();
+		search(FALSE);
 	}
 
 	/* Pick things up, not using extra energy */
@@ -2233,39 +2116,27 @@ void do_cmd_pickup(cmd_code code, cmd_arg args[])
  */
 void do_cmd_rest(cmd_code code, cmd_arg args[])
 {
-	/* Save the rest code */
-	switch (args[0].choice)
+	/* 
+	 * A little sanity checking on the input - only the specified negative 
+	 * values are valid. 
+	 */
+	if ((args[0].choice < 0) &&
+		((args[0].choice != REST_COMPLETE) &&
+		 (args[0].choice != REST_ALL_POINTS) &&
+		 (args[0].choice != REST_SOME_POINTS))) 
 	{
-		case REST_ALL:
-		{
-			p_ptr->resting = -2;
-			break;
-		}
-
-		case REST_ALL_POINTS:
-		{
-			p_ptr->resting = -1;
-			break;
-		}
-
-		case REST_SOME_POINTS:
-		{
-			p_ptr->resting = -3;
-			break;
-		}
-
-		default:
-		{
-			p_ptr->resting = p_ptr->command_arg;
-		}
+		return;
 	}
 
+	/* Save the rest code */
+	p_ptr->resting = args[0].choice;
+	
+	/* Truncate overlarge values */
+	if (p_ptr->resting > 9999) p_ptr->resting = 9999;
+	
 	/* Take a turn XXX XXX XXX (?) */
 	p_ptr->energy_use = 100;
 
-	/* Cancel the arg */
-	p_ptr->command_arg = 0;
-
 	/* Cancel searching */
 	p_ptr->searching = FALSE;
 
@@ -2298,29 +2169,33 @@ void textui_cmd_rest(void)
 		/* Rest until done */
 		if (out_val[0] == '&')
 		{
-			cmd_insert(CMD_REST, REST_ALL);
+			cmd_insert(CMD_REST);
+			cmd_set_arg_choice(cmd_get_top(), 0, REST_COMPLETE);
 		}
 
 		/* Rest a lot */
 		else if (out_val[0] == '*')
 		{
-			cmd_insert(CMD_REST, REST_ALL_POINTS);
+			cmd_insert(CMD_REST);
+			cmd_set_arg_choice(cmd_get_top(), 0, REST_ALL_POINTS);
 		}
 
 		/* Rest until HP or SP filled */
 		else if (out_val[0] == '!')
 		{
-			cmd_insert(CMD_REST, REST_SOME_POINTS);
+			cmd_insert(CMD_REST);
+			cmd_set_arg_choice(cmd_get_top(), 0, REST_SOME_POINTS);
 		}
 		
 		/* Rest some */
 		else
 		{
-			p_ptr->command_arg = atoi(out_val);
-			if (p_ptr->command_arg <= 0) return;
-			if (p_ptr->command_arg > 9999) p_ptr->command_arg = 9999;
+			int turns = atoi(out_val);
+			if (turns <= 0) return;
+			if (turns > 9999) turns = 9999;
 			
-			cmd_insert(CMD_REST, REST_TURNS);
+			cmd_insert(CMD_REST);
+			cmd_set_arg_choice(cmd_get_top(), 0, turns);
 		}
 	}
 }
