@@ -15,16 +15,26 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-#include "angband.h"
-#include "object/tvalsval.h"
 
+#include "angband.h"
+#include "button.h"
+#include "cave.h"
 #include "cmds.h"
+#include "monster/monster.h"
+#include "object/inventory.h"
+#include "object/tvalsval.h"
+#include "object/object.h"
+#include "squelch.h"
+#include "target.h"
 
 /*
  * Display inventory
  */
 void do_cmd_inven(void)
 {
+	ui_event_data e;
+	int diff = weight_remaining();
+
 	/* Hack -- Start in "inventory" mode */
 	p_ptr->command_wrk = (USE_INVEN);
 
@@ -35,29 +45,25 @@ void do_cmd_inven(void)
 	item_tester_full = TRUE;
 
 	/* Display the inventory */
-	show_inven();
+	show_inven(OLIST_WEIGHT | OLIST_QUIVER);
 
 	/* Hack -- hide empty slots */
 	item_tester_full = FALSE;
 
 	/* Prompt for a command */
-	prt(format("(Inventory) Burden %d.%dlb (%d%% capacity). Command: ",
-	    p_ptr->total_weight / 10, p_ptr->total_weight % 10,
-	    (10 * p_ptr->total_weight) / (6 * adj_str_wgt[p_ptr->state.stat_ind[A_STR]])), 0, 0);
+	prt(format("(Inventory) Burden %d.%d lb (%d.%d lb %s). Command: ",
+		        p_ptr->total_weight / 10, p_ptr->total_weight % 10,
+		        abs(diff) / 10, abs(diff) % 10,
+		        (diff < 0 ? "overweight" : "remaining")),
+	    0, 0);
 
-	/* Hack -- Get a new command */
-	p_ptr->command_new = inkey();
+	/* Get a new command */
+	e = inkey_ex();
+	if (!(e.type == EVT_KBRD && e.key == ESCAPE))
+		Term_event_push(&e);
 
 	/* Load screen */
 	screen_load();
-
-
-	/* Hack -- Process "Escape" */
-	if (p_ptr->command_new == ESCAPE)
-	{
-		/* Reset stuff */
-		p_ptr->command_new = 0;
-	}
 }
 
 
@@ -66,6 +72,8 @@ void do_cmd_inven(void)
  */
 void do_cmd_equip(void)
 {
+	ui_event_data e;
+
 	/* Hack -- Start in "equipment" mode */
 	p_ptr->command_wrk = (USE_EQUIP);
 
@@ -76,7 +84,7 @@ void do_cmd_equip(void)
 	item_tester_full = TRUE;
 
 	/* Display the equipment */
-	show_equip();
+	show_equip(OLIST_WEIGHT);
 
 	/* Hack -- undo the hack above */
 	item_tester_full = FALSE;
@@ -84,19 +92,13 @@ void do_cmd_equip(void)
 	/* Prompt for a command */
 	prt("(Equipment) Command: ", 0, 0);
 
-	/* Hack -- Get a new command */
-	p_ptr->command_new = inkey();
+	/* Get a new command */
+	e = inkey_ex();
+	if (!(e.type == EVT_KBRD && e.key == ESCAPE))
+		Term_event_push(&e);
 
 	/* Load screen */
 	screen_load();
-
-
-	/* Hack -- Process "Escape" */
-	if (p_ptr->command_new == ESCAPE)
-	{
-		/* Reset stuff */
-		p_ptr->command_new = 0;
-	}
 }
 
 
@@ -108,9 +110,20 @@ void wield_item(object_type *o_ptr, int item, int slot)
 	object_type object_type_body;
 	object_type *i_ptr = &object_type_body;
 
-	cptr act;
+	cptr fmt;
 	char o_name[80];
 
+	bool combined_ammo = FALSE;
+	int num = 1;
+
+	/* If we are stacking ammo in the quiver */
+	if (obj_is_ammo(o_ptr))
+	{
+		num = o_ptr->number;
+		combined_ammo = object_similar(o_ptr, &p_ptr->inventory[slot],
+			OSTACK_QUIVER);
+	}
+
 	/* Take a turn */
 	p_ptr->energy_use = 100;
 
@@ -118,73 +131,93 @@ void wield_item(object_type *o_ptr, int item, int slot)
 	object_copy(i_ptr, o_ptr);
 
 	/* Modify quantity */
-	i_ptr->number = 1;
+	i_ptr->number = num;
 
 	/* Decrease the item (from the pack) */
 	if (item >= 0)
 	{
-		inven_item_increase(item, -1);
+		inven_item_increase(item, -num);
 		inven_item_optimize(item);
 	}
 
 	/* Decrease the item (from the floor) */
 	else
 	{
-		floor_item_increase(0 - item, -1);
+		floor_item_increase(0 - item, -num);
 		floor_item_optimize(0 - item);
 	}
 
 	/* Get the wield slot */
-	o_ptr = &inventory[slot];
+	o_ptr = &p_ptr->inventory[slot];
 
-	/* Take off existing item */
-	if (o_ptr->k_idx)
+	if (combined_ammo)
+	{
+		/* Add the new ammo to the already-quiver-ed ammo */
+		object_absorb(o_ptr, i_ptr);
+	}
+	else 
 	{
 		/* Take off existing item */
-		(void)inven_takeoff(slot, 255);
+		if (o_ptr->k_idx)
+			(void)inven_takeoff(slot, 255);
+
+		/* If we are wielding ammo we may need to "open" the slot by shifting
+		 * later ammo up the quiver; this is because we already called the
+		 * inven_item_optimize() function. */
+		if (slot >= QUIVER_START)
+			open_quiver_slot(slot);
+	
+		/* Wear the new stuff */
+		object_copy(o_ptr, i_ptr);
+
+		/* Increment the equip counter by hand */
+		p_ptr->equip_cnt++;
 	}
 
-	/* Wear the new stuff */
-	object_copy(o_ptr, i_ptr);
-
 	/* Increase the weight */
-	p_ptr->total_weight += i_ptr->weight;
-
-	/* Increment the equip counter by hand */
-	p_ptr->equip_cnt++;
+	p_ptr->total_weight += i_ptr->weight * num;
 
 	/* Do any ID-on-wield */
 	object_notice_on_wield(o_ptr);
 
 	/* Where is the item now */
 	if (slot == INVEN_WIELD)
-		act = "You are wielding";
+		fmt = "You are wielding %s (%c).";
 	else if (slot == INVEN_BOW)
-		act = "You are shooting with";
-	else if (slot == INVEN_LITE)
-		act = "Your light source is";
+		fmt = "You are shooting with %s (%c).";
+	else if (slot == INVEN_LIGHT)
+		fmt = "Your light source is %s (%c).";
+	else if (combined_ammo)
+		fmt = "You combine %s in your quiver (%c).";
+	else if (slot >= QUIVER_START && slot < QUIVER_END)
+		fmt = "You add %s to your quiver (%c).";
 	else
-		act = "You are wearing";
+		fmt = "You are wearing %s (%c).";
 
 	/* Describe the result */
-	object_desc(o_name, sizeof(o_name), o_ptr, TRUE, ODESC_FULL);
+	object_desc(o_name, sizeof(o_name), o_ptr, ODESC_PREFIX | ODESC_FULL);
 
 	/* Message */
-	sound(MSG_WIELD);
-	msg_format("%s %s (%c).", act, o_name, index_to_label(slot));
+	message_format(MSG_WIELD, 0, fmt, o_name, index_to_label(slot));
 
 	/* Cursed! */
 	if (cursed_p(o_ptr))
 	{
 		/* Warn the player */
-		sound(MSG_CURSED);
-		msg_print("Oops! It feels deathly cold!");
+		message_format(MSG_CURSED, 0, "Oops! It feels deathly cold!");
 
 		/* Sense the object */
 		object_notice_curses(o_ptr);
 	}
 
+	/* Save quiver size */
+	save_quiver_size(p_ptr);
+
+	/* See if we have to overflow the pack */
+	pack_overflow();
+
 	/* Recalculate bonuses, torch, mana */
+	p_ptr->notice |= PN_SORT_QUIVER;
 	p_ptr->update |= (PU_BONUS | PU_TORCH | PU_MANA);
 	p_ptr->redraw |= (PR_INVEN | PR_EQUIP);
 }
@@ -231,7 +264,8 @@ void do_cmd_destroy(cmd_code code, cmd_arg args[])
 
 	/* Describe the destroyed object by taking a copy with the right "amt" */
 	object_copy_amt(&destroyed_obj, o_ptr, amt);
-	object_desc(o_name, sizeof(o_name), &destroyed_obj, TRUE, ODESC_FULL);
+	object_desc(o_name, sizeof(o_name), &destroyed_obj,
+				ODESC_PREFIX | ODESC_FULL);
 
 	/* Artifacts cannot be destroyed */
 	if (artifact_p(o_ptr))
@@ -282,6 +316,7 @@ void textui_cmd_destroy(void)
 
 	object_type obj_to_destroy;
 
+	char result;
 	char o_name[120];
 	char out_val[160];
 
@@ -290,23 +325,19 @@ void textui_cmd_destroy(void)
 	/* Get an item */
 	q = "Destroy which item? ";
 	s = "You have nothing to destroy.";
-	if (!get_item(&item, q, s, (USE_INVEN | USE_EQUIP | USE_FLOOR | CAN_SQUELCH))) return;
+	if (!get_item(&item, q, s, CMD_DESTROY, (USE_INVEN | USE_EQUIP | USE_FLOOR | CAN_SQUELCH))) return;
 
 	/* Deal with squelched items */
 	if (item == ALL_SQUELCHED)
 	{
-		cmd_insert(CMD_DESTROY, item, 0);
+		cmd_insert(CMD_DESTROY);
+		cmd_set_arg_item(cmd_get_top(), 0, item);
 		return;
 	}
 	
 	o_ptr = object_from_item_idx(item);
 
 	/* Ask if player would prefer squelching instead of destruction */
-	if (squelch_interactive(o_ptr))
-	{
-		p_ptr->notice |= PN_SQUELCH;
-		return;
-	}
 
 	/* Get a quantity */
 	amt = get_quantity(NULL, o_ptr->number);
@@ -314,14 +345,29 @@ void textui_cmd_destroy(void)
 
 	/* Describe the destroyed object by taking a copy with the right "amt" */
 	object_copy_amt(&obj_to_destroy, o_ptr, amt);
-	object_desc(o_name, sizeof(o_name), &obj_to_destroy, TRUE, ODESC_FULL);
+	object_desc(o_name, sizeof(o_name), &obj_to_destroy,
+				ODESC_PREFIX | ODESC_FULL);
 
 	/* Verify destruction */
 	strnfmt(out_val, sizeof(out_val), "Really destroy %s? ", o_name);
-	if (!get_check(out_val)) return;
+	
+	result = get_char(out_val, "yns", 3, 'n');
 
-	/* Tell the game to destroy the item. */
-	cmd_insert(CMD_DESTROY, item, amt);
+	if (result == 'y')
+	{
+		cmd_insert(CMD_DESTROY);
+		cmd_set_arg_item(cmd_get_top(), 0, item);
+		cmd_set_arg_number(cmd_get_top(), 1, amt);
+	}
+	else if (result == 's' && squelch_interactive(o_ptr))
+	{
+		p_ptr->notice |= PN_SQUELCH;
+
+		/* If the item is not equipped, we can rely on it being dropped and */
+		/* ignored, otherwise we should continue on to check if we should */
+		/* still destroy it. */
+		if (item < INVEN_WIELD) return;
+	}
 }
 
 void refill_lamp(object_type *j_ptr, object_type *o_ptr, int item)
@@ -340,7 +386,7 @@ void refill_lamp(object_type *j_ptr, object_type *o_ptr, int item)
 	}
 
 	/* Refilled from a lantern */
-	if (o_ptr->sval == SV_LITE_LANTERN)
+	if (o_ptr->sval == SV_LIGHT_LANTERN)
 	{
 		/* Unstack if necessary */
 		if (o_ptr->number > 1)
@@ -366,9 +412,9 @@ void refill_lamp(object_type *j_ptr, object_type *o_ptr, int item)
 
 			/* Carry or drop */
 			if (item >= 0)
-				item = inven_carry(i_ptr);
+				item = inven_carry(p_ptr, i_ptr);
 			else
-				drop_near(i_ptr, 0, p_ptr->py, p_ptr->px);
+				drop_near(i_ptr, 0, p_ptr->py, p_ptr->px, FALSE);
 		}
 
 		/* Empty a single lantern */
@@ -415,12 +461,26 @@ void refill_lamp(object_type *j_ptr, object_type *o_ptr, int item)
 
 void refuel_torch(object_type *j_ptr, object_type *o_ptr, int item)
 {
+	bitflag f[OF_SIZE];
+	bitflag g[OF_SIZE];
+
 	/* Refuel */
 	j_ptr->timeout += o_ptr->timeout + 5;
 
 	/* Message */
 	msg_print("You combine the torches.");
 
+	/* Transfer the LIGHT flag if refuelling from a torch with it to one
+	   without it */
+	object_flags(o_ptr, f);
+	object_flags(j_ptr, g);
+	if (of_has(f, OF_LIGHT) && !of_has(g, OF_LIGHT))
+	{
+		of_on(j_ptr->flags, OF_LIGHT);
+		if (!j_ptr->name2) j_ptr->name2 = EGO_BRIGHTNESS;
+		msg_print("Your torch shines further!");
+	}
+
 	/* Over-fuel message */
 	if (j_ptr->timeout >= FUEL_TORCH)
 	{
@@ -467,20 +527,18 @@ void refuel_torch(object_type *j_ptr, object_type *o_ptr, int item)
  */
 void do_cmd_target(void)
 {
-	/* Target set */
 	if (target_set_interactive(TARGET_KILL, -1, -1))
-	{
 		msg_print("Target Selected.");
-	}
-
-	/* Target aborted */
 	else
-	{
 		msg_print("Target Aborted.");
-	}
 }
 
 
+void do_cmd_target_closest(void)
+{
+	target_set_closest(TARGET_KILL);
+}
+
 
 /*
  * Look command
@@ -686,103 +744,54 @@ static cptr ident_info[] =
 	NULL
 };
 
-
-
-/*
- * Sorting hook -- Comp function -- see below
- *
- * We use "u" to point to array of monster indexes,
- * and "v" to select the type of sorting to perform on "u".
- */
-bool ang_sort_comp_hook(const void *u, const void *v, int a, int b)
+static int cmp_mexp(const void *a, const void *b)
 {
-	const u16b *who = (const u16b *)(u);
-	const u16b *why = (const u16b *)(v);
-
-	int w1 = who[a];
-	int w2 = who[b];
-
-	int z1, z2;
-
-
-	/* Sort by player kills */
-	if (*why >= 4)
-	{
-		/* Extract player kills */
-		z1 = l_list[w1].pkills;
-		z2 = l_list[w2].pkills;
-
-		/* Compare player kills */
-		if (z1 < z2) return (TRUE);
-		if (z1 > z2) return (FALSE);
-	}
-
-
-	/* Sort by total kills */
-	if (*why >= 3)
-	{
-		/* Extract total kills */
-		z1 = l_list[w1].tkills;
-		z2 = l_list[w2].tkills;
-
-		/* Compare total kills */
-		if (z1 < z2) return (TRUE);
-		if (z1 > z2) return (FALSE);
-	}
-
-
-	/* Sort by monster level */
-	if (*why >= 2)
-	{
-		/* Extract levels */
-		z1 = r_info[w1].level;
-		z2 = r_info[w2].level;
-
-		/* Compare levels */
-		if (z1 < z2) return (TRUE);
-		if (z1 > z2) return (FALSE);
-	}
-
-
-	/* Sort by monster experience */
-	if (*why >= 1)
-	{
-		/* Extract experience */
-		z1 = r_info[w1].mexp;
-		z2 = r_info[w2].mexp;
-
-		/* Compare experience */
-		if (z1 < z2) return (TRUE);
-		if (z1 > z2) return (FALSE);
-	}
-
-
-	/* Compare indexes */
-	return (w1 <= w2);
+	u16b ia = *(const u16b *)a;
+	u16b ib = *(const u16b *)b;
+	if (r_info[ia].mexp < r_info[ib].mexp)
+		return -1;
+	if (r_info[ia].mexp > r_info[ib].mexp)
+		return 1;
+	return (a < b ? -1 : (a > b ? 1 : 0));
 }
 
-
-/*
- * Sorting hook -- Swap function -- see below
- *
- * We use "u" to point to array of monster indexes,
- * and "v" to select the type of sorting to perform.
- */
-void ang_sort_swap_hook(void *u, void *v, int a, int b)
+static int cmp_level(const void *a, const void *b)
 {
-	u16b *who = (u16b*)(u);
-
-	u16b holder;
-
-	/* Unused parameter */
-	(void)v;
-
-	/* Swap */
-	holder = who[a];
-	who[a] = who[b];
-	who[b] = holder;
+	u16b ia = *(const u16b *)a;
+	u16b ib = *(const u16b *)b;
+	if (r_info[ia].level < r_info[ib].level)
+		return -1;
+	if (r_info[ia].level > r_info[ib].level)
+		return 1;
+	return cmp_mexp(a, b);
 }
 
+static int cmp_tkill(const void *a, const void *b)
+{
+	u16b ia = *(const u16b *)a;
+	u16b ib = *(const u16b *)b;
+	if (l_list[ia].tkills < l_list[ib].tkills)
+		return -1;
+	if (l_list[ia].tkills > l_list[ib].tkills)
+		return 1;
+	return cmp_level(a, b);
+}
+
+static int cmp_pkill(const void *a, const void *b)
+{
+	u16b ia = *(const u16b *)a;
+	u16b ib = *(const u16b *)b;
+	if (l_list[ia].pkills < l_list[ib].pkills)
+		return -1;
+	if (l_list[ia].pkills > l_list[ib].pkills)
+		return 1;
+	return cmp_tkill(a, b);
+}
+
+int cmp_monsters(const void *a, const void *b)
+{
+	return cmp_level(a, b);
+}
 
 /*
  * Identify a character, allow recall of monsters
@@ -810,7 +819,6 @@ void do_cmd_query_symbol(void)
 
 	bool recall = FALSE;
 
-	u16b why = 0;
 	u16b *who;
 
 
@@ -865,10 +873,10 @@ void do_cmd_query_symbol(void)
 		if (!OPT(cheat_know) && !l_ptr->sights) continue;
 
 		/* Require non-unique monsters if needed */
-		if (norm && (r_ptr->flags[0] & (RF0_UNIQUE))) continue;
+		if (norm && rf_has(r_ptr->flags, RF_UNIQUE)) continue;
 
 		/* Require unique monsters if needed */
-		if (uniq && !(r_ptr->flags[0] & (RF0_UNIQUE))) continue;
+		if (uniq && !rf_has(r_ptr->flags, RF_UNIQUE)) continue;
 
 		/* Collect "appropriate" monsters */
 		if (all || (r_ptr->d_char == sym)) who[n++] = i;
@@ -909,12 +917,12 @@ void do_cmd_query_symbol(void)
 	if (query.key == 'k')
 	{
 		/* Sort by kills (and level) */
-		why = 4;
+		sort(who, n, sizeof(*who), cmp_pkill);
 	}
 	else if (query.key == 'y' || query.key == 'p')
 	{
 		/* Sort by level; accept 'p' as legacy */
-		why = 2;
+		sort(who, n, sizeof(*who), cmp_level);
 	}
 	else
 	{
@@ -926,14 +934,6 @@ void do_cmd_query_symbol(void)
 		return;
 	}
 
-
-	/* Select the sort method */
-	ang_sort_comp = ang_sort_comp_hook;
-	ang_sort_swap = ang_sort_swap_hook;
-
-	/* Sort the array */
-	ang_sort(who, &why, n);
-
 	/* Start at the end */
 	i = n - 1;
 
@@ -1024,3 +1024,9 @@ void do_cmd_query_symbol(void)
 	/* Free the "who" array */
 	FREE(who);
 }
+
+/* Centers the map on the player */
+void do_cmd_center_map(void)
+{
+	center_panel();
+}
