@@ -9,40 +9,26 @@
  */
 typedef void (*cmd_handler_fn)(cmd_code code, cmd_arg args[]);
 
-/* cmd.c */
-extern void cmd_init(void);
-
 /* cmd0.c */
-void do_cmd_quit(cmd_code code, cmd_arg args[]);
+extern void cmd_init(void);
+unsigned char cmd_lookup_key(cmd_code cmd);
+cmd_code cmd_lookup(unsigned char key);
 
 /* cmd-obj.c */
 void do_cmd_uninscribe(cmd_code code, cmd_arg args[]);
+void do_cmd_inscribe(cmd_code code, cmd_arg args[]);
 void do_cmd_takeoff(cmd_code code, cmd_arg args[]);
 void do_cmd_wield(cmd_code code, cmd_arg args[]);
 void do_cmd_drop(cmd_code code, cmd_arg args[]);
 void do_cmd_use(cmd_code code, cmd_arg args[]);
 void do_cmd_refill(cmd_code code, cmd_arg args[]);
-void do_cmd_inscribe(cmd_code code, cmd_arg args[]);
-
-void textui_cmd_uninscribe(void);
-void textui_cmd_inscribe(void);
-void do_cmd_observe(void);
-void textui_cmd_takeoff(void);
-void textui_cmd_wield(void);
-void textui_cmd_drop(void);
-void do_cmd_browse(void);
+void do_cmd_study_spell(cmd_code code, cmd_arg args[]);
 void do_cmd_cast(cmd_code code, cmd_arg args[]);
-void textui_cmd_study(void);
-void textui_cmd_cast(void);
-void textui_cmd_pray(void);
-void textui_cmd_refill(void);
-void textui_cmd_use_staff(void);
-void textui_cmd_aim_wand(void);
-void textui_cmd_zap_rod(void);
-void textui_cmd_activate(void);
-void textui_cmd_eat_food(void);
-void textui_cmd_quaff_potion(void);
-void textui_cmd_read_scroll(void);
+void do_cmd_study_book(cmd_code code, cmd_arg args[]);
+
+void textui_obj_inscribe(object_type *o_ptr, int item);
+void textui_obj_examine(object_type *o_ptr, int item);
+void textui_obj_wield(object_type *o_ptr, int item);
 
 /* cmd2.c */
 void do_cmd_go_up(cmd_code code, cmd_arg args[]);
@@ -67,34 +53,24 @@ void do_cmd_suicide(cmd_code code, cmd_arg args[]);
 void do_cmd_save_game(cmd_code code, cmd_arg args[]);
 
 void do_cmd_alter_aux(int dir);
-void textui_cmd_open(void);
-void textui_cmd_close(void);
-void textui_cmd_tunnel(void);
-void textui_cmd_disarm(void);
-void textui_cmd_bash(void);
-void textui_cmd_alter(void);
-void textui_cmd_spike(void);
-void textui_cmd_walk(void);
-void textui_cmd_jump(void);
-void textui_cmd_run(void);
 void textui_cmd_rest(void);
 void textui_cmd_suicide(void);
 
-
 /* cmd3.c */
 void do_cmd_inven(void);
 void do_cmd_equip(void);
 void wield_item(object_type *o_ptr, int item, int slot);
+void wield_combine_ammo(object_type *o_ptr, int item, int slot);
 void do_cmd_destroy(cmd_code code, cmd_arg args[]);
 void textui_cmd_destroy(void);
 void refill_lamp(object_type *j_ptr, object_type *o_ptr, int item);
 void refuel_torch(object_type *j_ptr, object_type *o_ptr, int item);
 void do_cmd_target(void);
+void do_cmd_target_closest(void);
 void do_cmd_look(void);
 void do_cmd_locate(void);
-bool ang_sort_comp_hook(const void *u, const void *v, int a, int b);
-void ang_sort_swap_hook(void *u, void *v, int a, int b);
 void do_cmd_query_symbol(void);
+void do_cmd_center_map(void);
 
 /* cmd4.c */
 extern void do_cmd_redraw(void);
@@ -104,28 +80,26 @@ extern void do_cmd_message_one(void);
 extern void do_cmd_messages(void);
 extern void do_cmd_options(void);
 extern void do_cmd_pref(void);
-extern void do_cmd_macros(void);
-extern void do_cmd_visuals(void);
-extern void do_cmd_colors(void);
 extern void do_cmd_note(void);
 extern void do_cmd_version(void);
 extern void do_cmd_feeling(void);
 extern void do_cmd_load_screen(void);
 extern void do_cmd_save_screen(void);
-extern void do_cmd_knowledge(void);
-extern void init_cmd4_c(void);
 
-/* cmd5.c */
-void do_cmd_study_spell(cmd_code code, cmd_arg args[]);
-void do_cmd_study_book(cmd_code code, cmd_arg args[]);
-
-/* cmd-know.c */
-extern void do_cmd_knowledge_objects(void *obj, const char *name);
-extern void init_cmd_know(void);
+/* cmd-misc.c */
+void do_cmd_wizard(void);
+void do_cmd_try_debug(void);
+void do_cmd_try_borg(void);
+void do_cmd_quit(cmd_code code, cmd_arg args[]);
+void do_cmd_port(void);
+void do_cmd_xxx_options(void);
+void do_cmd_monlist(void);
+void do_cmd_itemlist(void);
+void do_cmd_unknown(void);
 
 /* attack.c */
 extern void do_cmd_fire(cmd_code code, cmd_arg args[]);
-extern void textui_cmd_fire(void);
+extern void textui_cmd_fire_at_nearest(void);
 extern void do_cmd_throw(cmd_code code, cmd_arg args[]);
 extern void textui_cmd_throw(void);
 
@@ -144,6 +118,22 @@ typedef enum
 	USE_SINGLE
 } use_type;
 
+/* XXX */
+extern int cmp_monsters(const void *a, const void *b);
+
+
+
+/* ui-spell.c -- just for now */
+void textui_book_browse(const object_type *o_ptr);
+void textui_spell_browse(void);
+void textui_obj_study(void);
+void textui_obj_cast(void);
+
+/* ui-knowledge.c */
+extern void big_pad(int col, int row, byte a, byte c);
+extern void textui_browse_object_knowledge(void *obj, const char *name);
+extern void textui_knowledge_init(void);
+extern void textui_browse_knowledge(void);
 
 #endif
 
