@@ -15,11 +15,20 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-#include "angband.h"
-#include "object/tvalsval.h"
-#include "cmds.h"
-#include "game-event.h"
 
+#include "angband.h"
+#include "button.h"
+#include "cave.h"
+#include "cmds.h"
+#include "files.h"
+#include "game-event.h"
+#include "generate.h"
+#include "init.h"
+#include "monster/monster.h"
+#include "object/tvalsval.h"
+#include "prefs.h"
+#include "spells.h"
+#include "target.h"
 
 /*
  * Change dungeon level - e.g. by going up stairs or with WoR.
@@ -28,7 +37,47 @@ void dungeon_change_level(int dlev)
 {
 	/* New depth */
 	p_ptr->depth = dlev;
-	
+
+	/* If we're returning to town, update the store contents
+	   according to how long we've been away */
+	if (!dlev && daycount)
+	{
+		if (OPT(cheat_xtra)) msg_print("Updating Shops...");
+		while (daycount--)
+		{
+			int n;
+
+			/* Maintain each shop (except home) */
+			for (n = 0; n < MAX_STORES; n++)
+			{
+				/* Skip the home */
+				if (n == STORE_HOME) continue;
+
+				/* Maintain */
+				store_maint(n);
+			}
+
+			/* Sometimes, shuffle the shop-keepers */
+			if (one_in_(STORE_SHUFFLE))
+			{
+				/* Message */
+				if (OPT(cheat_xtra)) msg_print("Shuffling a Shopkeeper...");
+
+				/* Pick a random shop (except home) */
+				while (1)
+				{
+					n = randint0(MAX_STORES);
+					if (n != STORE_HOME) break;
+				}
+
+				/* Shuffle it */
+				store_shuffle(n);
+			}
+		}
+		daycount = 0;
+		if (OPT(cheat_xtra)) msg_print("Done.");
+	}
+
 	/* Leaving */
 	p_ptr->leaving = TRUE;
 
@@ -77,8 +126,8 @@ static void regenhp(int percent)
 	{
 		/* Redraw */
 		p_ptr->redraw |= (PR_HP);
-		wieldeds_notice_flag(2, TR2_REGEN);
-		wieldeds_notice_flag(2, TR2_IMPAIR_HP);
+		wieldeds_notice_flag(OF_REGEN);
+		wieldeds_notice_flag(OF_IMPAIR_HP);
 	}
 }
 
@@ -122,8 +171,8 @@ static void regenmana(int percent)
 	{
 		/* Redraw */
 		p_ptr->redraw |= (PR_MANA);
-		wieldeds_notice_flag(2, TR2_REGEN);
-		wieldeds_notice_flag(2, TR2_IMPAIR_MANA);
+		wieldeds_notice_flag(OF_REGEN);
+		wieldeds_notice_flag(OF_IMPAIR_MANA);
 	}
 }
 
@@ -161,7 +210,7 @@ static void regen_monsters(void)
 			if (!frac) frac = 1;
 
 			/* Hack -- Some monsters regenerate quickly */
-			if (r_ptr->flags[1] & (RF1_REGENERATE)) frac *= 2;
+			if (rf_has(r_ptr->flags, RF_REGENERATE)) frac *= 2;
 
 			/* Hack -- Regenerate */
 			m_ptr->hp += frac;
@@ -180,6 +229,7 @@ static void regen_monsters(void)
  * If player has inscribed the object with "!!", let him know when it's
  * recharged. -LM-
  * Also inform player when first item of a stack has recharged. -HK-
+ * Notify all recharges w/o inscription if notify_recharge option set -WP-
  */
 static void recharged_notice(const object_type *o_ptr, bool all)
 {
@@ -187,47 +237,56 @@ static void recharged_notice(const object_type *o_ptr, bool all)
 
 	cptr s;
 
-	/* No inscription */
-	if (!o_ptr->note) return;
+	bool notify = FALSE;
 
-	/* Find a '!' */
-	s = strchr(quark_str(o_ptr->note), '!');
-
-	/* Process notification request */
-	while (s)
+	if (OPT(notify_recharge))
 	{
-		/* Find another '!' */
-		if (s[1] == '!')
-		{
-			/* Describe (briefly) */
-			object_desc(o_name, sizeof(o_name), o_ptr, FALSE, ODESC_BASE);
-
-			/* Disturb the player */
-			disturb(0, 0);
-
-			/* Notify the player */
-			if (o_ptr->number > 1)
-			{
-				if (all) msg_format("Your %s have recharged.", o_name);
-				else msg_format("One of your %s has recharged.", o_name);
-			}
-
-			/* Artifacts */
-			else if (o_ptr->name1)
-			{
-				msg_format("The %s has recharged.", o_name);
-			}
-
-			/* Single, non-artifact items */
-			else msg_format("Your %s has recharged.", o_name);
-
-			/* Done */
-			return;
-		}
-
-		/* Keep looking for '!'s */
-		s = strchr(s + 1, '!');
+		notify = TRUE;
 	}
+	else if (o_ptr->note)
+	{
+		/* Find a '!' */
+		s = strchr(quark_str(o_ptr->note), '!');
+
+		/* Process notification request */
+		while (s)
+		{
+			/* Find another '!' */
+			if (s[1] == '!')
+			{
+				notify = TRUE;
+				break;
+			}
+
+			/* Keep looking for '!'s */
+			s = strchr(s + 1, '!');
+		}
+	}
+
+	if (!notify) return;
+
+
+	/* Describe (briefly) */
+	object_desc(o_name, sizeof(o_name), o_ptr, ODESC_BASE);
+
+	/* Disturb the player */
+	disturb(0, 0);
+
+	/* Notify the player */
+	if (o_ptr->number > 1)
+	{
+		if (all) msg_format("Your %s have recharged.", o_name);
+		else msg_format("One of your %s has recharged.", o_name);
+	}
+
+	/* Artifacts */
+	else if (o_ptr->name1)
+	{
+		msg_format("The %s has recharged.", o_name);
+	}
+
+	/* Single, non-artifact items */
+	else msg_format("Your %s has recharged.", o_name);
 }
 
 
@@ -239,32 +298,27 @@ static void recharge_objects(void)
 {
 	int i;
 
-	int charged = 0;
+	bool charged = FALSE, discharged_stack;
 
 	object_type *o_ptr;
 	object_kind *k_ptr;
 
-
-	/* Process equipment */
+	/*** Recharge equipment ***/
 	for (i = INVEN_WIELD; i < INVEN_TOTAL; i++)
 	{
 		/* Get the object */
-		o_ptr = &inventory[i];
+		o_ptr = &p_ptr->inventory[i];
 
 		/* Skip non-objects */
 		if (!o_ptr->k_idx) continue;
 
 		/* Recharge activatable objects */
-		if (o_ptr->timeout > 0 && !(o_ptr->tval == TV_LITE && !artifact_p(o_ptr)))
+		if (recharge_timeout(o_ptr))
 		{
-			/* Recharge */
-			o_ptr->timeout--;
+			charged = TRUE;
 
-			/* Notice changes */
-			if (!o_ptr->timeout) charged++;
-
-			/* Message if item is recharged, if inscribed with "!!" */
-			if (!o_ptr->timeout) recharged_notice(o_ptr, TRUE);
+			/* Message if an item recharged */
+			recharged_notice(o_ptr, TRUE);
 		}
 	}
 
@@ -275,40 +329,34 @@ static void recharge_objects(void)
 		p_ptr->redraw |= (PR_EQUIP);
 	}
 
-	charged = 0;
+	charged = FALSE;
 
-	/* Recharge rods */
+	/*** Recharge the inventory ***/
 	for (i = 0; i < INVEN_PACK; i++)
 	{
-		o_ptr = &inventory[i];
+		o_ptr = &p_ptr->inventory[i];
 		k_ptr = &k_info[o_ptr->k_idx];
 
 		/* Skip non-objects */
 		if (!o_ptr->k_idx) continue;
 
-		/* Examine all charging rods */
-		if ((o_ptr->tval == TV_ROD) && (o_ptr->timeout))
+		discharged_stack = (number_charging(o_ptr) == o_ptr->number) ? TRUE : FALSE;
+
+		/* Recharge rods, and update if any rods are recharged */
+		if (o_ptr->tval == TV_ROD && recharge_timeout(o_ptr))
 		{
-			/* Determine how many rods are charging */
-			int temp = (o_ptr->timeout + (k_ptr->time_base - 1)) / k_ptr->time_base;
-			if (temp > o_ptr->number) temp = o_ptr->number;
+			charged = TRUE;
 
-			/* Decrease timeout by that number */
-			o_ptr->timeout -= temp;
-
-			/* Boundary control */
-			if (o_ptr->timeout < 0) o_ptr->timeout = 0;
-
-			/* Update if any rods are recharged */
-			if (temp > (o_ptr->timeout + (k_ptr->time_base - 1)) / k_ptr->time_base)
+			/* Entire stack is recharged */
+			if (o_ptr->timeout == 0)
 			{
-				/* Update window */
-				charged++;
+				recharged_notice(o_ptr, TRUE);
+			}
 
-				/* Message if whole stack is recharged, if inscribed with "!!" */
-				if (!o_ptr->timeout) recharged_notice(o_ptr, TRUE);
-				/* Message if first in a stack is recharged, if inscribed with "!!" -HK- */
-				else if (temp == o_ptr->number) recharged_notice(o_ptr, FALSE);
+			/* Previously exhausted stack has acquired a charge */
+			else if (discharged_stack)
+			{
+				recharged_notice(o_ptr, FALSE);
 			}
 		}
 	}
@@ -323,10 +371,7 @@ static void recharge_objects(void)
 		p_ptr->redraw |= (PR_INVEN);
 	}
 
-
-	/*** Process Objects ***/
-
-	/* Process objects */
+	/*** Recharge the ground ***/
 	for (i = 1; i < o_max; i++)
 	{
 		/* Get the object */
@@ -336,14 +381,8 @@ static void recharge_objects(void)
 		if (!o_ptr->k_idx) continue;
 
 		/* Recharge rods on the ground */
-		if ((o_ptr->tval == TV_ROD) && o_ptr->timeout)
-		{
-			/* Charge it */
-			o_ptr->timeout -= o_ptr->number;
-
-			/* Boundary control */
-			if (o_ptr->timeout < 0) o_ptr->timeout = 0;
-		}
+		if (o_ptr->tval == TV_ROD)
+			recharge_timeout(o_ptr);
 	}
 }
 
@@ -386,13 +425,13 @@ static void play_ambient_sound(void)
 	}
 
 	/* Dungeon level 61-80 */
-	else if (p_ptr->depth <= 80) 
+	else if (p_ptr->depth <= 80)
 	{
 		sound(MSG_AMBIENT_DNG4);
 	}
 
 	/* Dungeon level 80- */
-	else  
+	else
 	{
 		sound(MSG_AMBIENT_DNG5);
 	}
@@ -490,46 +529,10 @@ static void process_world(void)
 	/* While in the dungeon */
 	else
 	{
-		/*** Update the Stores ***/
-
-		/* Update the stores once a day (while in dungeon) */
-		if (!(turn % (10L * STORE_TURNS)))
-		{
-			int n;
-
-			/* Message */
-			if (OPT(cheat_xtra)) msg_print("Updating Shops...");
-
-			/* Maintain each shop (except home) */
-			for (n = 0; n < MAX_STORES; n++)
-			{
-				/* Skip the home */
-				if (n == STORE_HOME) continue;
-
-				/* Maintain */
-				store_maint(n);
-			}
-
-			/* Sometimes, shuffle the shop-keepers */
-			if (one_in_(STORE_SHUFFLE))
-			{
-				/* Message */
-				if (OPT(cheat_xtra)) msg_print("Shuffling a Shopkeeper...");
-
-				/* Pick a random shop (except home) */
-				while (1)
-				{
-					n = randint0(MAX_STORES);
-					if (n != STORE_HOME) break;
-				}
-
-				/* Shuffle it */
-				store_shuffle(n);
-			}
-
-			/* Message */
-			if (OPT(cheat_xtra)) msg_print("Done.");
-		}
+		/* Update the stores once a day (while in the dungeon).
+		   The changes are not actually made until return to town,
+		   to avoid giving details away in the knowledge menu. */
+		if (!(turn % (10L * STORE_TURNS))) daycount++;
 	}
 
 
@@ -696,12 +699,12 @@ static void process_world(void)
 	/*** Process Light ***/
 
 	/* Check for light being wielded */
-	o_ptr = &inventory[INVEN_LITE];
+	o_ptr = &p_ptr->inventory[INVEN_LIGHT];
 
-	/* Burn some fuel in the current lite */
-	if (o_ptr->tval == TV_LITE)
+	/* Burn some fuel in the current light */
+	if (o_ptr->tval == TV_LIGHT)
 	{
-		u32b f[OBJ_FLAG_N];
+		bitflag f[OF_SIZE];
 		bool burn_fuel = TRUE;
 
 		/* Get the object flags */
@@ -712,7 +715,7 @@ static void process_world(void)
 			burn_fuel = FALSE;
 
 		/* If the light has the NO_FUEL flag, well... */
-		if (f[2] & TR2_NO_FUEL)
+		if (of_has(f, OF_NO_FUEL))
 		    burn_fuel = FALSE;
 
 		/* Use some fuel (except on artifacts, or during the day) */
@@ -767,7 +770,7 @@ static void process_world(void)
 			check_experience();
 		}
 
-		wieldeds_notice_flag(2, TR2_DRAIN_EXP);
+		wieldeds_notice_flag(OF_DRAIN_EXP);
 	}
 
 	/* Recharge activatable objects and rods */
@@ -782,7 +785,7 @@ static void process_world(void)
 	/* Random teleportation */
 	if (p_ptr->state.teleport && one_in_(100))
 	{
-		wieldeds_notice_flag(2, TR2_TELEPORT);
+		wieldeds_notice_flag(OF_TELEPORT);
 		teleport_player(40);
 		disturb(0, 0);
 	}
@@ -799,18 +802,15 @@ static void process_world(void)
 			/* Disturbing! */
 			disturb(0, 0);
 
-			/* Sound */
-			sound(MSG_TPLEVEL);
-
 			/* Determine the level */
 			if (p_ptr->depth)
 			{
-				msg_print("You feel yourself yanked upwards!");
+				message_format(MSG_TPLEVEL, 0, "You feel yourself yanked upwards!");
 				dungeon_change_level(0);
 			}
 			else
 			{
-				msg_print("You feel yourself yanked downwards!");
+				message_format(MSG_TPLEVEL, 0, "You feel yourself yanked downwards!");
 
 				/* New depth - back to max depth or 1, whichever is deeper */
 				dungeon_change_level(p_ptr->max_depth < 1 ? 1: p_ptr->max_depth);
@@ -834,16 +834,14 @@ static void process_player_aux(void)
 	bool changed = FALSE;
 
 	static int old_monster_race_idx = 0;
-
-	static u32b old_flags[RACE_FLAG_STRICT_UB] = { 0L, 0L, 0L };
-	static u32b old_spell_flags[RACE_FLAG_SPELL_STRICT_UB] = { 0L, 0L, 0L };
+	static bitflag old_flags[RF_SIZE];
+	static bitflag old_spell_flags[RSF_SIZE];
 
 	static byte old_blows[MONSTER_BLOW_MAX];
 
 	static byte	old_cast_innate = 0;
 	static byte	old_cast_spell = 0;
 
-
 	/* Tracking a monster */
 	if (p_ptr->monster_race_idx)
 	{
@@ -862,8 +860,8 @@ static void process_player_aux(void)
 		/* Check for change of any kind */
 		if (changed ||
 		    (old_monster_race_idx != p_ptr->monster_race_idx) ||
-		    race_flags_differ(old_flags, l_ptr->flags) ||
-		    race_flags_differ_spell(old_spell_flags, l_ptr->spell_flags) ||
+		    !rf_is_equal(old_flags, l_ptr->flags) ||
+		    !rsf_is_equal(old_spell_flags, l_ptr->spell_flags) ||
 		    (old_cast_innate != l_ptr->cast_innate) ||
 		    (old_cast_spell != l_ptr->cast_spell))
 		{
@@ -871,8 +869,8 @@ static void process_player_aux(void)
 			old_monster_race_idx = p_ptr->monster_race_idx;
 
 			/* Memorize flags */
-			race_flags_assign(old_flags, l_ptr->flags);
-			race_flags_assign_spell(old_spell_flags, l_ptr->spell_flags);
+			rf_copy(old_flags, l_ptr->flags);
+			rsf_copy(old_spell_flags, l_ptr->spell_flags);
 
 			/* Memorize blows */
 			memmove(old_blows, l_ptr->blows, sizeof(byte)*MONSTER_BLOW_MAX);
@@ -911,14 +909,13 @@ static void process_player(void)
 {
 	int i;
 
-
 	/*** Check for interrupts ***/
 
 	/* Complete resting */
 	if (p_ptr->resting < 0)
 	{
 		/* Basic resting */
-		if (p_ptr->resting == -1)
+		if (p_ptr->resting == REST_ALL_POINTS)
 		{
 			/* Stop resting */
 			if ((p_ptr->chp == p_ptr->mhp) &&
@@ -929,7 +926,7 @@ static void process_player(void)
 		}
 
 		/* Complete resting */
-		else if (p_ptr->resting == -2)
+		else if (p_ptr->resting == REST_COMPLETE)
 		{
 			/* Stop resting */
 			if ((p_ptr->chp == p_ptr->mhp) &&
@@ -946,7 +943,7 @@ static void process_player(void)
 		}
 		
 		/* Rest until HP or SP are filled */
-		else if (p_ptr->resting == -3)
+		else if (p_ptr->resting == REST_SOME_POINTS)
 		{
 			/* Stop resting */
 			if ((p_ptr->chp == p_ptr->mhp) ||
@@ -959,7 +956,7 @@ static void process_player(void)
 
 	/* Check for "player abort" */
 	if (p_ptr->running ||
-	    p_ptr->command_rep ||
+	    cmd_get_nrepeats() > 0 ||
 	    (p_ptr->resting && !(turn & 0x7F)))
 	{
 		/* Do not wait */
@@ -1001,57 +998,25 @@ static void process_player(void)
 		/* Refresh (optional) */
 		Term_fresh();
 
-
 		/* Hack -- Pack Overflow */
-		if (inventory[INVEN_PACK].k_idx)
-		{
-			int item = INVEN_PACK;
-
-			char o_name[80];
-
-			object_type *o_ptr;
-
-			/* Get the slot to be dropped */
-			o_ptr = &inventory[item];
-
-			/* Disturbing */
-			disturb(0, 0);
-
-			/* Warning */
-			msg_print("Your pack overflows!");
-
-			/* Describe */
-			object_desc(o_name, sizeof(o_name), o_ptr, TRUE, ODESC_FULL);
-
-			/* Message */
-			msg_format("You drop %s (%c).", o_name, index_to_label(item));
-
-			/* Drop it (carefully) near the player */
-			drop_near(o_ptr, 0, p_ptr->py, p_ptr->px);
-
-			/* Modify, Describe, Optimize */
-			inven_item_increase(item, -255);
-			inven_item_describe(item);
-			inven_item_optimize(item);
-
-			/* Notice stuff (if needed) */
-			if (p_ptr->notice) notice_stuff();
-
-			/* Update stuff (if needed) */
-			if (p_ptr->update) update_stuff();
-
-			/* Redraw stuff (if needed) */
-			if (p_ptr->redraw) redraw_stuff();
-		}
-
-
-		/* Hack -- reset to inventory display */
-		if (!p_ptr->command_new) p_ptr->command_wrk = USE_INVEN;
-
+		pack_overflow();
 
 		/* Assume free turn */
 		p_ptr->energy_use = 0;
 
+		/* Dwarves detect treasure */
+		if (player_has(PF_SEE_ORE))
+		{
+			/* Only if they are in good shape */
+			if (!p_ptr->timed[TMD_IMAGE] &&
+					!p_ptr->timed[TMD_CONFUSED] &&
+					!p_ptr->timed[TMD_AMNESIA] &&
+					!p_ptr->timed[TMD_STUN] &&
+					!p_ptr->timed[TMD_PARALYZED] &&
+					!p_ptr->timed[TMD_TERROR] &&
+					!p_ptr->timed[TMD_AFRAID])
+				detect_close_buried_treasure();
+		}
 
 		/* Paralyzed or Knocked Out */
 		if ((p_ptr->timed[TMD_PARALYZED]) || (p_ptr->timed[TMD_STUN] >= 100))
@@ -1085,6 +1050,9 @@ static void process_player(void)
 
 			/* Take a turn */
 			p_ptr->energy_use = 100;
+
+			/* Increment the resting counter */
+			p_ptr->resting_turn++;
 		}
 
 		/* Running */
@@ -1095,7 +1063,7 @@ static void process_player(void)
 		}
 
 		/* Repeated command */
-		else if (p_ptr->command_rep)
+		else if (cmd_get_nrepeats() > 0)
 		{
 			/* Hack -- Assume messages were seen */
 			msg_flag = FALSE;
@@ -1105,16 +1073,6 @@ static void process_player(void)
 
 			/* Process the command */
 			process_command(CMD_GAME, TRUE);
-
-			/* Count this execution */
-			if (p_ptr->command_rep)
-			{
-				/* Count this execution */
-				p_ptr->command_rep--;
-
-				/* Redraw the state */
-				p_ptr->redraw |= (PR_STATE);
-			}
 		}
 
 		/* Normal command */
@@ -1128,6 +1086,10 @@ static void process_player(void)
 
 			/* Get and process a command */
 			process_command(CMD_GAME, FALSE);
+
+			/* Mega hack - redraw if big graphics - sorry NRM */
+			if ((tile_width > 1) || (tile_height > 1)) 
+			        p_ptr->redraw |= (PR_MAP);
 		}
 
 
@@ -1139,6 +1101,8 @@ static void process_player(void)
 			/* Use some energy */
 			p_ptr->energy -= p_ptr->energy_use;
 
+			/* Increment the total energy counter */
+			p_ptr->total_energy += p_ptr->energy_use;
 
 			/* Hack -- constant hallucination */
 			if (p_ptr->timed[TMD_IMAGE])
@@ -1168,13 +1132,13 @@ static void process_player(void)
 					r_ptr = &r_info[m_ptr->r_idx];
 
 					/* Skip non-multi-hued monsters */
-					if (!(r_ptr->flags[0] & (RF0_ATTR_MULTI))) continue;
+					if (!rf_has(r_ptr->flags, RF_ATTR_MULTI)) continue;
 
 					/* Reset the flag */
 					shimmer_monsters = TRUE;
 
 					/* Redraw regardless */
-					lite_spot(m_ptr->fy, m_ptr->fx);
+					light_spot(m_ptr->fy, m_ptr->fx);
 				}
 			}
 
@@ -1269,6 +1233,95 @@ static void process_player(void)
 	while (!p_ptr->energy_use && !p_ptr->leaving);
 }
 
+byte flicker = 0;
+byte color_flicker[MAX_COLORS][3] = 
+{
+	{TERM_DARK, TERM_L_DARK, TERM_L_RED},
+	{TERM_WHITE, TERM_L_WHITE, TERM_L_BLUE},
+	{TERM_SLATE, TERM_WHITE, TERM_L_DARK},
+	{TERM_ORANGE, TERM_YELLOW, TERM_L_RED},
+	{TERM_RED, TERM_L_RED, TERM_L_PINK},
+	{TERM_GREEN, TERM_L_GREEN, TERM_L_TEAL},
+	{TERM_BLUE, TERM_L_BLUE, TERM_SLATE},
+	{TERM_UMBER, TERM_L_UMBER, TERM_MUSTARD},
+	{TERM_L_DARK, TERM_SLATE, TERM_L_VIOLET},
+	{TERM_WHITE, TERM_SLATE, TERM_L_WHITE},
+	{TERM_L_PURPLE, TERM_PURPLE, TERM_L_VIOLET},
+	{TERM_YELLOW, TERM_L_YELLOW, TERM_MUSTARD},
+	{TERM_L_RED, TERM_RED, TERM_L_PINK},
+	{TERM_L_GREEN, TERM_L_TEAL, TERM_GREEN},
+	{TERM_L_BLUE, TERM_DEEP_L_BLUE, TERM_BLUE_SLATE},
+	{TERM_L_UMBER, TERM_UMBER, TERM_MUD},
+	{TERM_PURPLE, TERM_VIOLET, TERM_MAGENTA},
+	{TERM_VIOLET, TERM_L_VIOLET, TERM_MAGENTA},
+	{TERM_TEAL, TERM_L_TEAL, TERM_L_GREEN},
+	{TERM_MUD, TERM_YELLOW, TERM_UMBER},
+	{TERM_L_YELLOW, TERM_WHITE, TERM_L_UMBER},
+	{TERM_MAGENTA, TERM_L_PINK, TERM_L_RED},
+	{TERM_L_TEAL, TERM_L_WHITE, TERM_TEAL},
+	{TERM_L_VIOLET, TERM_L_PURPLE, TERM_VIOLET},
+	{TERM_L_PINK, TERM_L_RED, TERM_L_WHITE},
+	{TERM_MUSTARD, TERM_YELLOW, TERM_UMBER},
+	{TERM_BLUE_SLATE, TERM_BLUE, TERM_SLATE},
+	{TERM_DEEP_L_BLUE, TERM_L_BLUE, TERM_BLUE},
+};
+
+byte get_flicker(byte a)
+{
+	switch(flicker % 3)
+	{
+		case 1: return color_flicker[a][1];
+		case 2: return color_flicker[a][2];
+	}
+	return a;
+}
+
+/*
+ * This animates monsters and/or items as necessary.
+ */
+void do_animation(void)
+{
+	int i;
+
+	for (i = 1; i < mon_max; i++)
+	{
+		byte attr;
+		monster_type *m_ptr = &mon_list[i];
+		monster_race *r_ptr = &r_info[m_ptr->r_idx];
+
+		if (!m_ptr || !m_ptr->ml)
+			continue;
+		else if (rf_has(r_ptr->flags, RF_ATTR_MULTI))
+			attr = randint1(BASIC_COLORS - 1);
+		else if (rf_has(r_ptr->flags, RF_ATTR_FLICKER))
+			attr = get_flicker(r_ptr->x_attr);
+		else
+			continue;
+
+		m_ptr->attr = attr;
+		p_ptr->redraw |= (PR_MAP | PR_MONLIST);
+	}
+	flicker++;
+}
+
+
+/*
+ * This is used when the user is idle to allow for simple animations.
+ * Currently the only thing it really does is animate shimmering monsters.
+ */
+void idle_update(void)
+{
+	if (!character_dungeon) return;
+
+	if (!OPT(animate_flicker)) return;
+
+	/* Animate and redraw if necessary */
+	do_animation();
+	redraw_stuff();
+
+	/* Refresh the main screen */
+	Term_fresh();
+}
 
 
 /*
@@ -1294,11 +1347,7 @@ static void dungeon(void)
 
 
 	/* Reset the "command" vars */
-	p_ptr->command_cmd = 0;
-	p_ptr->command_new = 0;
-	p_ptr->command_rep = 0;
 	p_ptr->command_arg = 0;
-	p_ptr->command_dir = 0;
 
 
 	/* Cancel the target */
@@ -1396,7 +1445,7 @@ static void dungeon(void)
 	p_ptr->update |= (PU_BONUS | PU_HP | PU_MANA | PU_SPELLS);
 
 	/* Combine / Reorder the pack */
-	p_ptr->notice |= (PN_COMBINE | PN_REORDER);
+	p_ptr->notice |= (PN_COMBINE | PN_REORDER | PN_SORT_QUIVER);
 
 	/* Make basic mouse buttons */
 	(void) button_add("[ESC]", ESCAPE);
@@ -1426,8 +1475,9 @@ static void dungeon(void)
 	/* Announce (or repeat) the feeling */
 	if (p_ptr->depth) do_cmd_feeling();
 
-	/* Player gets to go first */
-	p_ptr->energy = 100;
+	/* Give player minimum energy to start a new level, but do not reduce higher value from savefile for level in progress */
+	if (p_ptr->energy < INITIAL_DUNGEON_ENERGY)
+		p_ptr->energy = INITIAL_DUNGEON_ENERGY;
 
 
 	/*** Process this dungeon level ***/
@@ -1448,16 +1498,22 @@ static void dungeon(void)
 		/* Hack -- Compress the object list occasionally */
 		if (o_cnt + 32 < o_max) compact_objects(0);
 
-
 		/* Can the player move? */
 		while ((p_ptr->energy >= 100) && !p_ptr->leaving)
 		{
+    		/* Do any necessary animations */
+    		do_animation(); 
+
 			/* process monster with even more energy first */
 			process_monsters((byte)(p_ptr->energy + 1));
 
 			/* if still alive */
 			if (!p_ptr->leaving)
 			{
+			        /* Mega hack -redraw big graphics - sorry NRM */
+			        if ((tile_width > 1) || (tile_height > 1)) 
+				        p_ptr->redraw |= (PR_MAP);
+
 				/* Process the player */
 				process_player();
 			}
@@ -1472,7 +1528,7 @@ static void dungeon(void)
 		/* Redraw stuff */
 		if (p_ptr->redraw) redraw_stuff();
 
-		/* Hack -- Hilite the player */
+		/* Hack -- Highlight the player */
 		move_cursor_relative(p_ptr->py, p_ptr->px);
 
 		/* Handle "leaving" */
@@ -1491,7 +1547,7 @@ static void dungeon(void)
 		/* Redraw stuff */
 		if (p_ptr->redraw) redraw_stuff();
 
-		/* Hack -- Hilite the player */
+		/* Hack -- Highlight the player */
 		move_cursor_relative(p_ptr->py, p_ptr->px);
 
 		/* Handle "leaving" */
@@ -1510,13 +1566,12 @@ static void dungeon(void)
 		/* Redraw stuff */
 		if (p_ptr->redraw) redraw_stuff();
 
-		/* Hack -- Hilite the player */
+		/* Hack -- Highlight the player */
 		move_cursor_relative(p_ptr->py, p_ptr->px);
 
 		/* Handle "leaving" */
 		if (p_ptr->leaving) break;
 
-
 		/*** Apply energy ***/
 
 		/* Give the player some energy */
@@ -1551,13 +1606,13 @@ static void process_some_user_pref_files(void)
 
 
 	/* Process the "user.prf" file */
-	(void)process_pref_file("user.prf");
+	(void)process_pref_file("user.prf", TRUE);
 
 	/* Get the "PLAYER.prf" filename */
 	(void)strnfmt(buf, sizeof(buf), "%s.prf", op_ptr->base_name);
 
 	/* Process the "PLAYER.prf" file */
-	(void)process_pref_file(buf);
+	(void)process_pref_file(buf, TRUE);
 }
 
 
@@ -1648,9 +1703,6 @@ void play_game(void)
 
 		/* The dungeon is not ready */
 		character_dungeon = FALSE;
-
-		/* XXX This is the place to add automatic character
-		   numbering (i.e. Rocky IV, V, etc.) Probably. */
 	}
 
 	/* Hack -- Default base_name */
@@ -1734,6 +1786,8 @@ void play_game(void)
 	/* Flash a message */
 	prt("Please wait...", 0, 0);
 
+	/* Allow big cursor */
+	smlcurs = FALSE;
 
 	/* Flush the message */
 	Term_fresh();
@@ -1891,6 +1945,9 @@ void play_game(void)
 		generate_cave();
 	}
 
+	/* Disallow big cursor */
+	smlcurs = TRUE;
+
 	/* Tell the UI we're done with the game state */
 	event_signal(EVENT_LEAVE_GAME);
 
