@@ -15,9 +15,13 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-#include "angband.h"
-#include "object/tvalsval.h"
 
+#include "angband.h"
+#include "cave.h"
+#include "files.h"
+#include "monster/monster.h"
+#include "object/tvalsval.h"
+#include "trap.h"
 
 /*
  * Note that Level generation is *not* an important bottleneck,
@@ -176,10 +180,27 @@
 
 
 /*
- * Maximal number of room types
+ * Maximum number of room types
  */
-#define ROOM_MAX	9
+#define ROOM_MAX	10
 
+/*
+ * Treasure allocation probabilities for nests
+ */
+#define JELLY_NEST_OBJ 30
+#define ANIMAL_NEST_OBJ 10
+#define UNDEAD_NEST_OBJ 5
+
+
+/*
+ * Height and width for the currently generated level
+ *
+ * This differs from DUNGEON_HGT (and dungeon_hgt) in that it bounds the part
+ * of the level that might actually contain open squares. It will vary from
+ * level to level, unlike the constant.
+ */
+int level_hgt  = DUNGEON_HGT;
+int level_wid  = DUNGEON_WID;
 
 
 /*
@@ -266,7 +287,8 @@ static const room_data room[ROOM_MAX] =
 	{ 0, 0, -1, 1, 5 },		/* 5 = Monster nest (33x11) */
 	{ 0, 0, -1, 1, 5 },		/* 6 = Monster pit (33x11) */
 	{ 0, 1, -1, 1, 5 },		/* 7 = Lesser vault (33x22) */
-	{ -1, 2, -2, 3, 10 }	/* 8 = Greater vault (66x44) */
+	{ 0, 1, -1, 1, 5 },		/* 8 = Medium vault (33x22) */
+	{ -1, 2, -2, 3, 10 }	/* 9 = Greater vault (66x44) */
 };
 
 
@@ -277,21 +299,17 @@ static const room_data room[ROOM_MAX] =
 static void correct_dir(int *rdir, int *cdir, int y1, int x1, int y2, int x2)
 {
 	/* Extract vertical and horizontal directions */
-	*rdir = (y1 == y2) ? 0 : (y1 < y2) ? 1 : -1;
-	*cdir = (x1 == x2) ? 0 : (x1 < x2) ? 1 : -1;
+	*rdir = CMP(y2, y1);
+	*cdir = CMP(x2, x1);
 
-	/* Never move diagonally */
-	if (*rdir && *cdir)
-	{
-		if (randint0(100) < 50)
-		{
-			*rdir = 0;
-		}
-		else
-		{
-			*cdir = 0;
-		}
-	}
+	/* If we only have one direction to go, then we're done */
+	if (!*rdir || !*cdir) return;
+
+	/* If we need to go diagonally, then choose a random direction */
+	if (randint0(100) < 50)
+		*rdir = 0;
+	else
+		*cdir = 0;
 }
 
 
@@ -320,8 +338,8 @@ static void new_player_spot(void)
 	while (1)
 	{
 		/* Pick a legal spot */
-		y = rand_range(1, DUNGEON_HGT - 2);
-		x = rand_range(1, DUNGEON_WID - 2);
+		y = rand_range(1, level_hgt - 2);
+		x = rand_range(1, level_wid - 2);
 
 		/* Must be a "naked" floor grid */
 		if (!cave_naked_bold(y, x)) continue;
@@ -331,11 +349,13 @@ static void new_player_spot(void)
 
 		if (!OPT(adult_no_stairs))
 		{
-			if (p_ptr->create_down_stair) {
+			if (p_ptr->create_down_stair)
+			{
 				cave_set_feat(y, x, FEAT_MORE);
 				p_ptr->create_down_stair = FALSE;
 			}
-			else if (p_ptr->create_up_stair) {
+			else if (p_ptr->create_up_stair)
+			{
 				cave_set_feat(y, x, FEAT_LESS);
 				p_ptr->create_up_stair = FALSE;
 			}
@@ -604,8 +624,8 @@ static void alloc_stairs(int feat, int num, int walls)
 			for (j = 0; !flag && j <= 3000; j++)
 			{
 				/* Pick a random grid */
-				y = randint0(DUNGEON_HGT);
-				x = randint0(DUNGEON_WID);
+				y = randint0(level_hgt);
+				x = randint0(level_wid);
 
 				/* Require "naked" floor grid */
 				if (!cave_naked_bold(y, x)) continue;
@@ -651,19 +671,22 @@ static void alloc_stairs(int feat, int num, int walls)
  */
 static void alloc_object(int set, int typ, int num, int depth)
 {
-	int y, x, k;
+	int y, x, k, tries;
+	bool room;
 
 	/* Place some objects */
 	for (k = 0; k < num; k++)
 	{
+		tries = 0;
+
 		/* Pick a "legal" spot */
-		while (TRUE)
+		while (tries < 10000)
 		{
-			bool room;
+			tries++;
 
 			/* Location */
-			y = randint0(DUNGEON_HGT);
-			x = randint0(DUNGEON_WID);
+			y = randint0(level_hgt);
+			x = randint0(level_wid);
 
 			/* Require "naked" floor grid */
 			if (!cave_naked_bold(y, x)) continue;
@@ -772,91 +795,6 @@ static void build_streamer(int feat, int chance)
 }
 
 
-/*
- * Build a destroyed level
- */
-static void destroy_level(void)
-{
-	int y1, x1, y, x, k, t, n;
-
-
-	/* Note destroyed levels */
-	if (OPT(cheat_room)) msg_print("Destroyed Level");
-
-	/* Drop a few epi-centers (usually about two) */
-	for (n = 0; n < randint1(5); n++)
-	{
-		/* Pick an epi-center */
-		x1 = rand_range(5, DUNGEON_WID-1 - 5);
-		y1 = rand_range(5, DUNGEON_HGT-1 - 5);
-
-		/* Big area of affect */
-		for (y = (y1 - 15); y <= (y1 + 15); y++)
-		{
-			for (x = (x1 - 15); x <= (x1 + 15); x++)
-			{
-				/* Skip illegal grids */
-				if (!in_bounds_fully(y, x)) continue;
-
-				/* Extract the distance */
-				k = distance(y1, x1, y, x);
-
-				/* Stay in the circle of death */
-				if (k >= 16) continue;
-
-				/* Delete the monster (if any) */
-				delete_monster(y, x);
-
-				/* Destroy valid grids */
-				if (cave_valid_bold(y, x))
-				{
-					/* Delete objects */
-					delete_object(y, x);
-
-					/* Wall (or floor) type */
-					t = randint0(200);
-
-					/* Granite */
-					if (t < 20)
-					{
-						/* Create granite wall */
-						cave_set_feat(y, x, FEAT_WALL_EXTRA);
-					}
-
-					/* Quartz */
-					else if (t < 70)
-					{
-						/* Create quartz vein */
-						cave_set_feat(y, x, FEAT_QUARTZ);
-					}
-
-					/* Magma */
-					else if (t < 100)
-					{
-						/* Create magma vein */
-						cave_set_feat(y, x, FEAT_MAGMA);
-					}
-
-					/* Floor */
-					else
-					{
-						/* Create floor */
-						cave_set_feat(y, x, FEAT_FLOOR);
-					}
-
-					/* No longer part of a room or vault */
-					cave_info[y][x] &= ~(CAVE_ROOM | CAVE_ICKY);
-
-					/* No longer illuminated */
-					cave_info[y][x] &= ~(CAVE_GLOW);
-				}
-			}
-		}
-	}
-}
-
-
-
 /*
  * Create up to "num" objects near the given coordinates
  * Only really called by some of the "vault" routines.
@@ -1646,7 +1584,7 @@ static bool vault_aux_jelly(int r_idx)
 	monster_race *r_ptr = &r_info[r_idx];
 
 	/* Decline unique monsters */
-	if (r_ptr->flags[0] & (RF0_UNIQUE)) return (FALSE);
+	if (rf_has(r_ptr->flags, RF_UNIQUE)) return (FALSE);
 
 	/* Require icky thing, jelly, mold, or mushroom */
 	if (!strchr("ijm,", r_ptr->d_char)) return (FALSE);
@@ -1664,10 +1602,10 @@ static bool vault_aux_animal(int r_idx)
 	monster_race *r_ptr = &r_info[r_idx];
 
 	/* Decline unique monsters */
-	if (r_ptr->flags[0] & (RF0_UNIQUE)) return (FALSE);
+	if (rf_has(r_ptr->flags, RF_UNIQUE)) return (FALSE);
 
 	/* Require "animal" flag */
-	if (!(r_ptr->flags[2] & (RF2_ANIMAL))) return (FALSE);
+	if (!rf_has(r_ptr->flags, RF_ANIMAL)) return (FALSE);
 
 	/* Okay */
 	return (TRUE);
@@ -1682,10 +1620,10 @@ static bool vault_aux_undead(int r_idx)
 	monster_race *r_ptr = &r_info[r_idx];
 
 	/* Decline unique monsters */
-	if (r_ptr->flags[0] & (RF0_UNIQUE)) return (FALSE);
+	if (rf_has(r_ptr->flags, RF_UNIQUE)) return (FALSE);
 
 	/* Require Undead */
-	if (!(r_ptr->flags[2] & (RF2_UNDEAD))) return (FALSE);
+	if (!rf_has(r_ptr->flags, RF_UNDEAD)) return (FALSE);
 
 	/* Okay */
 	return (TRUE);
@@ -1700,7 +1638,7 @@ static bool vault_aux_orc(int r_idx)
 	monster_race *r_ptr = &r_info[r_idx];
 
 	/* Decline unique monsters */
-	if (r_ptr->flags[0] & (RF0_UNIQUE)) return (FALSE);
+	if (rf_has(r_ptr->flags, RF_UNIQUE)) return (FALSE);
 
 	/* Hack -- Require "o" monsters */
 	if (!strchr("o", r_ptr->d_char)) return (FALSE);
@@ -1718,7 +1656,7 @@ static bool vault_aux_troll(int r_idx)
 	monster_race *r_ptr = &r_info[r_idx];
 
 	/* Decline unique monsters */
-	if (r_ptr->flags[0] & (RF0_UNIQUE)) return (FALSE);
+	if (rf_has(r_ptr->flags, RF_UNIQUE)) return (FALSE);
 
 	/* Hack -- Require "T" monsters */
 	if (!strchr("T", r_ptr->d_char)) return (FALSE);
@@ -1736,7 +1674,7 @@ static bool vault_aux_giant(int r_idx)
 	monster_race *r_ptr = &r_info[r_idx];
 
 	/* Decline unique monsters */
-	if (r_ptr->flags[0] & (RF0_UNIQUE)) return (FALSE);
+	if (rf_has(r_ptr->flags, RF_UNIQUE)) return (FALSE);
 
 	/* Hack -- Require "P" monsters */
 	if (!strchr("P", r_ptr->d_char)) return (FALSE);
@@ -1749,7 +1687,7 @@ static bool vault_aux_giant(int r_idx)
 /*
  * Hack -- breath type for "vault_aux_dragon()"
  */
-static u32b vault_aux_dragon_mask0;
+static bitflag vault_aux_dragon_mask[RSF_SIZE];
 
 
 /*
@@ -1758,15 +1696,19 @@ static u32b vault_aux_dragon_mask0;
 static bool vault_aux_dragon(int r_idx)
 {
 	monster_race *r_ptr = &r_info[r_idx];
+	bitflag mon_breath[RSF_SIZE];
 
 	/* Decline unique monsters */
-	if (r_ptr->flags[0] & (RF0_UNIQUE)) return (FALSE);
+	if (rf_has(r_ptr->flags, RF_UNIQUE)) return (FALSE);
 
 	/* Hack -- Require "d" or "D" monsters */
 	if (!strchr("Dd", r_ptr->d_char)) return (FALSE);
 
 	/* Hack -- Require correct "breath attack" */
-	if (r_ptr->spell_flags[0] != vault_aux_dragon_mask0) return (FALSE);
+	rsf_copy(mon_breath, r_ptr->spell_flags);
+	flags_mask(mon_breath, RSF_SIZE, RSF_BREATH_MASK, FLAG_END);
+
+	if (!rsf_is_equal(mon_breath, vault_aux_dragon_mask)) return (FALSE);
 
 	/* Okay */
 	return (TRUE);
@@ -1781,7 +1723,7 @@ static bool vault_aux_demon(int r_idx)
 	monster_race *r_ptr = &r_info[r_idx];
 
 	/* Decline unique monsters */
-	if (r_ptr->flags[0] & (RF0_UNIQUE)) return (FALSE);
+	if (rf_has(r_ptr->flags, RF_UNIQUE)) return (FALSE);
 
 	/* Hack -- Require "U" monsters */
 	if (!strchr("U", r_ptr->d_char)) return (FALSE);
@@ -1822,6 +1764,8 @@ static void build_type5(int y0, int x0)
 
 	int tmp, i;
 
+	int alloc_obj;
+
 	s16b what[64];
 
 	cptr name;
@@ -1872,6 +1816,9 @@ static void build_type5(int y0, int x0)
 
 		/* Restrict to jelly */
 		get_mon_num_hook = vault_aux_jelly;
+
+		/* Get treasure probability */
+		alloc_obj = JELLY_NEST_OBJ;
 	}
 
 	/* Monster nest (animal) */
@@ -1882,6 +1829,9 @@ static void build_type5(int y0, int x0)
 
 		/* Restrict to animal */
 		get_mon_num_hook = vault_aux_animal;
+
+		/* Get treasure probability */
+		alloc_obj = ANIMAL_NEST_OBJ;
 	}
 
 	/* Monster nest (undead) */
@@ -1892,6 +1842,9 @@ static void build_type5(int y0, int x0)
 
 		/* Restrict to undead */
 		get_mon_num_hook = vault_aux_undead;
+
+		/* Get treasure probability */
+		alloc_obj = UNDEAD_NEST_OBJ;
 	}
 
 	/* Prepare allocation table */
@@ -1948,6 +1901,10 @@ static void build_type5(int y0, int x0)
 
 			/* Place that "random" monster (no groups) */
 			(void)place_monster_aux(y, x, r_idx, FALSE, FALSE);
+
+			/* Occasionally place an item, making it good 1/3 of the time */
+			if (one_in_(alloc_obj)) 
+				place_object(y, x, p_ptr->depth + 10, one_in_(3), FALSE);
 		}
 	}
 }
@@ -2086,7 +2043,7 @@ static void build_type6(int y0, int x0)
 				name = "acid dragon";
 
 				/* Restrict dragon breath type */
-				vault_aux_dragon_mask0 = RSF0_BR_ACID;
+				flags_init(vault_aux_dragon_mask, RSF_SIZE, RSF_BR_ACID, FLAG_END);
 
 				/* Done */
 				break;
@@ -2099,7 +2056,7 @@ static void build_type6(int y0, int x0)
 				name = "electric dragon";
 
 				/* Restrict dragon breath type */
-				vault_aux_dragon_mask0 = RSF0_BR_ELEC;
+				flags_init(vault_aux_dragon_mask, RSF_SIZE, RSF_BR_ELEC, FLAG_END);
 
 				/* Done */
 				break;
@@ -2112,7 +2069,7 @@ static void build_type6(int y0, int x0)
 				name = "fire dragon";
 
 				/* Restrict dragon breath type */
-				vault_aux_dragon_mask0 = RSF0_BR_FIRE;
+				flags_init(vault_aux_dragon_mask, RSF_SIZE, RSF_BR_FIRE, FLAG_END);
 
 				/* Done */
 				break;
@@ -2125,7 +2082,7 @@ static void build_type6(int y0, int x0)
 				name = "cold dragon";
 
 				/* Restrict dragon breath type */
-				vault_aux_dragon_mask0 = RSF0_BR_COLD;
+				flags_init(vault_aux_dragon_mask, RSF_SIZE, RSF_BR_COLD, FLAG_END);
 
 				/* Done */
 				break;
@@ -2138,7 +2095,7 @@ static void build_type6(int y0, int x0)
 				name = "poison dragon";
 
 				/* Restrict dragon breath type */
-				vault_aux_dragon_mask0 = RSF0_BR_POIS;
+				flags_init(vault_aux_dragon_mask, RSF_SIZE, RSF_BR_POIS, FLAG_END);
 
 				/* Done */
 				break;
@@ -2151,9 +2108,8 @@ static void build_type6(int y0, int x0)
 				name = "multi-hued dragon";
 
 				/* Restrict dragon breath type */
-				vault_aux_dragon_mask0 = (RSF0_BR_ACID | RSF0_BR_ELEC |
-				                          RSF0_BR_FIRE | RSF0_BR_COLD |
-				                          RSF0_BR_POIS);
+				flags_init(vault_aux_dragon_mask, RSF_SIZE, RSF_BR_ACID, RSF_BR_ELEC,
+				           RSF_BR_FIRE, RSF_BR_COLD, RSF_BR_POIS, FLAG_END);
 
 				/* Done */
 				break;
@@ -2467,7 +2423,7 @@ static void build_type7(int y0, int x0)
 	}
 
 	/* Message */
-	if (OPT(cheat_room)) msg_format("Lesser vault (%s)", v_name + v_ptr->name);
+	if (OPT(cheat_room)) msg_format("Lesser vault (%s)", v_ptr->name);
 
 	/* Boost the rating */
 	rating += v_ptr->rat;
@@ -2480,30 +2436,64 @@ static void build_type7(int y0, int x0)
 	}
 
 	/* Hack -- Build the vault */
-	build_vault(y0, x0, v_ptr->hgt, v_ptr->wid, v_text + v_ptr->text);
+	build_vault(y0, x0, v_ptr->hgt, v_ptr->wid, v_ptr->text);
 }
 
 
 
 /*
- * Type 8 -- greater vaults (see "vault.txt")
+ * Type 8 -- medium vaults (see "vault.txt")
  */
 static void build_type8(int y0, int x0)
 {
 	vault_type *v_ptr;
 
-	/* Pick a lesser vault */
+	/* Pick a medium vault */
+	while (TRUE)
+	{
+		/* Get a random vault record */
+		v_ptr = &v_info[randint0(z_info->v_max)];
+
+		/* Accept the first mdium vault */
+		if (v_ptr->typ == 7) break;
+	}
+
+	/* Message */
+	if (OPT(cheat_room)) msg_format("Medium vault (%s)", v_ptr->name);
+
+	/* Boost the rating */
+	rating += v_ptr->rat;
+
+	/* (Sometimes) Cause a special feeling */
+	if ((p_ptr->depth <= 50) ||
+	    (randint1((p_ptr->depth-40) * (p_ptr->depth-40) + 1) < 400))
+	{
+		good_item_flag = TRUE;
+	}
+
+	/* Hack -- Build the vault */
+	build_vault(y0, x0, v_ptr->hgt, v_ptr->wid, v_ptr->text);
+}
+
+/*
+ * Type 9 -- greater vaults (see "vault.txt")
+ */
+static void build_type9(int y0, int x0)
+{
+	vault_type *v_ptr;
+
+	/* Pick a greater vault */
 	while (TRUE)
 	{
 		/* Get a random vault record */
 		v_ptr = &v_info[randint0(z_info->v_max)];
 
 		/* Accept the first greater vault */
-		if (v_ptr->typ == 7) break;
+		if (v_ptr->typ == 8) break;
 	}
 
 	/* Message */
-	if (OPT(cheat_room)) msg_format("Greater vault (%s)", v_name + v_ptr->name);
+	if (OPT(cheat_room)) msg_format("Greater vault (%s)", v_ptr->name);
 
 	/* Boost the rating */
 	rating += v_ptr->rat;
@@ -2516,7 +2506,7 @@ static void build_type8(int y0, int x0)
 	}
 
 	/* Hack -- Build the vault */
-	build_vault(y0, x0, v_ptr->hgt, v_ptr->wid, v_text + v_ptr->text);
+	build_vault(y0, x0, v_ptr->hgt, v_ptr->wid, v_ptr->text);
 }
 
 
@@ -2916,6 +2906,7 @@ static bool room_build(int by0, int bx0, int typ)
 	switch (typ)
 	{
 		/* Build an appropriate room */
+		case 9: build_type9(y, x); break;
 		case 8: build_type8(y, x); break;
 		case 7: build_type7(y, x); break;
 		case 6: build_type6(y, x); break;
@@ -2961,50 +2952,50 @@ static bool room_build(int by0, int bx0, int typ)
  */
 static void cave_gen(void)
 {
-	int i, k, y, x, y1, x1;
-
+	int i, j, k, l, y, x, y1, x1;
 	int by, bx;
+	int num_rooms, size_percent;
 
-	bool destroyed = FALSE;
+	bool blocks_tried[MAX_ROOMS_ROW][MAX_ROOMS_COL];
 
 	dun_data dun_body;
 
+	/* Possibly generate fewer rooms in a smaller area via a scaling factor.
+	 * Since we scale row_rooms and col_rooms by the same amount, DUN_ROOMS
+	 * gives the same "room density" no matter what size the level turns out
+	 * to be. TODO: vary room density slightly? */
+	/* XXX: Until vault generation is improved, scaling variance is reduced */
+	i = randint1(10);
+	if (is_quest(p_ptr->depth)) size_percent = 100;
+	else if (i < 2) size_percent = 75;
+	else if (i < 3) size_percent = 80;
+	else if (i < 4) size_percent = 85;
+	else if (i < 5) size_percent = 90;
+	else if (i < 6) size_percent = 95;
+	else size_percent = 100;
+	size_percent = 100;
+
+	/* scale the various generation variables */
+	num_rooms = DUN_ROOMS * size_percent / 100;
+	level_hgt = DUNGEON_HGT * size_percent / 100;
+	level_wid  = DUNGEON_WID * size_percent / 100;
 
 	/* Global data */
 	dun = &dun_body;
 
-
 	/* Hack -- Start with basic granite */
 	for (y = 0; y < DUNGEON_HGT; y++)
-	{
 		for (x = 0; x < DUNGEON_WID; x++)
-		{
-			/* Create granite wall */
 			cave_set_feat(y, x, FEAT_WALL_EXTRA);
-		}
-	}
-
-
-	/* Possible "destroyed" level */
-	if ((p_ptr->depth > 10) && one_in_(DUN_DEST)) destroyed = TRUE;
-
-	/* Hack -- No destroyed "quest" levels */
-	if (is_quest(p_ptr->depth)) destroyed = FALSE;
-
 
 	/* Actual maximum number of rooms on this level */
-	dun->row_rooms = DUNGEON_HGT / BLOCK_HGT;
-	dun->col_rooms = DUNGEON_WID / BLOCK_WID;
+	dun->row_rooms = level_hgt / BLOCK_HGT;
+	dun->col_rooms = level_wid / BLOCK_WID;
 
 	/* Initialize the room table */
 	for (by = 0; by < dun->row_rooms; by++)
-	{
 		for (bx = 0; bx < dun->col_rooms; bx++)
-		{
-			dun->room_map[by][bx] = FALSE;
-		}
-	}
-
+			dun->room_map[by][bx] = blocks_tried[by][bx]  = FALSE;
 
 	/* No "crowded" rooms yet */
 	dun->crowded = FALSE;
@@ -3014,32 +3005,73 @@ static void cave_gen(void)
 	dun->cent_n = 0;
 
 	/* Build some rooms */
-	for (i = 0; i < DUN_ROOMS; i++)
+	i = 0;
+	while(i < num_rooms)
 	{
-		/* Pick a block for the room */
-		by = randint0(dun->row_rooms);
-		bx = randint0(dun->col_rooms);
+		i++;
 
-#if 0
-		/* Align dungeon rooms */
-		if (dungeon_align)
+		/* Pick a block for the room; j counts blocks we haven't tried */
+		j = 0;
+		for(by=0; by < dun->row_rooms; by++)
+			for(bx=0; bx < dun->col_rooms; bx++)
+				if (!blocks_tried[by][bx]) j++;
+
+		/* If we've tried all blocks we're done */
+		if (j == 0) break;
+
+		/* OK, choose one of the j blocks we haven't tried. Then figure out */
+		/* which one that actually was */
+		k = randint0(j);
+		l = 0;
+		for(by=0; by < dun->row_rooms; by++)
 		{
-			/* Slide some rooms right */
-			if ((bx % 3) == 0) bx++;
-
-			/* Slide some rooms left */
-			if ((bx % 3) == 2) bx--;
+			for(bx=0; bx < dun->col_rooms; bx++)
+			{
+				if (blocks_tried[by][bx]) continue;
+				if (l == k) break;
+				l++;
+			}
+			if (l == k) break;
 		}
-#endif
+		
+		blocks_tried[by][bx] = TRUE;
 
-		/* Destroyed levels are boring */
-		if (destroyed)
+		/* Move GV vault creation to the beginning */
+		/* There are two problems with GV creation, overlapping other rooms, 
+		 * and running off the boundaries.  Moving GV creation to the beginning 
+		 * automatically solves the overlapping problem, but makes
+		 * no claims that the block picked will fit on the map.  
+		 * The dungeon is 6 blocks tall and 18 wide and the size of a GV is 4 tall
+		 * and 6 wide.  That means a GV will fall out of bounds 63% of the time!
+		 * The following code should therefore make a greater vault frequencies like:
+		 * Dlevel         GV frequency
+		 * 100		18%
+		 * 90-99		16-18%
+		 * 80-89		10 -11%
+		 * 70-79		5.7 - 6.5%
+		 * 60-69		3.3 - 3.8%
+		 * 50-59		1.8 - 2.1%
+		 * and less than 1% below 50 */
+		
+		/* Only attempt a GV if you are on the first room */ 
+		if (i == 1 && randint0(DUN_UNUSUAL) < p_ptr->depth)
 		{
-			/* Attempt a "trivial" room */
-			if (room_build(by, bx, 1)) continue;
+			int i;
+			int numerator   = 2;
+			int denominator = 3;
 
-			/* Never mind */
-			continue;
+			/* For building greater vaults, we make a check based on depth:
+			 * At level 90 and above, you have a 2/3 chance of trying to build
+			 * a GV. At levels 80-89 you have a 4/9 chance, and so on... */
+			for(i = 90; i > p_ptr->depth; i -= 10)
+			{
+				numerator *= 2;
+				denominator *= 3;
+			}
+
+			/* Attempt to pass the depth check and build a GV */
+			if (randint0(denominator) < numerator && room_build(by, bx, 9))
+				continue;
 		}
 
 		/* Attempt an "unusual" room */
@@ -3051,7 +3083,7 @@ static void cave_gen(void)
 			/* Attempt a very unusual room */
 			if (randint0(DUN_UNUSUAL) < p_ptr->depth)
 			{
-				/* Type 8 -- Greater vault (10%) */
+				/* Type 8 -- Medium vault (10%) */
 				if ((k < 10) && room_build(by, bx, 8)) continue;
 
 				/* Type 7 -- Lesser vault (15%) */
@@ -3078,44 +3110,22 @@ static void cave_gen(void)
 		if (room_build(by, bx, 1)) continue;
 	}
 
-
-	/* Special boundary walls -- Top */
-	for (x = 0; x < DUNGEON_WID; x++)
-	{
-		y = 0;
-
-		/* Clear previous contents, add "solid" perma-wall */
-		cave_set_feat(y, x, FEAT_PERM_SOLID);
-	}
-
 	/* Special boundary walls -- Bottom */
 	for (x = 0; x < DUNGEON_WID; x++)
 	{
-		y = DUNGEON_HGT-1;
-
 		/* Clear previous contents, add "solid" perma-wall */
-		cave_set_feat(y, x, FEAT_PERM_SOLID);
+		cave_set_feat(0, x, FEAT_PERM_SOLID);
+		cave_set_feat(DUNGEON_HGT - 1, x, FEAT_PERM_SOLID);
 	}
 
 	/* Special boundary walls -- Left */
 	for (y = 0; y < DUNGEON_HGT; y++)
 	{
-		x = 0;
-
 		/* Clear previous contents, add "solid" perma-wall */
-		cave_set_feat(y, x, FEAT_PERM_SOLID);
+		cave_set_feat(y, 0, FEAT_PERM_SOLID);
+		cave_set_feat(y, DUNGEON_WID - 1, FEAT_PERM_SOLID);
 	}
 
-	/* Special boundary walls -- Right */
-	for (y = 0; y < DUNGEON_HGT; y++)
-	{
-		x = DUNGEON_WID-1;
-
-		/* Clear previous contents, add "solid" perma-wall */
-		cave_set_feat(y, x, FEAT_PERM_SOLID);
-	}
-
-
 	/* Hack -- Scramble the room order */
 	for (i = 0; i < dun->cent_n; i++)
 	{
@@ -3175,10 +3185,6 @@ static void cave_gen(void)
 	}
 
 
-	/* Destroy the level if necessary */
-	if (destroyed) destroy_level();
-
-
 	/* Place 3 or 4 down stairs near some walls */
 	alloc_stairs(FEAT_MORE, rand_range(3, 4), 3);
 
@@ -3218,17 +3224,17 @@ static void cave_gen(void)
 			monster_race *r_ptr = &r_info[i];
 
 			/* Ensure quest monsters */
-			if ((r_ptr->flags[0] & (RF0_QUESTOR)) &&
-			    (r_ptr->level == p_ptr->depth) &&
-			    (r_ptr->cur_num <= 0))
+			if (rf_has(r_ptr->flags, RF_QUESTOR) &&
+			    r_ptr->level == p_ptr->depth &&
+			    r_ptr->cur_num <= 0)
 			{
 				int y, x;
 
 				/* Pick a location */
 				while (1)
 				{
-					y = randint0(DUNGEON_HGT);
-					x = randint0(DUNGEON_WID);
+					y = randint0(level_hgt);
+					x = randint0(level_wid);
 
 					if (cave_naked_bold(y, x)) break;
 				}
@@ -3609,10 +3615,7 @@ static int calculate_feeling(int rating, int depth)
 void generate_cave(void)
 {
 	const char *error = "no generation";
-
-	/* The dungeon is not ready */
-	character_dungeon = FALSE;
-
+	int counter = 0;
 
 	/* Generate */
 	while (error)
@@ -3620,6 +3623,13 @@ void generate_cave(void)
 		error = NULL;
 		clear_cave();
 
+		/* The dungeon is not ready - we set this after calling clear_
+		   cave for the first time, so that unpreserved artifacts are
+		   lost when leaving real levels, but not when abandoning
+		   levels through errors in generation - see wipe_o_list in
+		   obj-util.c */
+		character_dungeon = FALSE;
+
 		if (!p_ptr->depth)
 			town_gen();
 		else
@@ -3641,15 +3651,18 @@ void generate_cave(void)
 
 		if (OPT(cheat_room) && error)
 			msg_format("Generation restarted: %s.", error);
-	}
 
+		counter++;
+		if (counter > 100)
+		{
+			msg_format("cave_gen() failed 100 times!");
+			exit_game_panic();
+		}
+	}
 
 	/* The dungeon is ready */
 	character_dungeon = TRUE;
 
-	/* Remember when this level was "created" */
-	old_turn = turn;
+	/* Remember when the last dungeon level was created */
+	if (p_ptr->depth > 0) old_turn = turn;
 }
-
-
-
