@@ -21,6 +21,20 @@
 #ifdef USE_GTK
 #include "main-gtk.h"
 #include "textui.h"
+#include "files.h"
+#include "macro.h"
+#include "init.h"
+
+/* this is used to draw the various terrain characters */
+static unsigned int graphics_table[32] = {
+	000, '*', '#', '?', '?', '?', '?', '.',
+	'+', '?', '?', '+', '+', '+', '+', '+',
+	'~', '-', '-', '-', '_', '+', '+', '+',
+	'+', '|', '?', '?', '?', '?', '?', '.',
+};
+
+iconv_t conv;
+
 /* 
  *Add a bunch of debugger message, to trace where problems are. 
  */
@@ -203,9 +217,9 @@ gboolean on_big_tiles(GtkWidget *widget, GdkEventButton *event, gpointer user_da
 	term_data *td = &data[0];
 	
 	if ((widget != NULL) && (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))))
-		use_bigtile = FALSE;
+		tile_width = 1;
 	else
-		use_bigtile = TRUE;
+		tile_width = 2;
 	
 	load_font_by_name(td, td->font.name);
 	term_data_redraw(td);
@@ -499,6 +513,52 @@ static errr Term_wipe_gtk(int x, int y, int n)
 	return (0);
 }
 
+static byte Term_xchar_gtk(byte c)
+{
+	/* Can't translate Latin-1 to UTF-8 here since we have to return a byte. */
+	return c;
+}
+
+char *process_control_chars(int n, cptr s)
+{
+	char *s2 = (char *)malloc(sizeof(char) * n);
+	int i;
+	for (i = 0; i < n; i++) {
+		unsigned char c = s[i];
+		if (c < 32) {
+			s2[i] = graphics_table[c];
+		} else if (c == 127) {
+			s2[i] = '#';
+		} else {
+			s2[i] = c;
+		}
+	}
+
+	return s2;
+}
+
+char *latin1_to_utf8(int n, cptr s)
+{
+	size_t inbytes = n;
+	char *s2 = process_control_chars(n, s);
+	char *p2 = s2;
+
+	size_t outbytes = 4 * n;
+	char *s3 = (char *)malloc(sizeof(char) * outbytes);
+	char *p3 = s3;
+
+	size_t result = iconv(conv, &p2, &inbytes, &p3, &outbytes);
+
+	if (result == (size_t)(-1)) {
+		printf("iconv() failed: %d\n", errno);
+		free(s3);
+		return s2;
+	} else {
+		free(s2);
+		return s3;
+	}
+}
+
 /*
  * Draw some textual characters.
  */
@@ -506,11 +566,18 @@ static errr Term_text_gtk(int x, int y, int n, byte a, cptr s)
 {
 	term_data *td = (term_data*)(Term->data);
 	cairo_rectangle_t r;
-	
+
+	char *s2;
+	if (conv == NULL)
+		s2 = process_control_chars(n, s);
+	else
+		s2 = latin1_to_utf8(n, s);
+
 	init_cairo_rect(&r, (td->font.w * x), (td->font.h * y),  (td->font.w * n), td->font.h);
-	draw_text(td->surface, &td->font, &td->actual, x, y, n, a, s);
+	draw_text(td->surface, &td->font, &td->actual, x, y, n, a, s2);
 	invalidate_drawing_area(td->drawing_area, r);
-			
+	
+	free(s2);
 	return (0);
 }
 
@@ -727,16 +794,8 @@ static void load_font_by_name(term_data *td, cptr font_name)
 	if (td->font.size == 0) td->font.size = 12;
 	get_font_size(&td->font);
 	
-	if (use_bigtile)
-	{
-		td->actual.w = td->font.w * 2;
-		td->actual.h = td->font.h;
-	}
-	else
-	{
-		td->actual.w = td->font.w;
-		td->actual.h = td->font.h;
-	}
+	td->actual.w = td->font.w * tile_width;
+	td->actual.h = td->font.h;
 	
 	td->size.w = td->cols * td->font.w;
 	td->size.h = td->rows * td->font.h;
@@ -1092,7 +1151,8 @@ static void save_prefs(void)
 	file_putf(fff, "[General Settings]\n");
 	/* Graphics setting */
 	file_putf(fff,"Tile set=%d\n", arg_graphics);
-	file_putf(fff,"Big Tiles=%d\n", use_bigtile);
+	file_putf(fff,"Tile Width=%d\n", tile_width);
+	file_putf(fff,"Tile Height=%d\n", tile_height);
 
 	/* New section */
 	file_putf(fff, "\n");
@@ -1227,11 +1287,11 @@ static void load_term_prefs()
 				arg_graphics = val;
 				continue;
 			}
-			if (prefix(buf, "Big Tiles="))
+			if (prefix(buf, "Tile Width="))
 			{
 				val = get_value(buf);
-				sscanf(buf, "Big Tiles=%d", &val);
-				use_bigtile = val;
+				sscanf(buf, "Tile Width=%d", &val);
+				tile_width = val;
 				continue;
 			}
 		}
@@ -1625,6 +1685,15 @@ static void init_graf(int g)
 			break;
 		}
 
+		case GRAPHICS_NOMAD:
+		{
+			ANGBAND_GRAF = "nomad";
+			path_build(buf, sizeof(buf), ANGBAND_DIR_XTRA_GRAF, "8x16.png");
+			use_transparency = TRUE;
+			td->tile.w = td->tile.h =16;
+			break;
+		}
+
 		case GRAPHICS_DAVID_GERVAIS:
 		{
 			ANGBAND_GRAF = "david";
@@ -1670,7 +1739,8 @@ static void setup_graphics_menu(GladeXML *xml)
 	char s[12];
 	int i;
 	
-	for (i = 0; i < 4; i++)
+	// FIXME: we should be using a numerical constant here
+	for (i = 0; i < 5; i++)
 	{
 		bool checked = (i == arg_graphics);
 
@@ -1937,7 +2007,7 @@ static void show_windows(void)
 			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(xd->menu),TRUE);
 	}
 	
-	if ((use_bigtile) && (big_tile_item != NULL))
+	if ((tile_width == 2) && (big_tile_item != NULL))
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(big_tile_item), TRUE);
 }
 
@@ -1991,6 +2061,12 @@ static errr term_data_init(term_data *td, int i)
 {
 	term *t = &td->t;
 
+	conv = iconv_open("UTF-8", "ISO-8859-1");
+	if (conv == (iconv_t)(-1)) {
+		printf("iconv_open() failed: %d\n", errno);
+		conv = NULL;
+	}
+
 	td->cols = 80;
 	td->rows = 24;
 
@@ -2015,6 +2091,8 @@ static errr term_data_init(term_data *td, int i)
 	t->wipe_hook = Term_wipe_gtk;
 	t->curs_hook = Term_curs_gtk;
 	t->pict_hook = Term_pict_gtk;
+	if (conv != NULL)
+		t->xchar_hook = Term_xchar_gtk;
 	
 	/* Save the data */
 	t->data = td;
@@ -2026,13 +2104,13 @@ static errr term_data_init(term_data *td, int i)
 	return (0);
 }
 
-static errr get_init_cmd()
+static errr get_init_cmd(bool wait)
 {
 	Term_fresh();
 
 	/* Prompt the user */
 	prt("[Choose 'New' or 'Open' from the 'File' menu]", 23, 17);
-	CheckEvent(FALSE);
+	CheckEvent(wait);
 
 	return 0;
 }
@@ -2056,7 +2134,7 @@ static void init_color_tags(xtra_win_data *xd)
 
 	tags = gtk_text_buffer_get_tag_table(xd->buf);
 	
-	for (i = 0; i <= 15; i++)
+	for (i = 0; i <= MAX_COLORS; i++)
 	{
 		strnfmt(colorname, sizeof(colorname),  "color-%d", i);
 		strnfmt(str, sizeof(str), "#%02x%02x%02x", angband_color_table[i][1], angband_color_table[i][2], angband_color_table[i][3]);
@@ -2154,6 +2232,17 @@ static void glog(cptr fmt, ...)
 		plog(str);
 }
 
+static void reinitialize_text_buffer(xtra_win_data *xd)
+{
+	if (!GTK_IS_TEXT_BUFFER(xd->buf))
+		xd->buf = gtk_text_buffer_new(NULL);
+	else
+		gtk_text_buffer_set_text(xd->buf, "", 0);
+	gtk_text_view_set_buffer(GTK_TEXT_VIEW(xd->text_view), xd->buf);
+	init_color_tags(xd);
+}
+
+
 /*
  * Update our own personal message window.
  */
@@ -2166,10 +2255,7 @@ static void handle_message(game_event_type type, game_event_data *data, void *us
 
 	if (!xd) return;
 
-	xd->buf = gtk_text_buffer_new(NULL);
-	gtk_text_view_set_buffer(GTK_TEXT_VIEW (xd->text_view), xd->buf);
-	
-	init_color_tags(xd);
+	reinitialize_text_buffer(xd);
 	
 	num = messages_num();
 	
@@ -2188,7 +2274,7 @@ static int last_inv_slot(void)
 	/* Find the "final" slot */
 	for (i = 0; i < INVEN_PACK; i++)
 	{
-		o_ptr = &inventory[i];
+		o_ptr = &p_ptr->inventory[i];
 
 		/* Skip non-objects */
 		if (!o_ptr->k_idx) continue;
@@ -2208,13 +2294,13 @@ static void inv_slot(char *str, size_t len, int i, bool equip)
 	int name_size = 80;
 	
 	/* Examine the item */
-	o_ptr = &inventory[i];
+	o_ptr = &p_ptr->inventory[i];
 
 	/* Is this item "acceptable"? */
 	if (item_tester_okay(o_ptr) || equip)
 	{
 		/* Obtain an item description */
-		object_desc(o_name, sizeof(o_name), o_ptr, TRUE, ODESC_FULL);
+		object_desc(o_name, sizeof(o_name), o_ptr, ODESC_PREFIX | ODESC_FULL);
 			
 		/* Obtain the length of the description */
 		n = strlen(o_name);
@@ -2256,13 +2342,9 @@ static void handle_inv(game_event_type type, game_event_data *data, void *user)
 	register int i, z;
 	byte attr;
 
-
 	if (!xd) return;
 
-	xd->buf = gtk_text_buffer_new(NULL);
-	gtk_text_view_set_buffer(GTK_TEXT_VIEW(xd->text_view), xd->buf);
-	
-	init_color_tags(xd);
+	reinitialize_text_buffer(xd);
 	
 	z = last_inv_slot();
 	
@@ -2270,7 +2352,7 @@ static void handle_inv(game_event_type type, game_event_data *data, void *user)
 	for (i = 0; i < z; i++)
 	{
 		/* Examine the item */
-		object_type *o_ptr = &inventory[i];
+		object_type *o_ptr = &p_ptr->inventory[i];
 
 		/* Is this item "acceptable"? */
 		if (item_tester_okay(o_ptr))
@@ -2290,19 +2372,16 @@ static void handle_equip(game_event_type type, game_event_data *data, void *user
 	
 	register int i;
 	byte attr;
-		char str[80];
+	char str[80];
 	
 	if (!xd) return;
-
-	xd->buf = gtk_text_buffer_new(NULL);
-	gtk_text_view_set_buffer(GTK_TEXT_VIEW(xd->text_view), xd->buf);
 	
-	init_color_tags(xd);
+	reinitialize_text_buffer(xd);
 	
 	/* Display the pack */
 	for (i = INVEN_WIELD; i < INVEN_TOTAL; i++)
 	{
-		object_type *o_ptr = &inventory[i];
+		object_type *o_ptr = &p_ptr->inventory[i];
 	
 		attr = tval_to_attr[o_ptr->tval % N_ELEMENTS(tval_to_attr)];
 
@@ -2335,10 +2414,7 @@ static void handle_mons_list(game_event_type type, game_event_data *data, void *
 
 	if (!xd) return;
 
-	xd->buf = gtk_text_buffer_new(NULL);
-	gtk_text_view_set_buffer(GTK_TEXT_VIEW (xd->text_view), xd->buf);
-	
-	init_color_tags(xd);
+	reinitialize_text_buffer(xd);
 	
 	/* Allocate the array */
 	race_count = C_ZNEW(z_info->r_max, u16b);
@@ -2374,7 +2450,7 @@ static void handle_mons_list(game_event_type type, game_event_data *data, void *
 	text_view_print(xd, str, 1);
 	
 	/* Go over in reverse order (so we show harder monsters first) */
-for (i = 1; i < z_info->r_max; i++)
+	for (i = 1; i < z_info->r_max; i++)
 	{
 		monster_lore *l_ptr = &l_list[i];
 
@@ -2389,12 +2465,13 @@ for (i = 1; i < z_info->r_max; i++)
 
 		/* Get monster race and name */
 		r_ptr = &r_info[i];
-		m_name = r_name + r_ptr->name;
+		m_name = r_ptr->name;
 
 		/* Display uniques in a special colour */
-		if (r_ptr->flags[0] & RF0_UNIQUE)
+		if (rf_has(r_ptr->flags, RF_UNIQUE))
 			attr = TERM_VIOLET;
-		else if (l_ptr->tkills && (r_ptr->level > p_ptr->depth))
+		/* If the player has never killed it (ever) AND it is out of depth */
+		else if ((!l_ptr->tkills) && (r_ptr->level > p_ptr->depth))
 			attr = TERM_RED;
 		else
 			attr = TERM_WHITE;
@@ -2418,9 +2495,14 @@ static void draw_xtra_cr_text(xtra_win_data *xd, int x, int y, byte color, cptr
 	measurements size;
 	cairo_rectangle_t r;
 	int n = strlen(str);
+
+	/* Prevent gcc warnings */
+	r.width = 0;
+	r.height = 0;
+	r.x = 0;
+	r.y = 0;
 	
 	/* Set dimensions */
-	
 	size.w = xd->font.w;
 	size.h = xd->font.h;
 	draw_text(xd->surface, &xd->font, &size, x, y, n, color, str);
@@ -2459,7 +2541,7 @@ static void cr_print_equippy(xtra_win_data *xd, int y)
 	object_type *o_ptr;
 
 	/* No equippy chars if  we're in bigtile mode or creating a char */
-	if ((use_bigtile) || (arg_graphics != 0) || (!character_generated))
+	if ((tile_width == 2) || (arg_graphics != 0) || (!character_generated))
 	{
 		return;
 	}
@@ -2468,7 +2550,7 @@ static void cr_print_equippy(xtra_win_data *xd, int y)
 	for (i = INVEN_WIELD; i < INVEN_TOTAL; i++)
 	{
 		/* Object */
-		o_ptr = &inventory[i];
+		o_ptr = &p_ptr->inventory[i];
 
 		a = object_attr(o_ptr);
 		strnfmt(c, sizeof(c), "%c",object_char(o_ptr)); 
@@ -2524,15 +2606,15 @@ static void handle_sidebar(game_event_type type, game_event_data *data, void *us
 		draw_xtra_cr_text(xd, 0, 0, TERM_L_BLUE, str);
 		
 		/* Char Race */
-		strnfmt(str, sizeof(str), "%s", p_name + rp_ptr->name);
+		strnfmt(str, sizeof(str), "%s", rp_ptr->name);
 		draw_xtra_cr_text(xd, 0, 1, TERM_L_BLUE, str);
 		
 		/* Char Title*/
-		strnfmt(str, sizeof(str), "%s", c_text + cp_ptr->title[(p_ptr->lev - 1) / 5], TERM_L_BLUE); 
+		strnfmt(str, sizeof(str), "%s", cp_ptr->title[(p_ptr->lev - 1) / 5], TERM_L_BLUE); 
 		draw_xtra_cr_text(xd, 0, 2, TERM_L_BLUE, str);
 		
 		/* Char Class */
-		strnfmt(str, sizeof(str), "%s", c_name + cp_ptr->name); 
+		strnfmt(str, sizeof(str), "%s", cp_ptr->name); 
 		draw_xtra_cr_text(xd, 0, 3, TERM_L_BLUE, str);
 
 		/* Char Level */
@@ -2621,7 +2703,7 @@ static void handle_statusline(game_event_type type, game_event_data *data, void
 static errr gtk_get_cmd(cmd_context context, bool wait)
 {
 	if (context == CMD_INIT) 
-		return get_init_cmd();
+		return get_init_cmd(wait);
 	else 
 		return textui_get_cmd(context, wait);
 }
@@ -2707,7 +2789,9 @@ errr init_gtk(int argc, char **argv)
 		/* Save global entry */
 		angband_term[i] = Term;
 	}
-	
+
+	/* Init dirs */
+	create_needed_dirs();	
 	
 	/* Load Preferences */
 	load_prefs();
@@ -2746,8 +2830,10 @@ errr init_gtk(int argc, char **argv)
 	/* Let's play */
 	play_game();
 
-	/* Stop now */
-	exit(0);
+	/* Do all the things main() in main.c already does */
+	cleanup_angband();
+	quit(NULL);
+	exit(0); /* just in case */
 
 	/* Success */
 	return (0);
