@@ -10,15 +10,6 @@
 
 #else
 
-/*
- * Extract the "RISCOS" flag from the compiler
- */
-# ifdef __riscos
-#  ifndef RISCOS
-#   define RISCOS
-#  endif
-# endif
-
 /*
  * Extract the "WINDOWS" flag from the compiler
  */
@@ -53,10 +44,8 @@
 /*
  * Everyone except RISC OS has fcntl.h and sys/stat.h
  */
-#ifndef RISCOS
-# define HAVE_FCNTL_H
-# define HAVE_STAT
-#endif
+#define HAVE_FCNTL_H
+#define HAVE_STAT
 
 #endif /* HAVE_CONFIG_H */
 
@@ -75,7 +64,7 @@
  * Basically, SET_UID should *only* be set for "Unix" machines.
  */
 #if !defined(MACH_O_CARBON) && !defined(WINDOWS) && \
-    !defined(RISCOS) && !defined(GAMEBOY) && !defined(NDS)
+		!defined(GAMEBOY) && !defined(NDS)
 # define SET_UID
 
 /* Without autoconf, turn on some things */
@@ -103,10 +92,13 @@
  */
 #undef PATH_SEP
 #define PATH_SEP "/"
+#define PATH_SEPC '/'
 
 #ifdef WINDOWS
 # undef PATH_SEP
+# undef PATH_SEPC
 # define PATH_SEP "\\"
+# define PATH_SEPC '\\'
 #endif
 
 
@@ -125,6 +117,7 @@
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include <string.h>
 #include <time.h>
 
