@@ -1,12 +1,21 @@
-/* File: main-xxx.c */
-
 /*
+ * File: main-xxx.c
+ * Purpose: Outline how to make a new "main-xxx" file.
+ *
  * Copyright (c) 1997 Ben Harrison
  *
- * This software may be copied and distributed for educational, research,
- * and not for profit purposes provided that this copyright and statement
- * are included in all such copies.
+ * This work is free software; you can redistribute it and/or modify it
+ * under the terms of either:
+ *
+ * a) the GNU General Public License as published by the Free Software
+ *    Foundation, version 2, or
+ *
+ * b) the "Angband licence":
+ *    This software may be copied and distributed for educational, research,
+ *    and not for profit purposes provided that this copyright and statement
+ *    are included in all such copies.  Other copyrights may also apply.
  */
+#include "angband.h"
 
 
 /*
@@ -66,9 +75,6 @@
  */
 
 
-#include "angband.h"
-
-
 #ifdef USE_XXX
 
 #include "main.h"
@@ -458,6 +464,16 @@ static errr Term_wipe_xxx(int x, int y, int n)
 	return (0);
 }
 
+/*
+ * Given a position in the ISO Latin-1 character set, return
+ * the correct character on this system.
+ */
+ static byte Term_xchar_xxx(byte c)
+{
+ 	/* The xxx port uses the Latin-1 standard */
+ 	return (c);
+}
+
 
 /*
  * Draw some text on the screen
@@ -613,6 +629,7 @@ static void term_data_link(int i)
 	t->wipe_hook = Term_wipe_xxx;
 	t->text_hook = Term_text_xxx;
 	t->pict_hook = Term_pict_xxx;
+	t->xchar_hook = Term_xchar_xxx;
 
 	/* Remember where we came from */
 	t->data = td;
@@ -628,7 +645,7 @@ static void term_data_link(int i)
 /*
  * Help message.
  *   1st line = max 68 chars.
- *   Start next lines with 11 spaces, as in main-xaw.c.
+ *   Start next lines with 11 spaces
  */
 const char help_xxx[] = "Describe XXX, subopts -describe suboptions here";
 
@@ -724,7 +741,7 @@ static void init_stuff(void)
 	my_strcpy(path, "XXX XXX XXX", sizeof(path));
 
 	/* Prepare the filepaths */
-	init_file_paths(path);
+	init_file_paths(path, path, path);
 
 
 #ifdef USE_SOUND
