@@ -15,21 +15,21 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-#include "angband.h"
 
+#include "angband.h"
+#include "files.h"
+#include "init.h"
 
 /*
  * Some machines have a "main()" function in their "main-xxx.c" file,
  * all the others use this file for their "main()" function.
  */
 
-
-#if defined(WIN32_CONSOLE_MODE) \
-    || (!defined(WINDOWS) && !defined(RISCOS)) \
-    || defined(USE_SDL)
+#if defined(WIN32_CONSOLE_MODE) || !defined(WINDOWS) || defined(USE_SDL)
 
 #include "main.h"
 #include "textui.h"
+#include "init.h"
 
 /*
  * List of the available modules in the order they are tried.
@@ -51,6 +51,10 @@ static const struct module modules[] =
 #ifdef USE_GCU
 	{ "gcu", help_gcu, init_gcu },
 #endif /* USE_GCU */
+
+#ifdef USE_TEST
+	{ "test", help_test, init_test },
+#endif /* !USE_TEST */
 };
 
 
@@ -112,7 +116,7 @@ static void quit_hook(cptr s)
  *
  * First, we'll look for the ANGBAND_PATH environment variable,
  * and then look for the files in there.  If that doesn't work,
- * we'll try the DEFAULT_PATH constant.  So be sure that one of
+ * we'll try the DEFAULT_PATH constants.  So be sure that one of
  * these two things works...
  *
  * We must ensure that the path ends with "PATH_SEP" if needed,
@@ -125,20 +129,27 @@ static void quit_hook(cptr s)
  */
 static void init_stuff(void)
 {
-	char path[1024];
-
+	char configpath[512];
+	char libpath[512];
+	char datapath[512];
 
 	/* Use the angband_path, or a default */
-	my_strcpy(path, DEFAULT_PATH, sizeof(path));
+	my_strcpy(configpath, DEFAULT_CONFIG_PATH, sizeof(configpath));
+	my_strcpy(libpath, DEFAULT_LIB_PATH, sizeof(libpath));
+	my_strcpy(datapath, DEFAULT_DATA_PATH, sizeof(datapath));
 
-	/* Make sure it's terminated */
-	path[511] = '\0';
+	/* Make sure they're terminated */
+	configpath[511] = '\0';
+	libpath[511] = '\0';
+	datapath[511] = '\0';
 
 	/* Hack -- Add a path separator (only if needed) */
-	if (!suffix(path, PATH_SEP)) my_strcat(path, PATH_SEP, sizeof(path));
+	if (!suffix(configpath, PATH_SEP)) my_strcat(configpath, PATH_SEP, sizeof(configpath));
+	if (!suffix(libpath, PATH_SEP)) my_strcat(libpath, PATH_SEP, sizeof(libpath));
+	if (!suffix(datapath, PATH_SEP)) my_strcat(datapath, PATH_SEP, sizeof(datapath));
 
 	/* Initialize */
-	init_file_paths(path);
+	init_file_paths(configpath, libpath, datapath);
 }
 
 
@@ -216,6 +227,18 @@ static errr default_get_cmd(cmd_context context, bool wait)
 		return textui_get_cmd(context, wait);
 }
 
+static void debug_opt(const char *arg) {
+	if (streq(arg, "mem-poison-alloc"))
+		mem_flags |= MEM_POISON_ALLOC;
+	else if (streq(arg, "mem-poison-free"))
+		mem_flags |= MEM_POISON_FREE;
+	else {
+		puts("Debug flags:");
+		puts("  mem-poison-alloc: Poison all memory allocations");
+		puts("   mem-poison-free: Poison all freed memory");
+		exit(0);
+	}
+}
 
 /*
  * Simple "main" function for multiple platforms.
@@ -247,10 +270,6 @@ int main(int argc, char *argv[])
 #endif /* SET_UID */
 
 
-	/* Get the file paths */
-	init_stuff();
-
-
 #ifdef SET_UID
 
 	/* Get the user id */
@@ -266,21 +285,6 @@ int main(int argc, char *argv[])
 	safe_setuid_drop();
 
 
-#ifdef SET_UID
-
-	/* Get the "user name" as a default player name */
-	user_name(op_ptr->full_name, sizeof(op_ptr->full_name), player_uid);
-
-#ifdef PRIVATE_USER_PATH
-
-	/* Create directories for the users files */
-	create_user_dirs();
-
-#endif /* PRIVATE_USER_PATH */
-
-#endif /* SET_UID */
-
-
 	/* Process the command line arguments */
 	for (i = 1; args && (i < argc); i++)
 	{
@@ -346,6 +350,10 @@ int main(int argc, char *argv[])
 				continue;
 			}
 
+			case 'x':
+				debug_opt(arg);
+				continue;
+
 			case '-':
 			{
 				argv[i] = argv[0];
@@ -360,11 +368,11 @@ int main(int argc, char *argv[])
 			{
 				/* Dump usage information */
 				puts("Usage: angband [options] [-- subopts]");
-				puts("  -n             Start a new character");
-				puts("  -L             Load a new-format save file");
+				puts("  -n             Start a new character (WARNING: overwrites default savefile without -u)");
 				puts("  -w             Resurrect dead character (marks savefile)");
-				puts("  -r             Rebalance monsters if monster.raw is absent");
+				puts("  -r             Rebalance monsters");
 				puts("  -g             Request graphics mode");
+				puts("  -x<opt>        Debug options; see -xhelp");
 				puts("  -u<who>        Use your <who> savefile");
 				puts("  -d<path>       Store pref files and screendumps in <path>");
 				puts("  -m<sys>        Use module <sys>, where <sys> can be:");
@@ -390,6 +398,11 @@ int main(int argc, char *argv[])
 		argv[1] = NULL;
 	}
 
+	/* Get the file paths */
+	init_stuff();
+
+	/* Install "quit" hook */
+	quit_aux = quit_hook;
 
 	/* Try the modules in the order specified by modules[] */
 	for (i = 0; i < (int)N_ELEMENTS(modules); i++)
@@ -397,10 +410,11 @@ int main(int argc, char *argv[])
 		/* User requested a specific module? */
 		if (!mstr || (streq(mstr, modules[i].name)))
 		{
+			ANGBAND_SYS = modules[i].name;
 			if (0 == modules[i].init(argc, argv))
 			{
-				ANGBAND_SYS = modules[i].name;
 				done = TRUE;
+				/*quit_fmt("ggg %s", ANGBAND_SYS);*/
 				break;
 			}
 		}
@@ -409,13 +423,22 @@ int main(int argc, char *argv[])
 	/* Make sure we have a display! */
 	if (!done) quit("Unable to prepare any 'display module'!");
 
+#ifdef SET_UID
+
+	/* Get the "user name" as a default player name, unless set with -u switch */
+	if (!op_ptr->full_name[0])
+	{
+		user_name(op_ptr->full_name, sizeof(op_ptr->full_name), player_uid);
+	}
+
+	/* Create any missing directories */
+	create_needed_dirs();
+
+#endif /* SET_UID */
 
 	/* Process the player name */
 	process_player_name(TRUE);
 
-	/* Install "quit" hook */
-	quit_aux = quit_hook;
-
 #ifdef USE_SOUND
 
 	/* Try the modules in the order specified by sound_modules[] */
@@ -450,4 +473,4 @@ int main(int argc, char *argv[])
 	return (0);
 }
 
-#endif /* !defined(MACINTOSH) && !defined(WINDOWS) && !defined(RISCOS) */
+#endif
