@@ -15,9 +15,10 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-#include "angband.h"
-#include "tvalsval.h"
 
+#include "angband.h"
+#include "squelch.h"
+#include "tvalsval.h"
 
 /*
  * Puts a very stripped-down version of an object's name into buf.
@@ -35,25 +36,21 @@ void object_kind_name(char *buf, size_t max, int k_idx, bool easy_know)
 	/* If not aware, use flavor */
 	if (!easy_know && !k_ptr->aware && k_ptr->flavor)
 	{
-		if (k_ptr->tval == TV_SCROLL)
+		if (k_ptr->tval == TV_FOOD && k_ptr->sval > SV_FOOD_MIN_SHROOM)
 		{
-			strnfmt(buf, max, "\"%s\"", scroll_adj[k_ptr->sval]);
-		}
-		else if (k_ptr->tval == TV_FOOD && k_ptr->sval > SV_FOOD_MIN_SHROOM)
-		{
-			strnfmt(buf, max, "%s Mushroom", flavor_text + flavor_info[k_ptr->flavor].text);
+			strnfmt(buf, max, "%s Mushroom", flavor_info[k_ptr->flavor].text);
 		}
 		else
 		{
 			/* Plain flavour (e.g. Copper) will do. */
-			my_strcpy(buf, flavor_text + flavor_info[k_ptr->flavor].text, max);
+			my_strcpy(buf, flavor_info[k_ptr->flavor].text, max);
 		}
 	}
 
 	/* Use proper name (Healing, or whatever) */
 	else
 	{
-		cptr str = (k_name + k_ptr->name);
+		cptr str = k_ptr->name;
 
 		if (k_ptr->tval == TV_FOOD && k_ptr->sval > SV_FOOD_MIN_SHROOM)
 		{
@@ -115,14 +112,12 @@ static const char *obj_desc_get_modstr(const object_type *o_ptr)
 		case TV_ROD:
 		case TV_POTION:
 		case TV_FOOD:
-			return (flavor_text + flavor_info[k_ptr->flavor].text);
-
 		case TV_SCROLL:
-			return scroll_adj[o_ptr->sval];
+			return flavor_info[k_ptr->flavor].text;
 
 		case TV_MAGIC_BOOK:
 		case TV_PRAYER_BOOK:
-			return (k_name + k_ptr->name);
+			return k_ptr->name;
 	}
 
 	return "";
@@ -142,7 +137,7 @@ static const char *obj_desc_get_basename(const object_type *o_ptr, bool aware)
 
 	/* Known artifacts get special treatment */
 	if (artifact_p(o_ptr) && aware)
-		return (k_name + k_ptr->name);
+		return k_ptr->name;
 
 	/* Analyze the object */
 	switch (o_ptr->tval)
@@ -170,8 +165,8 @@ static const char *obj_desc_get_basename(const object_type *o_ptr, bool aware)
 		case TV_SOFT_ARMOR:
 		case TV_HARD_ARMOR:
 		case TV_DRAG_ARMOR:
-		case TV_LITE:
-			return (k_name + k_ptr->name);
+		case TV_LIGHT:
+			return k_ptr->name;
 
 		case TV_AMULET:
 			return (show_flavor ? "& # Amulet~" : "& Amulet~");
@@ -192,7 +187,7 @@ static const char *obj_desc_get_basename(const object_type *o_ptr, bool aware)
 			return (show_flavor ? "& # Potion~" : "& Potion~");
 
 		case TV_SCROLL:
-			return (show_flavor ? "& Scroll~ titled \"#\"" : "& Scroll~");
+			return (show_flavor ? "& Scroll~ titled #" : "& Scroll~");
 
 		case TV_MAGIC_BOOK:
 			return "& Book~ of Magic Spells #";
@@ -204,7 +199,7 @@ static const char *obj_desc_get_basename(const object_type *o_ptr, bool aware)
 			if (o_ptr->sval > SV_FOOD_MIN_SHROOM)
 				return (show_flavor ? "& # Mushroom~" : "& Mushroom~");
 			else
-				return (k_name + k_ptr->name);
+				return k_ptr->name;
 	}
 
 	return "(nothing)";
@@ -233,6 +228,9 @@ static size_t obj_desc_name(char *buf, size_t max, size_t end,
 
 	bool pluralise = (mode & ODESC_PLURAL) ? TRUE : FALSE;
 
+	if (aware && !k_ptr->everseen)
+		k_ptr->everseen = TRUE;
+
 	if (o_ptr->number > 1)
 		pluralise = TRUE;
 	if (mode & ODESC_SINGULAR)
@@ -250,7 +248,7 @@ static size_t obj_desc_name(char *buf, size_t max, size_t end,
 		}
 		else if (o_ptr->number > 1)
 			strnfcat(buf, max, &end, "%d ", o_ptr->number);
-		else if (known && artifact_p(o_ptr))
+		else if ((object_name_is_visible(o_ptr) || known) && artifact_p(o_ptr))
 			strnfcat(buf, max, &end, "The ");
 
 		else if (*basename == '&')
@@ -394,14 +392,14 @@ static size_t obj_desc_name(char *buf, size_t max, size_t end,
 
 	/** Append extra names of various kinds **/
 
-	if (known && o_ptr->name1)
-		strnfcat(buf, max, &end, " %s", a_name + a_info[o_ptr->name1].name);
+	if ((object_name_is_visible(o_ptr) || known) && o_ptr->name1)
+		strnfcat(buf, max, &end, " %s", a_info[o_ptr->name1].name);
 
 	else if ((spoil && o_ptr->name2) || object_ego_is_visible(o_ptr))
-		strnfcat(buf, max, &end, " %s", e_name + e_info[o_ptr->name2].name);
+		strnfcat(buf, max, &end, " %s", e_info[o_ptr->name2].name);
 
 	else if (aware && !artifact_p(o_ptr) && (k_ptr->flavor || k_ptr->tval == TV_SCROLL))
-		strnfcat(buf, max, &end, " of %s", k_name + k_ptr->name);
+		strnfcat(buf, max, &end, " of %s", k_ptr->name);
 
 	return end;
 }
@@ -411,10 +409,11 @@ static size_t obj_desc_name(char *buf, size_t max, size_t end,
  */
 static bool obj_desc_show_weapon(const object_type *o_ptr)
 {
-	u32b f[OBJ_FLAG_N];
+	bitflag f[OF_SIZE];
+
 	object_flags(o_ptr, f);
 
-	if (f[2] & TR2_SHOW_MODS) return TRUE;
+	if (of_has(f, OF_SHOW_MODS)) return TRUE;
 	if (o_ptr->to_h && o_ptr->to_d) return TRUE;
 
 	/* You need to list both to_h and to_d for things like unaware rings of accuracy and damage e.g. to differentiate (+8) */
@@ -531,6 +530,13 @@ static size_t obj_desc_chest(const object_type *o_ptr, char *buf, size_t max, si
 static size_t obj_desc_combat(const object_type *o_ptr, char *buf, size_t max, 
 		size_t end, bool spoil)
 {
+	object_kind *k_ptr = &k_info[o_ptr->k_idx];
+	bitflag flags[OF_SIZE];
+	bitflag flags_known[OF_SIZE];
+
+	object_flags(o_ptr, flags);
+	object_flags_known(o_ptr, flags_known);
+
 	/* Dump base weapon info */
 	switch (o_ptr->tval)
 	{
@@ -543,14 +549,23 @@ static size_t obj_desc_combat(const object_type *o_ptr, char *buf, size_t max,
 		case TV_SWORD:
 		case TV_DIGGING:
 		{
-			strnfcat(buf, max, &end, " (%dd%d)", o_ptr->dd, o_ptr->ds);
+			/* Only display the real damage dice if the combat stats are known */
+			if (spoil || object_attack_plusses_are_visible(o_ptr))
+				strnfcat(buf, max, &end, " (%dd%d)", o_ptr->dd, o_ptr->ds);
+			else
+				strnfcat(buf, max, &end, " (%dd%d)", k_ptr->dd, k_ptr->ds);
 			break;
 		}
 
 		/* Missile launchers */
 		case TV_BOW:
 		{
-			strnfcat(buf, max, &end, " (x%d)", o_ptr->sval % 10);
+			/* Display shooting power as part of the multiplier */
+			if (of_has(flags, OF_MIGHT) &&
+			    (spoil || object_flag_is_known(o_ptr, OF_MIGHT)))
+				strnfcat(buf, max, &end, " (x%d)", (o_ptr->sval % 10) + o_ptr->pval);
+			else
+				strnfcat(buf, max, &end, " (x%d)", o_ptr->sval % 10);
 			break;
 		}
 	}
@@ -559,10 +574,19 @@ static size_t obj_desc_combat(const object_type *o_ptr, char *buf, size_t max,
 	/* Show weapon bonuses */
 	if (spoil || object_attack_plusses_are_visible(o_ptr))
 	{
-		if (obj_desc_show_weapon(o_ptr) || o_ptr->to_d)
-			strnfcat(buf, max, &end, " (%+d,%+d)", o_ptr->to_h, o_ptr->to_d);
-		else if (o_ptr->to_h)
-			strnfcat(buf, max, &end, " (%+d)", o_ptr->to_h);
+		if (obj_desc_show_weapon(o_ptr) || o_ptr->to_d || o_ptr->to_h)
+		{
+			/* Make an exception for body armor with only a to-hit penalty */
+			if (o_ptr->to_h < 0 && o_ptr->to_d == 0 &&
+			    (o_ptr->tval == TV_SOFT_ARMOR ||
+			     o_ptr->tval == TV_HARD_ARMOR ||
+			     o_ptr->tval == TV_DRAG_ARMOR))
+				strnfcat(buf, max, &end, " (%+d)", o_ptr->to_h);
+
+			/* Otherwise, always use the full tuple */
+			else
+				strnfcat(buf, max, &end, " (%+d,%+d)", o_ptr->to_h, o_ptr->to_d);
+		}
 	}
 
 
@@ -584,11 +608,12 @@ static size_t obj_desc_combat(const object_type *o_ptr, char *buf, size_t max,
 
 static size_t obj_desc_light(const object_type *o_ptr, char *buf, size_t max, size_t end)
 {
-	u32b f[OBJ_FLAG_N];
+	bitflag f[OF_SIZE];
+
 	object_flags(o_ptr, f);
 
 	/* Fuelled light sources get number of remaining turns appended */
-	if ((o_ptr->tval == TV_LITE) && !(f[2] & TR2_NO_FUEL))
+	if ((o_ptr->tval == TV_LIGHT) && !of_has(f, OF_NO_FUEL))
 		strnfcat(buf, max, &end, " (%d turns)", o_ptr->timeout);
 
 	return end;
@@ -596,29 +621,27 @@ static size_t obj_desc_light(const object_type *o_ptr, char *buf, size_t max, si
 
 static size_t obj_desc_pval(const object_type *o_ptr, char *buf, size_t max, size_t end)
 {
-	u32b f[OBJ_FLAG_N];
+	bitflag f[OF_SIZE];
+
 	object_flags(o_ptr, f);
 
-	if (!(f[0] & TR0_PVAL_MASK)) return end;
-
-	if (f[2] & TR2_HIDE_TYPE)
-	{
-		strnfcat(buf, max, &end, " (%+d)", o_ptr->pval);
-		return end;
-	}
+	if (!flags_test(f, OF_SIZE, OF_PVAL_MASK, FLAG_END)) return end;
 
 	strnfcat(buf, max, &end, " (%+d", o_ptr->pval);
 
-	if (f[0] & TR0_STEALTH)
-		strnfcat(buf, max, &end, " stealth");
-	else if (f[0] & TR0_SEARCH)
-		strnfcat(buf, max, &end, " searching");
-	else if (f[0] & TR0_INFRA)
-		strnfcat(buf, max, &end, " infravision");
-	else if (f[0] & TR0_SPEED)
-		strnfcat(buf, max, &end, " speed");
-	else if (f[0] & TR0_BLOWS)
-		strnfcat(buf, max, &end, " attack%s", PLURAL(o_ptr->pval));
+	if (!of_has(f, OF_HIDE_TYPE))
+	{
+		if (of_has(f, OF_STEALTH))
+			strnfcat(buf, max, &end, " stealth");
+		else if (of_has(f, OF_SEARCH))
+			strnfcat(buf, max, &end, " searching");
+		else if (of_has(f, OF_INFRA))
+			strnfcat(buf, max, &end, " infravision");
+		else if (of_has(f, OF_SPEED))
+			strnfcat(buf, max, &end, " speed");
+		else if (of_has(f, OF_BLOWS))
+			strnfcat(buf, max, &end, " attack%s", PLURAL(o_ptr->pval));
+	}
 
 	strnfcat(buf, max, &end, ")");
 
@@ -641,7 +664,7 @@ static size_t obj_desc_charges(const object_type *o_ptr, char *buf, size_t max,
 		if (o_ptr->tval == TV_ROD && o_ptr->number > 1)
 		{
 			int power;
-			int time_base = k_ptr->time_base;
+			int time_base = randcalc(k_ptr->time, 0, MINIMISE);
 
 			if (!time_base) time_base = 1;
 
@@ -659,7 +682,7 @@ static size_t obj_desc_charges(const object_type *o_ptr, char *buf, size_t max,
 		}
 
 		/* Artifacts, single rods */
-		else if (!(o_ptr->tval == TV_LITE && !artifact_p(o_ptr)))
+		else if (!(o_ptr->tval == TV_LIGHT && !artifact_p(o_ptr)))
 		{
 			strnfcat(buf, max, &end, " (charging)");
 		}
@@ -673,9 +696,9 @@ static size_t obj_desc_inscrip(const object_type *o_ptr, char *buf, size_t max,
 	const char *u[4] = { 0, 0, 0, 0 };
 	int n = 0;
 	int feel = object_pseudo(o_ptr);
+	bitflag flags_known[OF_SIZE];
 
-	u32b flags[OBJ_FLAG_N];
-	object_flags(o_ptr, flags);
+	object_flags_known(o_ptr, flags_known);
 
 	/* Get inscription */
 	if (o_ptr->note)
@@ -693,12 +716,18 @@ static size_t obj_desc_inscrip(const object_type *o_ptr, char *buf, size_t max,
 	else if ((o_ptr->ident & IDENT_EMPTY) && !object_is_known(o_ptr))
 		u[n++] = "empty";
 	else if (!object_is_known(o_ptr) && object_was_worn(o_ptr))
-		u[n++] = "wielded";
+	{
+		if (wield_slot(o_ptr) == INVEN_WIELD || wield_slot(o_ptr) == INVEN_BOW)
+			u[n++] = "wielded";
+		else u[n++] = "worn";
+	}
+	else if (!object_is_known(o_ptr) && object_was_fired(o_ptr))
+		u[n++] = "fired";
 	else if (!object_flavor_is_aware(o_ptr) && object_flavor_was_tried(o_ptr))
 		u[n++] = "tried";
 
 	/* Note curses */
-	if (o_ptr->known_flags[2] & flags[2] & TR2_CURSE_MASK)
+	if (flags_test(flags_known, OF_SIZE, OF_CURSE_MASK, FLAG_END))
 		u[n++] = "cursed";
 
 	/* Note squelch */
@@ -724,9 +753,21 @@ static size_t obj_desc_inscrip(const object_type *o_ptr, char *buf, size_t max,
 }
 
 
+/* Add "unseen" to the end of unaware items in stores */
+static size_t obj_desc_aware(const object_type *o_ptr, char *buf, size_t max,
+	size_t end)
+{
+	if (!object_flavor_is_aware(o_ptr))
+		strnfcat(buf, max, &end, " {unseen}");
+
+	return end;
+}
+
+
 /**
  * Describes item `o_ptr` into buffer `buf` of size `max`.
  *
+ * ODESC_PREFIX prepends a 'the', 'a' or number
  * ODESC_BASE results in a base description.
  * ODESC_COMBAT will add to-hit, to-dam and AC info.
  * ODESC_EXTRA will add pval/charge/inscription/squelch info.
@@ -740,32 +781,18 @@ static size_t obj_desc_inscrip(const object_type *o_ptr, char *buf, size_t max,
  * \returns The number of bytes used of the buffer.
  */
 size_t object_desc(char *buf, size_t max, const object_type *o_ptr,
-		bool prefix, odesc_detail_t mode)
+				   odesc_detail_t mode)
 {
 	object_kind *k_ptr = &k_info[o_ptr->k_idx];
 
+	bool prefix = mode & ODESC_PREFIX;
 	bool spoil = (mode & ODESC_SPOIL);
-	bool aware = object_flavor_is_aware(o_ptr) ||
-			(o_ptr->ident & IDENT_STORE) || spoil;
 	bool known = object_is_known(o_ptr) ||
 			(o_ptr->ident & IDENT_STORE) || spoil;
 
 	size_t end = 0;
 
 
-	/** Horrible, horrible squelch **/
-
-	/* Hack -- mark-to-squelch worthless items XXX */
-	if (aware && !k_ptr->everseen)
-	{
-		k_ptr->everseen = TRUE;
-		if (OPT(squelch_worthless) && k_ptr->cost == 0)
-		{
-			k_ptr->squelch = TRUE;
-			p_ptr->notice |= PN_SQUELCH;
-		}
-	}
-
 	/* We've seen it at least once now we're aware of it */
 	if (known && o_ptr->name2) e_info[o_ptr->name2].everseen = TRUE;
 
@@ -773,8 +800,9 @@ size_t object_desc(char *buf, size_t max, const object_type *o_ptr,
 	/*** Some things get really simple descriptions ***/
 
 	if (o_ptr->tval == TV_GOLD)
-		return strnfmt(buf, max, "%d gold pieces worth of %s",
-				o_ptr->pval, k_name + k_ptr->name);
+		return strnfmt(buf, max, "%d gold pieces worth of %s%s",
+				o_ptr->pval, k_ptr->name,
+				squelch_item_ok(o_ptr) ? " {squelch}" : "");
 	else if (!o_ptr->tval)
 		return strnfmt(buf, max, "(nothing)");
 
@@ -788,7 +816,7 @@ size_t object_desc(char *buf, size_t max, const object_type *o_ptr,
 	{
 		if (o_ptr->tval == TV_CHEST)
 			end = obj_desc_chest(o_ptr, buf, max, end);
-		else if (o_ptr->tval == TV_LITE)
+		else if (o_ptr->tval == TV_LIGHT)
 			end = obj_desc_light(o_ptr, buf, max, end);
 
 		end = obj_desc_combat(o_ptr, buf, max, end, spoil);
@@ -801,7 +829,11 @@ size_t object_desc(char *buf, size_t max, const object_type *o_ptr,
 
 		end = obj_desc_charges(o_ptr, buf, max, end);
 
-		if (!(mode & ODESC_STORE))
+		if (mode & ODESC_STORE)
+		{
+			end = obj_desc_aware(o_ptr, buf, max, end);
+		}
+		else
 			end = obj_desc_inscrip(o_ptr, buf, max, end);
 	}
 
