@@ -149,92 +149,6 @@ s16b get_obj_num(int level, bool good)
 }
 
 
-
-
-/*
- * Help determine an "enchantment bonus" for an object.
- *
- * To avoid floating point but still provide a smooth distribution of bonuses,
- * we simply round the results of division in such a way as to "average" the
- * correct floating point value.
- *
- * This function has been changed.  It uses "Rand_normal()" to choose values
- * from a normal distribution, whose mean moves from zero towards the max as
- * the level increases, and whose standard deviation is equal to 1/4 of the
- * max, and whose values are forced to lie between zero and the max, inclusive.
- *
- * Since the "level" rarely passes 100 before Morgoth is dead, it is very
- * rare to get the "full" enchantment on an object, even a deep levels.
- *
- * It is always possible (albeit unlikely) to get the "full" enchantment.
- *
- * A sample distribution of values from "m_bonus(10, N)" is shown below:
- *
- *   N       0     1     2     3     4     5     6     7     8     9    10
- * ---    ----  ----  ----  ----  ----  ----  ----  ----  ----  ----  ----
- *   0   66.37 13.01  9.73  5.47  2.89  1.31  0.72  0.26  0.12  0.09  0.03
- *   8   46.85 24.66 12.13  8.13  4.20  2.30  1.05  0.36  0.19  0.08  0.05
- *  16   30.12 27.62 18.52 10.52  6.34  3.52  1.95  0.90  0.31  0.15  0.05
- *  24   22.44 15.62 30.14 12.92  8.55  5.30  2.39  1.63  0.62  0.28  0.11
- *  32   16.23 11.43 23.01 22.31 11.19  7.18  4.46  2.13  1.20  0.45  0.41
- *  40   10.76  8.91 12.80 29.51 16.00  9.69  5.90  3.43  1.47  0.88  0.65
- *  48    7.28  6.81 10.51 18.27 27.57 11.76  7.85  4.99  2.80  1.22  0.94
- *  56    4.41  4.73  8.52 11.96 24.94 19.78 11.06  7.18  3.68  1.96  1.78
- *  64    2.81  3.07  5.65  9.17 13.01 31.57 13.70  9.30  6.04  3.04  2.64
- *  72    1.87  1.99  3.68  7.15 10.56 20.24 25.78 12.17  7.52  4.42  4.62
- *  80    1.02  1.23  2.78  4.75  8.37 12.04 27.61 18.07 10.28  6.52  7.33
- *  88    0.70  0.57  1.56  3.12  6.34 10.06 15.76 30.46 12.58  8.47 10.38
- *  96    0.27  0.60  1.25  2.28  4.30  7.60 10.77 22.52 22.51 11.37 16.53
- * 104    0.22  0.42  0.77  1.36  2.62  5.33  8.93 13.05 29.54 15.23 22.53
- * 112    0.15  0.20  0.56  0.87  2.00  3.83  6.86 10.06 17.89 27.31 30.27
- * 120    0.03  0.11  0.31  0.46  1.31  2.48  4.60  7.78 11.67 25.53 45.72
- * 128    0.02  0.01  0.13  0.33  0.83  1.41  3.24  6.17  9.57 14.22 64.07
- */
-static s16b m_bonus(int max, int level)
-{
-	int bonus, stand, extra, value;
-
-
-	/* Paranoia -- enforce maximal "level" */
-	if (level > MAX_DEPTH - 1) level = MAX_DEPTH - 1;
-
-
-	/* The "bonus" moves towards the max */
-	bonus = ((max * level) / MAX_DEPTH);
-
-	/* Hack -- determine fraction of error */
-	extra = ((max * level) % MAX_DEPTH);
-
-	/* Hack -- simulate floating point computations */
-	if (randint0(MAX_DEPTH) < extra) bonus++;
-
-
-	/* The "stand" is equal to one quarter of the max */
-	stand = (max / 4);
-
-	/* Hack -- determine fraction of error */
-	extra = (max % 4);
-
-	/* Hack -- simulate floating point computations */
-	if (randint0(4) < extra) stand++;
-
-
-	/* Choose an "interesting" value */
-	value = Rand_normal(bonus, stand);
-
-	/* Enforce the minimum value */
-	if (value < 0) return (0);
-
-	/* Enforce the maximum value */
-	if (value > max) return (max);
-
-	/* Result */
-	return (value);
-}
-
-
-
-
 /*
  * Cheat -- describe a created object for the user
  */
@@ -243,8 +157,7 @@ static void object_mention(const object_type *o_ptr)
 	char o_name[80];
 
 	/* Describe */
-	object_desc(o_name, sizeof(o_name), o_ptr, FALSE,
-			ODESC_BASE | ODESC_SPOIL);
+	object_desc(o_name, sizeof(o_name), o_ptr, ODESC_BASE | ODESC_SPOIL);
 
 	/* Provide a silly message */
 	if (artifact_p(o_ptr))
@@ -357,7 +270,7 @@ static int make_ego_item(object_type *o_ptr, int level, bool force_uncursed)
 	e_idx = (byte)table[i].index;
 	o_ptr->name2 = e_idx;
 
-	return (e_info[e_idx].flags[2] & TR2_CURSE_MASK ? -2 : 2);
+	return (flags_test(e_info[e_idx].flags, OF_SIZE, OF_CURSE_MASK, FLAG_END) ? -2 : 2);
 }
 
 
@@ -379,13 +292,20 @@ static void copy_artifact_data(object_type *o_ptr, const artifact_type *a_ptr)
 
 	/* Hack -- extract the "cursed" flags */
 	if (cursed_p(a_ptr))
-		o_ptr->flags[2] |= (a_ptr->flags[2] & TR2_CURSE_MASK);
+	{
+		bitflag curse_flags[OF_SIZE];
 
-	/* Mega-Hack -- increase the rating */
-	rating += 10;
+		of_copy(curse_flags, a_ptr->flags);
+		flags_mask(curse_flags, OF_SIZE, OF_CURSE_MASK, FLAG_END);
+		of_union(o_ptr->flags, curse_flags);
+	}
 
-	/* Mega-Hack -- increase the rating again */
-	if (a_ptr->cost > 50000L) rating += 10;
+	/* Mega-Hack -- increase the level rating
+	 * - a sizeable increase for any artifact (c.f. up to 30 for ego items)
+	 * - a bigger increase for more powerful artifacts
+	 */
+	rating += 30;
+	rating += object_power(o_ptr, FALSE, NULL, TRUE) / 25;
 
 	/* Set the good item flag */
 	good_item_flag = TRUE;
@@ -404,9 +324,7 @@ static void copy_artifact_data(object_type *o_ptr, const artifact_type *a_ptr)
  * Note -- see "make_artifact()" and "apply_magic()".
  *
  * We *prefer* to create the special artifacts in order, but this is
- * normally outweighed by the "rarity" rolls for those artifacts.  The
- * only major effect of this logic is that the Phial (with rarity one)
- * is always the first special artifact created.
+ * normally outweighed by the "rarity" rolls for those artifacts.
  */
 static bool make_artifact_special(object_type *o_ptr, int level)
 {
@@ -433,17 +351,20 @@ static bool make_artifact_special(object_type *o_ptr, int level)
 		if (a_ptr->created) continue;
 
 		/* Enforce minimum "depth" (loosely) */
-		if (a_ptr->level > p_ptr->depth)
+		if (a_ptr->alloc_min > p_ptr->depth)
 		{
 			/* Get the "out-of-depth factor" */
-			int d = (a_ptr->level - p_ptr->depth) * 2;
+			int d = (a_ptr->alloc_min - p_ptr->depth) * 2;
 
 			/* Roll for out-of-depth creation */
 			if (randint0(d) != 0) continue;
 		}
 
+		/* Enforce maximum depth (strictly) */
+		if (a_ptr->alloc_max < p_ptr->depth) continue;
+
 		/* Artifact "rarity roll" */
-		if (randint0(a_ptr->rarity) != 0) continue;
+		if (randint1(100) > a_ptr->alloc_prob) continue;
 
 		/* Find the base object */
 		k_idx = lookup_kind(a_ptr->tval, a_ptr->sval);
@@ -459,7 +380,7 @@ static bool make_artifact_special(object_type *o_ptr, int level)
 		}
 
 		/* Assign the template */
-		object_prep(o_ptr, k_idx);
+		object_prep(o_ptr, &k_info[k_idx], a_ptr->alloc_min, RANDOMISE);
 
 		/* Mark the item as an artifact */
 		o_ptr->name1 = i;
@@ -480,7 +401,9 @@ static bool make_artifact_special(object_type *o_ptr, int level)
 
 
 /*
- * Attempt to change an object into an artifact
+ * Attempt to change an object into an artifact.  If the object's name1
+ * is already set, use that artifact.  Otherwise, look for a suitable
+ * artifact and attempt to use it.
  *
  * This routine should only be called by "apply_magic()"
  *
@@ -488,11 +411,15 @@ static bool make_artifact_special(object_type *o_ptr, int level)
  */
 static bool make_artifact(object_type *o_ptr)
 {
+	artifact_type *a_ptr;
 	int i;
 
 
 	/* No artifacts, do nothing */
-	if (OPT(adult_no_artifacts)) return (FALSE);
+	if (OPT(adult_no_artifacts) &&
+	    o_ptr->name1 != ART_GROND &&
+	    o_ptr->name1 != ART_MORGOTH)
+		return (FALSE);
 
 	/* No artifacts in the town */
 	if (!p_ptr->depth) return (FALSE);
@@ -501,9 +428,9 @@ static bool make_artifact(object_type *o_ptr)
 	if (o_ptr->number != 1) return (FALSE);
 
 	/* Check the artifact list (skip the "specials") */
-	for (i = ART_MIN_NORMAL; i < z_info->a_max; i++)
+	for (i = ART_MIN_NORMAL; !o_ptr->name1 && i < z_info->a_max; i++)
 	{
-		artifact_type *a_ptr = &a_info[i];
+		a_ptr = &a_info[i];
 
 		/* Skip "empty" items */
 		if (!a_ptr->name) continue;
@@ -516,25 +443,33 @@ static bool make_artifact(object_type *o_ptr)
 		if (a_ptr->sval != o_ptr->sval) continue;
 
 		/* XXX XXX Enforce minimum "depth" (loosely) */
-		if (a_ptr->level > p_ptr->depth)
+		if (a_ptr->alloc_min > p_ptr->depth)
 		{
 			/* Get the "out-of-depth factor" */
-			int d = (a_ptr->level - p_ptr->depth) * 2;
+			int d = (a_ptr->alloc_min - p_ptr->depth) * 2;
 
 			/* Roll for out-of-depth creation */
 			if (randint0(d) != 0) continue;
 		}
 
+		/* Enforce maximum depth (strictly) */
+		if (a_ptr->alloc_max < p_ptr->depth) continue;
+
 		/* We must make the "rarity roll" */
-		if (randint0(a_ptr->rarity) != 0) continue;
+		if (randint1(100) > a_ptr->alloc_prob) continue;
 
 		/* Mark the item as an artifact */
 		o_ptr->name1 = i;
+	}
+
+	if (o_ptr->name1)
+	{
+		a_ptr = &a_info[o_ptr->name1];
 
 		/* Copy across all the data from the artifact struct */
 		copy_artifact_data(o_ptr, a_ptr);
 
-		/* Hack -- Mark the artifact as "created" */
+		/* Mark the artifact as "created" */
 		a_ptr->created = 1;
 
 		return TRUE;
@@ -677,7 +612,7 @@ static void a_m_aux_2(object_type *o_ptr, int level, int power)
 		case TV_DRAG_ARMOR:
 		{
 			/* Rating boost */
-			rating += 30;
+			rating += object_power(o_ptr, FALSE, NULL, TRUE) / 15;
 
 			/* Mention the item */
 			if (OPT(cheat_peek)) object_mention(o_ptr);
@@ -699,46 +634,73 @@ static void a_m_aux_2(object_type *o_ptr, int level, int power)
  */
 static void a_m_aux_3(object_type *o_ptr, int level, int power)
 {
+	/* Apply curses */
+	if (power < 0)
+	{
+		/* Rings */
+		if (o_ptr->tval == TV_RING)
+		{
+			switch (o_ptr->sval)
+			{
+				case SV_RING_STRENGTH:
+				case SV_RING_CONSTITUTION:
+				case SV_RING_DEXTERITY:
+				case SV_RING_INTELLIGENCE:
+				case SV_RING_SEARCHING:
+				case SV_RING_DAMAGE:
+				case SV_RING_ACCURACY:
+				case SV_RING_PROTECTION:
+				case SV_RING_SLAYING:
+				{
+					o_ptr->pval = -o_ptr->pval;
+					o_ptr->to_h = -o_ptr->to_h;
+					o_ptr->to_d = -o_ptr->to_d;
+					o_ptr->to_a = -o_ptr->to_a;
+					of_on(o_ptr->flags, OF_LIGHT_CURSE);
+
+					break;
+				}
+			}
+		}
+
+		/* Amulets */
+		else if (o_ptr->tval == TV_AMULET)
+		{
+			switch (o_ptr->sval)
+			{
+				case SV_AMULET_WISDOM:
+				case SV_AMULET_CHARISMA:
+				case SV_AMULET_INFRAVISION:
+				case SV_AMULET_SEARCHING:
+				{
+					o_ptr->pval = -o_ptr->pval;
+					o_ptr->to_h = -o_ptr->to_h;
+					o_ptr->to_d = -o_ptr->to_d;
+					o_ptr->to_a = -o_ptr->to_a;
+					of_on(o_ptr->flags, OF_LIGHT_CURSE);
+
+					break;
+				}
+			}
+		}
+	}
+
+
 	/* Apply magic (good or bad) according to type */
 	switch (o_ptr->tval)
 	{
 		case TV_RING:
 		{
 			/* Analyze */
+			
 			switch (o_ptr->sval)
 			{
-				/* Strength, Constitution, Dexterity, Intelligence */
-				case SV_RING_STRENGTH:
-				case SV_RING_CONSTITUTION:
-				case SV_RING_DEXTERITY:
-				case SV_RING_INTELLIGENCE:
-				{
-					/* Stat bonus */
-					o_ptr->pval = 1 + m_bonus(5, level);
-
-					/* Cursed */
-					if (power < 0)
-						o_ptr->pval = -o_ptr->pval;
-
-					break;
-				}
-
-				/* Ring of Speed! */
 				case SV_RING_SPEED:
 				{
-					/* Base speed (1 to 10) */
-					o_ptr->pval = randint1(5) + m_bonus(5, level);
-
 					/* Super-charge the ring */
 					while (randint0(100) < 50) o_ptr->pval++;
 
-					/* Cursed Ring */
-					if (power < 0)
-					{
-						/* Reverse pval */
-						o_ptr->pval = -o_ptr->pval;
-					}
-					else
+					if (power >= 0)
 					{
 						/* Rating boost */
 						rating += 25;
@@ -749,126 +711,6 @@ static void a_m_aux_3(object_type *o_ptr, int level, int power)
 
 					break;
 				}
-
-				/* Searching */
-				case SV_RING_SEARCHING:
-				{
-					/* Bonus to searching */
-					o_ptr->pval = 1 + m_bonus(5, level);
-
-					/* Cursed */
-					if (power < 0)
-						o_ptr->pval = -o_ptr->pval;
-
-					break;
-				}
-
-				/* Light, Dark */
-				case SV_RING_LIGHT:
-				case SV_RING_DARK:
-				{
-					/* Searching bonus */
-					o_ptr->pval = 1 + m_bonus(5, level);
-
-					break;
-				}
-
-				/* Flames, Acid, Ice, Lightning */
-				case SV_RING_FLAMES:
-				case SV_RING_ACID:
-				case SV_RING_ICE:
-				case SV_RING_LIGHTNING:
-				{
-					/* Bonus to armor class */
-					o_ptr->to_a = 5 + randint1(5) + m_bonus(10, level);
-
-					break;
-				}
-
-				/* Ring of damage */
-				case SV_RING_DAMAGE:
-				{
-					/* Bonus to damage */
-					o_ptr->to_d = 5 + randint1(3) + m_bonus(7, level);
-
-					/* Cursed */
-					if (power < 0)
-						o_ptr->to_d = -o_ptr->to_d;
-
-					break;
-				}
-
-				/* Ring of Accuracy */
-				case SV_RING_ACCURACY:
-				{
-					/* Bonus to hit */
-					o_ptr->to_h = 5 + randint1(3) + m_bonus(7, level);
-
-					/* Cursed */
-					if (power < 0)
-						o_ptr->to_h = -o_ptr->to_h;
-
-					break;
-				}
-
-				/* Ring of Protection */
-				case SV_RING_PROTECTION:
-				{
-					/* Bonus to armor class */
-					o_ptr->to_a = 5 + randint1(5) + m_bonus(10, level);
-
-					/* Cursed */
-					if (power < 0)
-						o_ptr->to_a = -o_ptr->to_a;
-
-					break;
-				}
-
-				/* Ring of Slaying */
-				case SV_RING_SLAYING:
-				{
-					/* Bonus to damage and to hit */
-					o_ptr->to_d = randint1(5) + m_bonus(5, level);
-					o_ptr->to_h = randint1(5) + m_bonus(5, level);
-
-					/* Cursed -- reverse bonuses */
-					if (power < 0)
-					{
-						o_ptr->to_h = -o_ptr->to_h;
-						o_ptr->to_d = -o_ptr->to_d;
-					}
-
-					break;
-				}
-
-				case SV_RING_RECKLESS_ATTACKS:
-				{
-					int amt = rand_range(3, 5);
-
-					o_ptr->to_d = o_ptr->to_h = amt;
-					o_ptr->to_a = -4 * amt;
-
-					break;
-				}
-
-				case SV_RING_OF_THE_MOUSE:
-				{
-					int amt = randint1(4);
-
-					/* Dex bonus, dam penalty */
-					o_ptr->pval = amt;
-					o_ptr->to_d = -3 * amt;
-
-					break;
-				}
-
-				case SV_RING_DELVING:
-				{
-					/* Digging bonus */
-					o_ptr->pval = rand_range(3, 5);
-
-					break;
-				}
 			}
 
 			break;
@@ -879,90 +721,11 @@ static void a_m_aux_3(object_type *o_ptr, int level, int power)
 			/* Analyze */
 			switch (o_ptr->sval)
 			{
-				/* Amulet of wisdom/charisma/infravision */
-				case SV_AMULET_WISDOM:
-				case SV_AMULET_CHARISMA:
-				case SV_AMULET_INFRAVISION:
-				{
-					o_ptr->pval = 1 + m_bonus(5, level);
-
-					/* Cursed */
-					if (power < 0)
-						o_ptr->pval = -o_ptr->pval;
-
-					break;
-				}
-
-				/* Amulet of searching */
-				case SV_AMULET_SEARCHING:
-				{
-					o_ptr->pval = randint1(5) + m_bonus(5, level);
-
-					/* Cursed */
-					if (power < 0)
-						o_ptr->pval = -o_ptr->pval;
-
-					break;
-				}
-
-				/* Amulet of ESP -- never cursed */
-				case SV_AMULET_ESP:
-				{
-					o_ptr->pval = randint1(5) + m_bonus(5, level);
-
-					break;
-				}
-
-				/* Amulet of the Magi -- never cursed */
 				case SV_AMULET_THE_MAGI:
-				{
-					o_ptr->pval = 1 + m_bonus(3, level);
-					o_ptr->to_a = randint1(5) + m_bonus(5, level);
-
-					/* Boost the rating */
-					rating += 25;
-
-					/* Mention the item */
-					if (OPT(cheat_peek)) object_mention(o_ptr);
-
-					break;
-				}
-
-				/* Amulet of Devotion -- never cursed */
 				case SV_AMULET_DEVOTION:
-				{
-					o_ptr->pval = 1 + m_bonus(3, level);
-
-					/* Boost the rating */
-					rating += 25;
-
-					/* Mention the item */
-					if (OPT(cheat_peek)) object_mention(o_ptr);
-
-					break;
-				}
-
-				/* Amulet of Weaponmastery -- never cursed */
 				case SV_AMULET_WEAPONMASTERY:
-				{
-					o_ptr->to_h = 1 + m_bonus(4, level);
-					o_ptr->to_d = 1 + m_bonus(4, level);
-					o_ptr->pval = 1 + m_bonus(2, level);
-
-					/* Boost the rating */
-					rating += 25;
-
-					/* Mention the item */
-					if (OPT(cheat_peek)) object_mention(o_ptr);
-
-					break;
-				}
-
-				/* Amulet of Trickery -- never cursed */
 				case SV_AMULET_TRICKERY:
 				{
-					o_ptr->pval = randint1(1) + m_bonus(3, level);
-
 					/* Boost the rating */
 					rating += 25;
 
@@ -976,10 +739,6 @@ static void a_m_aux_3(object_type *o_ptr, int level, int power)
 			break;
 		}
 	}
-
-	/* Just for now */
-	if (o_ptr->pval < 0)
-		o_ptr->flags[2] |= TR2_LIGHT_CURSE;
 }
 
 
@@ -990,8 +749,6 @@ static void a_m_aux_3(object_type *o_ptr, int level, int power)
  */
 static void a_m_aux_4(object_type *o_ptr, int level, int power)
 {
-	object_kind *k_ptr = &k_info[o_ptr->k_idx];
-
 	/* Unused parameters */
 	(void)level;
 	(void)power;
@@ -999,35 +756,13 @@ static void a_m_aux_4(object_type *o_ptr, int level, int power)
 	/* Apply magic (good or bad) according to type */
 	switch (o_ptr->tval)
 	{
-		case TV_LITE:
+		case TV_LIGHT:
 		{
-			/* Torches and lanterns get half-fuel */
-			if (o_ptr->sval == SV_LITE_TORCH)
-				o_ptr->timeout = FUEL_TORCH;
-			else if (o_ptr->sval == SV_LITE_LANTERN)
-				o_ptr->timeout = FUEL_LAMP / 2;
-
-			break;
-		}
-
-		case TV_WAND:
-		case TV_STAFF:
-		{
-			/* Charge staves and wands */
-			o_ptr->pval = k_ptr->charge_base;
-
-			if (k_ptr->charge_dd && k_ptr->charge_ds)
-				o_ptr->pval += damroll(k_ptr->charge_dd, k_ptr->charge_ds);
-
-			break;
-		}
-
-		case TV_ROD:
-		{
-			object_kind *k_ptr = &k_info[o_ptr->k_idx];
-
-			/* Transfer the pval. */
-			o_ptr->pval = k_ptr->pval;
+			/* Default fuel levels */
+			if (o_ptr->sval == SV_LIGHT_TORCH)
+				o_ptr->timeout = DEFAULT_TORCH;
+			else if (o_ptr->sval == SV_LIGHT_LANTERN)
+				o_ptr->timeout = DEFAULT_LAMP;
 
 			break;
 		}
@@ -1048,101 +783,139 @@ static void a_m_aux_4(object_type *o_ptr, int level, int power)
 	}
 }
 
-static const u32b ego_sustains[] =
+static const int ego_sustains[] =
 {
-	TR1_SUST_STR,
-	TR1_SUST_INT,
-	TR1_SUST_WIS,
-	TR1_SUST_DEX,
-	TR1_SUST_CON,
-	TR1_SUST_CHR,
+	OF_SUST_STR,
+	OF_SUST_INT,
+	OF_SUST_WIS,
+	OF_SUST_DEX,
+	OF_SUST_CON,
+	OF_SUST_CHR,
 };
 
-/*
- * Which TR? flags have the sustains
- */
-unsigned ego_xtra_sustain_idx(void)
+void set_ego_xtra_sustain(bitflag flags[OF_SIZE])
 {
-	return 1;
-}
-
-u32b ego_xtra_sustain_list(void)
-{
-	u32b ret = 0;
 	size_t i;
 
-	for (i = 0; i < N_ELEMENTS(ego_sustains); i++)
-		ret |= ego_sustains[i];
+	of_wipe(flags);
 
-	return ret;
+	for (i = 0; i < N_ELEMENTS(ego_sustains); i++)
+		of_on(flags, ego_sustains[i]);
 }
 
-static const u32b ego_resists[] =
+static const int ego_resists[] =
 {
-	TR1_RES_POIS,
-	TR1_RES_FEAR,
-	TR1_RES_LITE,
-	TR1_RES_DARK,
-	TR1_RES_BLIND,
-	TR1_RES_CONFU,
-	TR1_RES_SOUND,
-	TR1_RES_SHARD,
-	TR1_RES_NEXUS,
-	TR1_RES_NETHR,
-	TR1_RES_CHAOS,
-	TR1_RES_DISEN,
+	OF_RES_POIS,
+	OF_RES_FEAR,
+	OF_RES_LIGHT,
+	OF_RES_DARK,
+	OF_RES_BLIND,
+	OF_RES_CONFU,
+	OF_RES_SOUND,
+	OF_RES_SHARD,
+	OF_RES_NEXUS,
+	OF_RES_NETHR,
+	OF_RES_CHAOS,
+	OF_RES_DISEN,
 };
 
-/*
- * Which TR? flags have the random resists
- */
-unsigned ego_xtra_resist_idx(void)
+void set_ego_xtra_resist(bitflag flags[OF_SIZE])
 {
-	return 1;
-}
-
-u32b ego_xtra_resist_list(void)
-{
-	u32b ret = 0;
 	size_t i;
 
 	for (i = 0; i < N_ELEMENTS(ego_resists); i++)
-		ret |= ego_resists[i];
-
-	return ret;
+		of_on(flags, ego_resists[i]);
 }
 
-static const u32b ego_powers[] =
+static const int ego_powers[] =
 {
-	TR2_SLOW_DIGEST,
-	TR2_FEATHER,
-	TR2_LITE,
-	TR2_REGEN,
-	TR2_TELEPATHY,
-	TR2_SEE_INVIS,
-	TR2_FREE_ACT,
-	TR2_HOLD_LIFE,
+	OF_SLOW_DIGEST,
+	OF_FEATHER,
+	OF_LIGHT,
+	OF_REGEN,
+	OF_TELEPATHY,
+	OF_SEE_INVIS,
+	OF_FREE_ACT,
+	OF_HOLD_LIFE,
 };
 
-/*
- * Which TR? flags have the random powers
- */
-unsigned ego_xtra_power_idx(void)
+void set_ego_xtra_power(bitflag flags[OF_SIZE])
 {
-	return 2;
-}
-
-u32b ego_xtra_power_list(void)
-{
-	u32b ret = 0;
 	size_t i;
 
 	for (i = 0; i < N_ELEMENTS(ego_powers); i++)
-		ret |= ego_powers[i];
-
-	return ret;
+		of_on(flags, ego_powers[i]);
 }
 
+/**
+ * This is a safe way to choose a random new flag to add to an object.
+ * It takes the existing flags, an array of new attrs, and the size of
+ * the array, and returns an entry from attrs, or 0 if there are no
+ * new attrs.
+ */
+static int get_new_attr(bitflag flags[OF_SIZE], const int attrs[], size_t size)
+{
+	size_t i;
+	int options = 0;
+	int flag = 0;
+
+	for (i = 0; i < size; i++)
+	{
+		/* skip this one if the flag is already present */
+		if (of_has(flags, attrs[i])) continue;
+
+		/* each time we find a new possible option, we have a 1-in-N chance of
+		 * choosing it and an (N-1)-in-N chance of keeping a previous one */
+		if (one_in_(++options)) flag = attrs[i];
+	}
+
+	return flag;
+}
+
+
+/*
+ * Prepare an object based on an object kind.
+ * Use the specified randomization aspect
+ */
+void object_prep(object_type *o_ptr, struct object_kind *k, int lev, aspect rand_aspect)
+{
+	/* Clear the record */
+	(void)WIPE(o_ptr, object_type);
+
+	o_ptr->kind = k;
+	o_ptr->k_idx = k->kidx;
+
+	/* Efficiency -- tval/sval */
+	o_ptr->tval = k->tval;
+	o_ptr->sval = k->sval;
+
+	/* Default number */
+	o_ptr->number = 1;
+
+	/* Default "pval" */
+	o_ptr->pval = randcalc(k->pval, lev, rand_aspect);
+
+	/* Default weight */
+	o_ptr->weight = k->weight;
+	
+	/* Assign charges (wands/staves only) */
+	if (o_ptr->tval == TV_WAND || o_ptr->tval == TV_STAFF)
+		o_ptr->pval = randcalc(k->charge, lev, rand_aspect);
+
+	/* Default magic */
+	o_ptr->to_h = randcalc(k->to_h, lev, rand_aspect);
+	o_ptr->to_d = randcalc(k->to_d, lev, rand_aspect);
+	o_ptr->to_a = randcalc(k->to_a, lev, rand_aspect);
+
+	/* Default power */
+	o_ptr->ac = k->ac;
+	o_ptr->dd = k->dd;
+	o_ptr->ds = k->ds;
+
+	/* Hack -- cursed items are always "cursed" */
+	if (of_has(k->flags, OF_LIGHT_CURSE))
+	    of_on(o_ptr->flags, OF_LIGHT_CURSE);
+}
 
 
 /**
@@ -1162,8 +935,8 @@ u32b ego_xtra_power_list(void)
 void apply_magic(object_type *o_ptr, int lev, bool allow_artifacts, bool good, bool great)
 {
 	int power = 0;
-	u32b xtra = 0;
-	bool new = FALSE;
+	/*u32b xtra = 0;*/
+	/*bool new = FALSE;*/
 
 	/* Chance of being `good` and `great` */
 	int good_chance = (lev+2) * 3;
@@ -1195,7 +968,7 @@ void apply_magic(object_type *o_ptr, int lev, bool allow_artifacts, bool good, b
 
 
 	/* Roll for artifact creation */
-	if (allow_artifacts && !o_ptr->name1)
+	if (allow_artifacts)
 	{
 		int i;
 		int rolls = 0;
@@ -1273,7 +1046,7 @@ void apply_magic(object_type *o_ptr, int lev, bool allow_artifacts, bool good, b
 			break;
 		}
 
-		case TV_LITE:
+		case TV_LIGHT:
 		{
 			if (power == 2 || power == -2)
 				make_ego_item(o_ptr, lev, (bool)(power > 0));
@@ -1295,79 +1068,65 @@ void apply_magic(object_type *o_ptr, int lev, bool allow_artifacts, bool good, b
 	if (o_ptr->name2)
 	{
 		ego_item_type *e_ptr = &e_info[o_ptr->name2];
+		bitflag flags[OF_SIZE];
+
+		object_flags(o_ptr, flags);
 
 		/* Extra powers */
-		switch (e_ptr->xtra)
-		{
-			case OBJECT_XTRA_TYPE_SUSTAIN:
-			{
-				while (!new)
-				{
-					xtra = ego_sustains[randint0(N_ELEMENTS(ego_sustains))];
-					if ((o_ptr->flags[1] | xtra) != o_ptr->flags[1])
-					{
-						o_ptr->flags[1] |= xtra;
-						new = TRUE;
-					}
-				}
-				break;
-			}
-
-			case OBJECT_XTRA_TYPE_RESIST:
-			{
-				while (!new)
-				{
-					xtra = ego_resists[randint0(N_ELEMENTS(ego_resists))];
-					if ((o_ptr->flags[1] | xtra) != o_ptr->flags[1])
-					{
-						o_ptr->flags[1] |= xtra;
-						new = TRUE;
-					}
-				}
-				break;
-			}
-
-			case OBJECT_XTRA_TYPE_POWER:
-			{
-				while (!new)
-				{
-					xtra = ego_powers[randint0(N_ELEMENTS(ego_powers))];
-					if ((o_ptr->flags[2] | xtra) != o_ptr->flags[2])
-					{
-						o_ptr->flags[2] |= xtra;
-						new = TRUE;
-					}
-				}
-				break;
-			}
-		}
+		if (e_ptr->xtra == OBJECT_XTRA_TYPE_SUSTAIN)
+			of_on(o_ptr->flags, get_new_attr(flags, ego_sustains,
+											N_ELEMENTS(ego_sustains)));
+		else if (e_ptr->xtra == OBJECT_XTRA_TYPE_RESIST)
+			of_on(o_ptr->flags, get_new_attr(flags, ego_resists,
+											N_ELEMENTS(ego_resists)));
+		else if (e_ptr->xtra == OBJECT_XTRA_TYPE_POWER)
+			of_on(o_ptr->flags, get_new_attr(flags, ego_powers,
+											N_ELEMENTS(ego_powers)));
 
 		/* Hack -- acquire "cursed" flags */
 		if (cursed_p(e_ptr))
-			o_ptr->flags[2] |= (e_ptr->flags[2] & TR2_CURSE_MASK);
+		{
+			bitflag curse_flags[OF_SIZE];
+
+			of_copy(curse_flags, e_ptr->flags);
+			flags_mask(curse_flags, OF_SIZE, OF_CURSE_MASK, FLAG_END);
+			of_union(o_ptr->flags, curse_flags);
+		}
 
 		/* Hack -- apply extra penalties if needed */
 		if (cursed_p(o_ptr))
 		{
-			/* Hack -- obtain bonuses */
-			if (e_ptr->max_to_h > 0) o_ptr->to_h -= randint1(e_ptr->max_to_h);
-			if (e_ptr->max_to_d > 0) o_ptr->to_d -= randint1(e_ptr->max_to_d);
-			if (e_ptr->max_to_a > 0) o_ptr->to_a -= randint1(e_ptr->max_to_a);
+			/* Apply extra ego bonuses */
+			o_ptr->to_h -= randcalc(e_ptr->to_h, lev, RANDOMISE);
+			o_ptr->to_d -= randcalc(e_ptr->to_d, lev, RANDOMISE);
+			o_ptr->to_a -= randcalc(e_ptr->to_a, lev, RANDOMISE);
 
-			/* Hack -- obtain pval */
-			if (e_ptr->max_pval > 0) o_ptr->pval -= randint1(e_ptr->max_pval);
+			/* Apply ego pval */
+			o_ptr->pval -= randcalc(e_ptr->pval, lev, RANDOMISE);
+
+			/* Apply minimums */
+			if (o_ptr->to_h > -1 * e_ptr->min_to_h) o_ptr->to_h = -1 * e_ptr->min_to_h;
+			if (o_ptr->to_d > -1 * e_ptr->min_to_d) o_ptr->to_d = -1 * e_ptr->min_to_d;
+			if (o_ptr->to_a > -1 * e_ptr->min_to_a) o_ptr->to_a = -1 * e_ptr->min_to_a;
+			if (o_ptr->pval > -1 * e_ptr->min_pval) o_ptr->pval = -1 * e_ptr->min_pval;
 		}
 
 		/* Hack -- apply extra bonuses if needed */
 		else
 		{
-			/* Hack -- obtain bonuses */
-			if (e_ptr->max_to_h > 0) o_ptr->to_h += randint1(e_ptr->max_to_h);
-			if (e_ptr->max_to_d > 0) o_ptr->to_d += randint1(e_ptr->max_to_d);
-			if (e_ptr->max_to_a > 0) o_ptr->to_a += randint1(e_ptr->max_to_a);
+			/* Apply extra ego bonuses */
+			o_ptr->to_h += randcalc(e_ptr->to_h, lev, RANDOMISE);
+			o_ptr->to_d += randcalc(e_ptr->to_d, lev, RANDOMISE);
+			o_ptr->to_a += randcalc(e_ptr->to_a, lev, RANDOMISE);
 
-			/* Hack -- obtain pval */
-			if (e_ptr->max_pval > 0) o_ptr->pval += randint1(e_ptr->max_pval);
+			/* Apply ego pval */
+			o_ptr->pval += randcalc(e_ptr->pval, lev, RANDOMISE);
+
+			/* Apply minimums */
+			if (o_ptr->to_h < e_ptr->min_to_h) o_ptr->to_h = e_ptr->min_to_h;
+			if (o_ptr->to_d < e_ptr->min_to_d) o_ptr->to_d = e_ptr->min_to_d;
+			if (o_ptr->to_a < e_ptr->min_to_a) o_ptr->to_a = e_ptr->min_to_a;
+			if (o_ptr->pval < e_ptr->min_pval) o_ptr->pval = e_ptr->min_pval;
 		}
 
 		/* Hack -- apply rating bonus */
@@ -1388,7 +1147,13 @@ void apply_magic(object_type *o_ptr, int lev, bool allow_artifacts, bool good, b
 
 		/* Hack -- acquire "cursed" flag */
 		if (cursed_p(k_ptr))
-			o_ptr->flags[2] |= (k_ptr->flags[2] & TR2_CURSE_MASK);
+		{
+			bitflag curse_flags[OF_SIZE];
+
+			of_copy(curse_flags, k_ptr->flags);
+			flags_mask(curse_flags, OF_SIZE, OF_CURSE_MASK, FLAG_END);
+			of_union(o_ptr->flags, curse_flags);
+		}
 	}
 }
 
@@ -1418,7 +1183,7 @@ static bool kind_is_good(const object_kind *k_ptr)
 		case TV_HELM:
 		case TV_CROWN:
 		{
-			if (k_ptr->to_a < 0) return (FALSE);
+			if (randcalc(k_ptr->to_a, 0, MINIMISE) < 0) return (FALSE);
 			return (TRUE);
 		}
 
@@ -1429,8 +1194,8 @@ static bool kind_is_good(const object_kind *k_ptr)
 		case TV_POLEARM:
 		case TV_DIGGING:
 		{
-			if (k_ptr->to_h < 0) return (FALSE);
-			if (k_ptr->to_d < 0) return (FALSE);
+			if (randcalc(k_ptr->to_h, 0, MINIMISE) < 0) return (FALSE);
+			if (randcalc(k_ptr->to_d, 0, MINIMISE) < 0) return (FALSE);
 			return (TRUE);
 		}
 
@@ -1487,7 +1252,11 @@ bool make_object(object_type *j_ptr, int lev, bool good, bool great)
 
 	/* Try to make a special artifact */
 	if (one_in_(good ? 10 : 1000))
-		return make_artifact_special(j_ptr, lev);
+	{
+		if (make_artifact_special(j_ptr, lev)) return TRUE;
+		/* If we failed to make an artifact, the player gets a great item */
+		good = great = TRUE;
+	}
 
 	/* Base level for the object */
 	base = (good ? (lev + 10) : lev);
@@ -1497,7 +1266,7 @@ bool make_object(object_type *j_ptr, int lev, bool good, bool great)
 	if (!k_idx) return FALSE;
 
 	/* Prepare the object */
-	object_prep(j_ptr, k_idx);
+	object_prep(j_ptr, &k_info[k_idx], lev, RANDOMISE);
 
 	/* Apply magic (allow artifacts) */
 	apply_magic(j_ptr, lev, TRUE, good, great);
@@ -1509,7 +1278,7 @@ bool make_object(object_type *j_ptr, int lev, bool good, bool great)
 	if (k_ptr->gen_mult_prob >= 100 ||
 	    k_ptr->gen_mult_prob >= randint1(100))
 	{
-		j_ptr->number = damroll(k_ptr->gen_dice, k_ptr->gen_side);
+		j_ptr->number = randcalc(k_ptr->stack_size, lev, RANDOMISE);
 	}
 
 
@@ -1517,7 +1286,7 @@ bool make_object(object_type *j_ptr, int lev, bool good, bool great)
 	if (!cursed_p(j_ptr) && (k_info[j_ptr->k_idx].level > p_ptr->depth))
 	{
 		/* Rating increase */
-		rating += (k_info[j_ptr->k_idx].level - p_ptr->depth);
+		rating += (k_info[j_ptr->k_idx].alloc_min - p_ptr->depth);
 
 		/* Cheat -- peek at items */
 		if (OPT(cheat_peek)) object_mention(j_ptr);
@@ -1546,6 +1315,10 @@ void make_gold(object_type *j_ptr, int lev, int coin_type)
 	s32b spread = lev + 10;
 	s32b value = rand_spread(avg, spread);
 
+	/* Increase the range to infinite, moving the average to 110% */
+	while (one_in_(100) && value * 10 <= MAX_SHORT)
+		value *= 10;
+
 	/* Pick a treasure variety scaled by level, or force a type */
 	if (coin_type != SV_GOLD_ANY)
 		sval = coin_type;
@@ -1553,11 +1326,15 @@ void make_gold(object_type *j_ptr, int lev, int coin_type)
 		sval = (((value * 100) / MAX_GOLD_DROP) * SV_GOLD_MAX) / 100;
 
 	/* Do not create illegal treasure types */
-	if (sval > SV_GOLD_MAX) sval = SV_GOLD_MAX;
+	if (sval >= SV_GOLD_MAX) sval = SV_GOLD_MAX - 1;
 
 	/* Prepare a gold object */
 	k_idx = lookup_kind(TV_GOLD, sval);
-	object_prep(j_ptr, k_idx);
+	object_prep(j_ptr, &k_info[k_idx], lev, RANDOMISE);
+
+	/* If we're playing with no_selling, increase the value */
+	if (OPT(adult_no_selling)) value = 5 * value;
+
 	j_ptr->pval = value;
 }
 
