@@ -16,8 +16,10 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-#include "angband.h"
 
+#include "angband.h"
+#include "cave.h"
+#include "squelch.h"
 
 /****** Pathfinding code ******/
 
@@ -877,6 +879,11 @@ static bool run_test(void)
 		row = py + ddy[prev_dir] + ddy[new_dir];
 		col = px + ddx[prev_dir] + ddx[new_dir];
 		
+		/* HACK: Ugh. Sometimes we come up with illegal bounds. This will
+		 * treat the symptom but not the disease. */
+		if (row >= DUNGEON_HGT || col >= DUNGEON_WID) continue;
+		if (row < 0 || col < 0) continue;
+
 		/* Visible monsters abort running */
 		if (cave_m_idx[row][col] > 0)
 		{
@@ -1100,9 +1107,6 @@ void run_step(int dir)
 			}
 
 			p_ptr->run_cur_dir = pf_result[pf_result_index--] - '0';
-
-			/* Hack -- allow OPT(easy_alter) */
-			p_ptr->command_dir = p_ptr->run_cur_dir;
 		}
 	}
 
