@@ -15,10 +15,18 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-#include "angband.h"
-#include "game-cmd.h"
-/*#include "cmds.h"*/
 
+#include "angband.h"
+#include "cave.h"
+#include "game-cmd.h"
+#include "monster/monster.h"
+#include "squelch.h"
+
+/* 
+ * Height of the help screen; any higher than 4 will overlap the health
+ * bar which we want to keep in targeting mode.
+ */
+#define HELP_HEIGHT 3
 
 /*** File-wide variables ***/
 
@@ -47,8 +55,7 @@ static void look_mon_desc(char *buf, size_t max, int m_idx)
 
 
 	/* Determine if the monster is "living" (vs "undead") */
-	if (r_ptr->flags[2] & (RF2_UNDEAD | RF2_DEMON)) living = FALSE;
-	if (strchr("Egv", r_ptr->d_char)) living = FALSE;
+	if (monster_is_unusual(r_ptr)) living = FALSE;
 
 
 	/* Healthy monsters */
@@ -230,61 +237,38 @@ void target_set_location(int y, int x)
  * We use "u" and "v" to point to arrays of "x" and "y" positions,
  * and sort the arrays by double-distance to the player.
  */
-static bool ang_sort_comp_distance(const void *u, const void *v, int a, int b)
+static int cmp_distance(const void *a, const void *b)
 {
 	int py = p_ptr->py;
 	int px = p_ptr->px;
 
-	const byte *x = u;
-	const byte *y = v;
+	const struct point *pa = a;
+	const struct point *pb = b;
 
 	int da, db, kx, ky;
 
 	/* Absolute distance components */
-	kx = x[a]; kx -= px; kx = ABS(kx);
-	ky = y[a]; ky -= py; ky = ABS(ky);
+	kx = pa->x; kx -= px; kx = ABS(kx);
+	ky = pa->y; ky -= py; ky = ABS(ky);
 
 	/* Approximate Double Distance to the first point */
 	da = ((kx > ky) ? (kx + kx + ky) : (ky + ky + kx));
 
 	/* Absolute distance components */
-	kx = x[b]; kx -= px; kx = ABS(kx);
-	ky = y[b]; ky -= py; ky = ABS(ky);
+	kx = pb->x; kx -= px; kx = ABS(kx);
+	ky = pb->y; ky -= py; ky = ABS(ky);
 
 	/* Approximate Double Distance to the first point */
 	db = ((kx > ky) ? (kx + kx + ky) : (ky + ky + kx));
 
 	/* Compare the distances */
-	return (da <= db);
+	if (da < db)
+		return -1;
+	if (da > db)
+		return 1;
+	return 0;
 }
 
-
-/*
- * Sorting hook -- swap function -- by "distance to player"
- *
- * We use "u" and "v" to point to arrays of "x" and "y" positions,
- * and sort the arrays by distance to the player.
- */
-static void ang_sort_swap_distance(void *u, void *v, int a, int b)
-{
-	byte *x = (byte*)(u);
-	byte *y = (byte*)(v);
-
-	byte temp;
-
-	/* Swap "x" */
-	temp = x[a];
-	x[a] = x[b];
-	x[b] = temp;
-
-	/* Swap "y" */
-	temp = y[a];
-	y[a] = y[b];
-	y[b] = temp;
-}
-
-
-
 /*
  * Hack -- help "select" a location (see below)
  */
@@ -407,18 +391,17 @@ static bool target_set_interactive_accept(int y, int x)
 	return (FALSE);
 }
 
-
 /*
  * Prepare the "temp" array for "target_interactive_set"
  *
  * Return the number of target_able monsters in the set.
  */
+/* XXX: Untangle this. The whole temp_* thing is complete madness. */
 static void target_set_interactive_prepare(int mode)
 {
 	int y, x;
-
-	/* Reset "temp" array */
-	temp_n = 0;
+	struct point *pts = mem_zalloc(sizeof(*pts) * SCREEN_HGT * SCREEN_WID);
+	unsigned int n = 0;
 
 	/* Scan the current panel */
 	for (y = Term->offset_y; y < Term->offset_y + SCREEN_HGT; y++)
@@ -442,21 +425,162 @@ static void target_set_interactive_prepare(int mode)
 			}
 
 			/* Save the location */
-			temp_x[temp_n] = x;
-			temp_y[temp_n] = y;
-			temp_n++;
+			pts[n].x = x;
+			pts[n].y = y;
+			n++;
 		}
 	}
 
-	/* Set the sort hooks */
-	ang_sort_comp = ang_sort_comp_distance;
-	ang_sort_swap = ang_sort_swap_distance;
+	sort(pts, n, sizeof(*pts), cmp_distance);
 
-	/* Sort the positions */
-	ang_sort(temp_x, temp_y, temp_n);
+	/* Reset "temp" array */
+	temp_n = n;
+	while (n--) {
+		temp_x[n] = pts[n].x;
+		temp_y[n] = pts[n].y;
+	}
 }
 
 
+/*
+ * Perform the minimum "whole panel" adjustment to ensure that the given
+ * location is contained inside the current panel, and return TRUE if any
+ * such adjustment was performed. Optionally accounts for the targeting
+ * help window.
+ */
+bool adjust_panel_help(int y, int x, bool help)
+{
+	bool changed = FALSE;
+
+	int j;
+
+	int screen_hgt_main = help ? (Term->hgt - ROW_MAP - 3) 
+							   : (Term->hgt - ROW_MAP - 1);
+
+	/* Scan windows */
+	for (j = 0; j < ANGBAND_TERM_MAX; j++)
+	{
+		int wx, wy;
+		int screen_hgt, screen_wid;
+
+		term *t = angband_term[j];
+
+		/* No window */
+		if (!t) continue;
+
+		/* No relevant flags */
+		if ((j > 0) && !(op_ptr->window_flag[j] & PW_MAP)) continue;
+
+		wy = t->offset_y;
+		wx = t->offset_x;
+
+		screen_hgt = (j == 0) ? screen_hgt_main : t->hgt;
+		screen_wid = (j == 0) ? (Term->wid - COL_MAP - 1) : t->wid;
+
+		/* Bigtile panels need adjustment */
+		screen_wid = screen_wid / tile_width;
+		screen_hgt = screen_hgt / tile_height;
+
+		/* Adjust as needed */
+		while (y >= wy + screen_hgt) wy += screen_hgt / 2;
+		while (y < wy) wy -= screen_hgt / 2;
+
+		/* Adjust as needed */
+		while (x >= wx + screen_wid) wx += screen_wid / 2;
+		while (x < wx) wx -= screen_wid / 2;
+
+		/* Use "modify_panel" */
+		if (modify_panel(t, wy, wx)) changed = TRUE;
+	}
+
+	return (changed);
+}
+
+
+/*
+ * Describe a location relative to the player position.
+ * e.g. "12 S 35 W" or "0 N, 33 E" or "0 N, 0 E"
+ */
+void coords_desc(char *buf, int size, int y, int x) {
+	
+	char *east_or_west;
+	char *north_or_south;
+
+	int py = p_ptr->py;
+	int px = p_ptr->px;
+	
+	if (y > py)
+		north_or_south = "S";
+	else
+		north_or_south = "N";
+	
+	if (x < px)
+		east_or_west = "W";
+	else
+		east_or_west = "E";
+	
+	strnfmt(buf, size, "%d %s, %d %s",
+		ABS(y-py), north_or_south, ABS(x-px), east_or_west);
+}
+
+/*
+ * Display targeting help at the bottom of the screen.
+ */
+static void target_display_help(bool monster, bool free)
+{
+	/* Determine help location */
+	int wid, hgt, help_loc;
+	Term_get_size(&wid, &hgt);
+	help_loc = hgt - HELP_HEIGHT;
+	
+	/* Clear */
+	clear_from(help_loc);
+
+	/* Prepare help hooks */
+	text_out_hook = text_out_to_screen;
+	text_out_indent = 1;
+	Term_gotoxy(1, help_loc);
+
+	/* Display help */
+	text_out_c(TERM_L_GREEN, "<dir>");
+	text_out(" and ");
+	text_out_c(TERM_L_GREEN, "<click>");
+	text_out(" look around. '");
+	text_out_c(TERM_L_GREEN, "g");
+	text_out(" moves to the selection. '");
+	text_out_c(TERM_L_GREEN, "p");
+	text_out("' selects the player. '");
+	text_out_c(TERM_L_GREEN, "q");
+	text_out("' exits. '");
+	text_out_c(TERM_L_GREEN, "r");
+	text_out("' displays details. '");
+
+	if (free)
+	{
+		text_out_c(TERM_L_GREEN, "m");
+		text_out("' restricts to interesting places. ");
+	}
+	else
+	{
+		text_out_c(TERM_L_GREEN, "+");
+		text_out("' and '");
+		text_out_c(TERM_L_GREEN, "-");
+		text_out("' cycle through interesting places. '");
+		text_out_c(TERM_L_GREEN, "o");
+		text_out("' allows free selection. ");
+	}
+	
+	if (monster || free)
+	{
+		text_out("'");
+		text_out_c(TERM_L_GREEN, "t");
+		text_out("' targets the current selection.");
+	}
+
+	/* Reset */
+	text_out_indent = 0;
+}
+
 /*
  * Examine a grid, return a keypress.
  *
@@ -478,7 +602,7 @@ static void target_set_interactive_prepare(int mode)
  *
  * This function must handle blindness/hallucination.
  */
-static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr info)
+static ui_event_data target_set_interactive_aux(int y, int x, int mode)
 {
 	s16b this_o_idx = 0, next_o_idx = 0;
 
@@ -497,6 +621,11 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 
 	char out_val[256];
 
+	char coords[20];
+	
+	/* Describe the square location */
+	coords_desc(coords, sizeof(coords), y, x);
+
 	/* Repeat forever */
 	while (1)
 	{
@@ -522,7 +651,6 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 			s2 = "on ";
 		}
 
-
 		/* Hack -- hallucination */
 		if (p_ptr->timed[TMD_IMAGE])
 		{
@@ -532,12 +660,12 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 			if (p_ptr->wizard)
 			{
 				strnfmt(out_val, sizeof(out_val),
-				        "%s%s%s%s [%s] (%d:%d)", s1, s2, s3, name, info, y, x);
+						"%s%s%s%s, %s (%d:%d).", s1, s2, s3, name, coords, y, x);
 			}
 			else
 			{
 				strnfmt(out_val, sizeof(out_val),
-				        "%s%s%s%s [%s]", s1, s2, s3, name, info);
+						"%s%s%s%s, %s.", s1, s2, s3, name, coords);
 			}
 
 			prt(out_val, 0, 0);
@@ -551,7 +679,6 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 			continue;
 		}
 
-
 		/* Actual monsters */
 		if (cave_m_idx[y][x] > 0)
 		{
@@ -592,9 +719,6 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 						/* Recall on screen */
 						screen_roff(m_ptr->r_idx);
 
-						/* Hack -- Complete the prompt (again) */
-						Term_addstr(-1, TERM_WHITE, format("  [r,%s]", info));
-
 						/* Command */
 						query = inkey_ex();
 
@@ -614,14 +738,14 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 						if (p_ptr->wizard)
 						{
 							strnfmt(out_val, sizeof(out_val),
-							        "%s%s%s%s (%s) [r,%s] (%d:%d)",
-						            s1, s2, s3, m_name, buf, info, y, x);
+									"%s%s%s%s (%s), %s (%d:%d).",
+									s1, s2, s3, m_name, buf, coords, y, x);
 						}
 						else
 						{
 							strnfmt(out_val, sizeof(out_val),
-							        "%s%s%s%s (%s) [r,%s]",
-							        s1, s2, s3, m_name, buf, info);
+									"%s%s%s%s (%s), %s.",
+									s1, s2, s3, m_name, buf, coords);
 						}
 
 						prt(out_val, 0, 0);
@@ -650,8 +774,8 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 				s1 = "It is ";
 
 				/* Hack -- take account of gender */
-				if (r_ptr->flags[0] & (RF0_FEMALE)) s1 = "She is ";
-				else if (r_ptr->flags[0] & (RF0_MALE)) s1 = "He is ";
+				if (rf_has(r_ptr->flags, RF_FEMALE)) s1 = "She is ";
+				else if (rf_has(r_ptr->flags, RF_MALE)) s1 = "He is ";
 
 				/* Use a preposition */
 				s2 = "carrying ";
@@ -670,19 +794,20 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 					next_o_idx = o_ptr->next_o_idx;
 
 					/* Obtain an object description */
-					object_desc(o_name, sizeof(o_name), o_ptr, TRUE, ODESC_FULL);
+					object_desc(o_name, sizeof(o_name), o_ptr,
+								ODESC_PREFIX | ODESC_FULL);
 
 					/* Describe the object */
 					if (p_ptr->wizard)
 					{
 						strnfmt(out_val, sizeof(out_val),
-						        "%s%s%s%s [%s] (%d:%d)",
-						        s1, s2, s3, o_name, info, y, x);
+								"%s%s%s%s, %s (%d:%d).",
+								s1, s2, s3, o_name, coords, y, x);
 					}
 					else
 					{
 						strnfmt(out_val, sizeof(out_val),
-						        "%s%s%s%s [%s]", s1, s2, s3, o_name, info);
+								"%s%s%s%s, %s.", s1, s2, s3, o_name, coords);
 					}
 
 					prt(out_val, 0, 0);
@@ -732,14 +857,14 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 				if (p_ptr->wizard)
 				{
 					strnfmt(out_val, sizeof(out_val),
-					        "%s%s%sa pile of %d objects [r,%s] (%d:%d)",
-					        s1, s2, s3, floor_num, info, y, x);
+							"%s%s%sa pile of %d objects, %s (%d:%d).",
+							s1, s2, s3, floor_num, coords, y, x);
 				}
 				else
 				{
 					strnfmt(out_val, sizeof(out_val),
-					        "%s%s%sa pile of %d objects [r,%s]",
-					        s1, s2, s3, floor_num, info);
+							"%s%s%sa pile of %d objects, %s.",
+							s1, s2, s3, floor_num, coords);
 				}
 
 				prt(out_val, 0, 0);
@@ -757,7 +882,7 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 						screen_save();
 
 						/* Display */
-						show_floor(floor_list, floor_num, TRUE);
+						show_floor(floor_list, floor_num, (OLIST_WEIGHT | OLIST_GOLD));
 
 						/* Describe the pile */
 						prt(out_val, 0, 0);
@@ -797,19 +922,20 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 				boring = FALSE;
 
 				/* Obtain an object description */
-				object_desc(o_name, sizeof(o_name), o_ptr, TRUE, ODESC_FULL);
+				object_desc(o_name, sizeof(o_name), o_ptr,
+							ODESC_PREFIX | ODESC_FULL);
 
 				/* Describe the object */
 				if (p_ptr->wizard)
 				{
 					strnfmt(out_val, sizeof(out_val),
-					        "%s%s%s%s [%s] (%d:%d)",
-					        s1, s2, s3, o_name, info, y, x);
+							"%s%s%s%s, %s (%d:%d).",
+							s1, s2, s3, o_name, coords, y, x);
 				}
 				else
 				{
 					strnfmt(out_val, sizeof(out_val),
-					        "%s%s%s%s [%s]", s1, s2, s3, o_name, info);
+							"%s%s%s%s, %s.", s1, s2, s3, o_name, coords);
 				}
 
 				prt(out_val, 0, 0);
@@ -831,6 +957,7 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 				/* Preposition */
 				s2 = "on ";
 			}
+
 		}
 
 		/* Double break */
@@ -850,7 +977,7 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 		/* Terrain feature if needed */
 		if (boring || (feat > FEAT_INVIS))
 		{
-			cptr name = f_name + f_info[feat].name;
+			cptr name = f_info[feat].name;
 
 			/* Hack -- handle unknown grids */
 			if (feat == FEAT_NONE) name = "unknown grid";
@@ -871,12 +998,12 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 			if (p_ptr->wizard)
 			{
 				strnfmt(out_val, sizeof(out_val),
-				        "%s%s%s%s [%s] (%d:%d)", s1, s2, s3, name, info, y, x);
+						"%s%s%s%s, %s (%d:%d).", s1, s2, s3, name, coords, y, x);
 			}
 			else
 			{
 				strnfmt(out_val, sizeof(out_val),
-				        "%s%s%s%s [%s]", s1, s2, s3, name, info);
+						"%s%s%s%s, %s.", s1, s2, s3, name, coords);
 			}
 
 			prt(out_val, 0, 0);
@@ -896,6 +1023,64 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 }
 
 
+bool target_set_closest(int mode)
+{
+	int y, x, m_idx;
+	monster_type *m_ptr;
+	char m_name[80];
+	bool visibility;
+
+	/* Cancel old target */
+	target_set_monster(0);
+
+	/* Get ready to do targetting */
+	target_set_interactive_prepare(mode);
+
+	/* If nothing was prepared, then return */
+	if (temp_n < 1)
+	{
+		msg_print("No Available Target.");
+		return FALSE;
+	}
+
+	/* Find the first monster in the queue */
+	y = temp_y[0];
+	x = temp_x[0];
+	m_idx = cave_m_idx[y][x];
+	
+	/* Target the monster, if possible */
+	if ((m_idx <= 0) || !target_able(m_idx))
+	{
+		msg_print("No Available Target.");
+		return FALSE;
+	}
+
+	/* Target the monster */
+	m_ptr = &mon_list[m_idx];
+	monster_desc(m_name, sizeof(m_name), m_ptr, 0x00);
+	if (!(mode & TARGET_QUIET))
+		msg_format("%^s is targeted.", m_name);
+	Term_fresh();
+
+	/* Set up target information */
+	monster_race_track(m_ptr->r_idx);
+	health_track(cave_m_idx[y][x]);
+	target_set_monster(m_idx);
+
+	/* Visual cue */
+	Term_get_cursor(&visibility);
+	(void)Term_set_cursor(TRUE);
+	move_cursor_relative(y, x);
+	Term_redraw_section(x, y, x, y);
+
+	/* TODO: what's an appropriate amount of time to spend highlighting */
+	Term_xtra(TERM_XTRA_DELAY, 150);
+	(void)Term_set_cursor(visibility);
+
+	return TRUE;
+}
+
+
 
 
 /*
@@ -948,15 +1133,14 @@ static ui_event_data target_set_interactive_aux(int y, int x, int mode, cptr inf
 bool target_set_interactive(int mode, int x, int y)
 {
 	int i, d, m, t, bd;
+	int wid, hgt, help_prompt_loc;
 
 	bool done = FALSE;
-
 	bool flag = TRUE;
+	bool help = FALSE;
 
 	ui_event_data query;
 
-	char info[80];
-
 	/* If we haven't been given an initial location, start on the
 	   player. */
 	if (x == -1 || y == -1)
@@ -974,10 +1158,15 @@ bool target_set_interactive(int mode, int x, int y)
 	/* Cancel target */
 	target_set_monster(0);
 
-
 	/* Cancel tracking */
 	/* health_track(0); */
 
+	/* Calculate the window location for the help prompt */
+	Term_get_size(&wid, &hgt);
+	help_prompt_loc = hgt - 1;
+	
+	/* Display the help prompt */
+	prt("Press '?' for help.", help_prompt_loc, 0);
 
 	/* Prepare the "temp" array */
 	target_set_interactive_prepare(mode);
@@ -994,27 +1183,24 @@ bool target_set_interactive(int mode, int x, int y)
 			y = temp_y[m];
 			x = temp_x[m];
 
-			/* Allow target */
-			if ((cave_m_idx[y][x] > 0) && target_able(cave_m_idx[y][x]))
-			{
-				my_strcpy(info, "g,q,t,p,o,+,-,<dir>, <click>", sizeof(info));
-			}
-
-			/* Dis-allow target */
-			else
-			{
-				my_strcpy(info, "g,q,p,o,+,-,<dir>, <click>", sizeof(info));
-			}
 
 			/* Adjust panel if needed */
-			if (adjust_panel(y, x))
+			if (adjust_panel_help(y, x, help))
 			{
 				/* Handle stuff */
 				handle_stuff();
 			}
+		
+			/* Update help */
+			if (help)
+			{
+				bool good_target = ((cave_m_idx[y][x] > 0) &&
+								target_able(cave_m_idx[y][x]));
+				target_display_help(good_target, !(flag && temp_n));
+			}
 
 			/* Describe and Prompt */
-			query = target_set_interactive_aux(y, x, mode, info);
+			query = target_set_interactive_aux(y, x, mode);
 
 			/* Cancel tracking */
 			/* health_track(0); */
@@ -1022,6 +1208,20 @@ bool target_set_interactive(int mode, int x, int y)
 			/* Assume no "direction" */
 			d = 0;
 
+
+			/* If we click, move the target location to the click and
+			   switch to "free targetting" mode by unsetting 'flag'.
+			   This means we get some info about wherever we've picked. */
+			if (query.type == EVT_MOUSE)
+			{
+				x = KEY_GRID_X(query);
+				y = KEY_GRID_Y(query);
+				flag = FALSE;
+				break;
+			}
+			else
+			{
+
 			/* Analyze */
 			switch (query.key)
 			{
@@ -1073,16 +1273,6 @@ bool target_set_interactive(int mode, int x, int y)
 					break;
 				}
 
-				/* If we click, move the target location to the click and
-				   switch to "free targetting" mode by unsetting 'flag'.
-				   This means we get some info about wherever we've picked. */
-				case '\xff':
-				{
-					x = KEY_GRID_X(query);
-					y = KEY_GRID_Y(query);
-					flag = FALSE;
-					break;
-				}
 				case 't':
 				case '5':
 				case '0':
@@ -1105,10 +1295,25 @@ bool target_set_interactive(int mode, int x, int y)
 
 				case 'g':
 				{
-					cmd_insert(CMD_PATHFIND, y, x);
+					cmd_insert(CMD_PATHFIND);
+					cmd_set_arg_point(cmd_get_top(), 0, y, x);
 					done = TRUE;
 					break;
 				}
+				
+				case '?':
+				{
+					help = !help;
+					
+					/* Redraw main window */
+					p_ptr->redraw |= (PR_BASIC | PR_EXTRA | PR_MAP | PR_EQUIP);
+					Term_clear();
+					handle_stuff();
+					if (!help)
+						prt("Press '?' for help.", help_prompt_loc, 0);
+					
+					break;
+				}
 
 				default:
 				{
@@ -1122,6 +1327,8 @@ bool target_set_interactive(int mode, int x, int y)
 				}
 			}
 
+			}
+
 			/* Hack -- move around */
 			if (d)
 			{
@@ -1166,11 +1373,15 @@ bool target_set_interactive(int mode, int x, int y)
 		/* Arbitrary grids */
 		else
 		{
-			/* Default prompt */
-			my_strcpy(info, "g,q,t,p,m,+,-,<dir>, <click>", sizeof(info));
+			/* Update help */
+			if (help) 
+			{
+				bool good_target = ((cave_m_idx[y][x] > 0) && target_able(cave_m_idx[y][x]));
+				target_display_help(good_target, !(flag && temp_n));
+			}
 
 			/* Describe and Prompt (enable "TARGET_LOOK") */
-			query = target_set_interactive_aux(y, x, mode | TARGET_LOOK, info);
+			query = target_set_interactive_aux(y, x, mode | TARGET_LOOK);
 
 			/* Cancel tracking */
 			/* health_track(0); */
@@ -1178,6 +1389,44 @@ bool target_set_interactive(int mode, int x, int y)
 			/* Assume no direction */
 			d = 0;
 
+			if (query.type == EVT_MOUSE)
+			{
+				/* We only target if we click somewhere where the cursor
+				   is already (i.e. a double-click without a time limit) */
+				if (KEY_GRID_X(query) == x && KEY_GRID_Y(query) == y)
+				{
+					/* Make an attempt to target the monster on the given
+					   square rather than the square itself (it seems this
+					   is the more likely intention of clicking on a 
+					   monster). */
+					int m_idx = cave_m_idx[y][x];
+
+					if ((m_idx > 0) && target_able(m_idx))
+					{
+						health_track(m_idx);
+						target_set_monster(m_idx);
+					}
+					else
+					{
+						/* There is no monster, or it isn't targettable,
+						   so target the location instead. */
+						target_set_location(y, x);
+					}
+
+					done = TRUE;
+				}
+				else
+				{
+					/* Just move the cursor for now - another click will
+					   target. */
+					x = KEY_GRID_X(query);
+					y = KEY_GRID_Y(query);
+				}
+				break;
+			}
+			else
+			{
+
 			/* Analyze the keypress */
 			switch (query.key)
 			{
@@ -1239,41 +1488,6 @@ bool target_set_interactive(int mode, int x, int y)
 					break;
 				}
 
-				case '\xff':
-				{
-					/* We only target if we click somewhere where the cursor
-					   is already (i.e. a double-click without a time limit) */
-					if (KEY_GRID_X(query) == x && KEY_GRID_Y(query) == y)
-					{
-						/* Make an attempt to target the monster on the given
-						   square rather than the square itself (it seems this
-						   is the more likely intention of clicking on a 
-						   monster). */
-						int m_idx = cave_m_idx[y][x];
-
-						if ((m_idx > 0) && target_able(m_idx))
-						{
-							health_track(m_idx);
-							target_set_monster(m_idx);
-						}
-						else
-						{
-							/* There is no monster, or it isn't targettable,
-							   so target the location instead. */
-							target_set_location(y, x);
-						}
-
-						done = TRUE;
-					}
-					else
-					{
-						/* Just move the cursor for now - another click will
-						   target. */
-						x = KEY_GRID_X(query);
-						y = KEY_GRID_Y(query);
-					}
-					break;
-				}
 				case 't':
 				case '5':
 				case '0':
@@ -1286,11 +1500,26 @@ bool target_set_interactive(int mode, int x, int y)
 
 				case 'g':
 				{
-					cmd_insert(CMD_PATHFIND, y, x);
+					cmd_insert(CMD_PATHFIND);
+					cmd_set_arg_point(cmd_get_top(), 0, y, x);
 					done = TRUE;
 					break;
 				}
 
+				case '?':
+				{
+					help = !help;
+					
+					/* Redraw main window */
+					p_ptr->redraw |= (PR_BASIC | PR_EXTRA | PR_MAP | PR_EQUIP);
+					Term_clear();
+					handle_stuff();
+					if (!help)
+						prt("Press '?' for help.", help_prompt_loc, 0);
+					
+					break;
+				}
+
 				default:
 				{
 					/* Extract a direction */
@@ -1303,6 +1532,8 @@ bool target_set_interactive(int mode, int x, int y)
 				}
 			}
 
+			}
+
 			/* Handle "direction" */
 			if (d)
 			{
@@ -1322,7 +1553,7 @@ bool target_set_interactive(int mode, int x, int y)
 				else if (y <= 0) y++;
 
 				/* Adjust panel if needed */
-				if (adjust_panel(y, x))
+				if (adjust_panel_help(y, x, help))
 				{
 					/* Handle stuff */
 					handle_stuff();
@@ -1337,8 +1568,18 @@ bool target_set_interactive(int mode, int x, int y)
 	/* Forget */
 	temp_n = 0;
 
-	/* Clear the top line */
-	prt("", 0, 0);
+	/* Redraw as necessary */
+	if (help)
+	{
+		p_ptr->redraw |= (PR_BASIC | PR_EXTRA | PR_MAP | PR_EQUIP);
+		Term_clear();
+	}
+	else
+	{
+		prt("", 0, 0);
+		prt("", help_prompt_loc, 0);
+		p_ptr->redraw |= (PR_DEPTH | PR_STATUS);
+	}
 
 	/* Recenter around player */
 	verify_panel();
