@@ -30,10 +30,23 @@ static const region SCREEN_REGION = {0, 0, 0, 0};
 /* Erase the contents of a region */
 void region_erase(const region *loc);
 
+/* Erase the contents of a region + 1 char each way */
+void region_erase_bordered(const region *loc);
+
+/* Given a region with relative values, make them absolute */
+region region_calculate(region loc);
+
 /* Check whether a (mouse) event is inside a region */
 bool region_inside(const region *loc, const ui_event_data *key);
 
 
+/*** Text ***/
+
+#include "z-textblock.h"
+void textui_textblock_show(textblock *tb, region orig_area, const char *header);
+void textui_textblock_place(textblock *tb, region orig_area, const char *header);
+
+
 /*** Misc ***/
 
 void window_make(int origin_x, int origin_y, int end_x, int end_y);
