@@ -35,29 +35,13 @@ const char *copyright =
 	"   and not for profit purposes provided that this copyright and statement\n"
 	"   are included in all such copies.  Other copyrights may also apply.\n";
 
-
-/*
- * Executable version
- */
-byte version_major = VERSION_MAJOR;
-byte version_minor = VERSION_MINOR;
-byte version_patch = VERSION_PATCH;
-byte version_extra = VERSION_EXTRA;
-
-/*
- * Savefile version
- */
-byte sf_major;			/* Savefile's "version_major" */
-byte sf_minor;			/* Savefile's "version_minor" */
-byte sf_patch;			/* Savefile's "version_patch" */
-byte sf_extra;			/* Savefile's "version_extra" */
-
 /*
  * Run-time arguments
  */
 bool arg_wizard;			/* Command arg -- Request wizard mode */
 bool arg_rebalance;			/* Command arg -- Rebalance monsters */
 int arg_graphics;			/* Command arg -- Request graphics mode */
+bool arg_graphics_nice;			/* Command arg -- Request nice graphics mode */
 
 /*
  * Various things
@@ -85,7 +69,9 @@ s32b old_turn;			/* Hack -- Level feeling counter */
 
 
 int use_graphics;		/* The "graphics" mode is enabled */
-bool use_bigtile = FALSE;
+bool use_graphics_nice;	        /* The 'nice' "graphics" mode is enabled */
+byte tile_width = 1;            /* Tile width in units of font width */
+byte tile_height = 1;           /* Tile height in units of font height */
 
 s16b signal_count;		/* Hack -- Count interrupts */
 
@@ -112,11 +98,6 @@ s16b mon_max = 1;	/* Number of allocated monsters */
 s16b mon_cnt = 0;	/* Number of live monsters */
 
 
-/*
- * TRUE if process_command() is a repeated call.
- */
-bool command_repeating = FALSE;
-
 
 /*
  * Dungeon variables
@@ -134,24 +115,6 @@ bool closing_flag;		/* Dungeon is closing */
  * Buffer to hold the current savefile name
  */
 char savefile[1024];
-char panic_savefile[1024];
-
-
-/*
- * Number of active macros.
- */
-s16b macro__num;
-
-/*
- * Array of macro patterns [MACRO_MAX]
- */
-char **macro__pat;
-
-/*
- * Array of macro actions [MACRO_MAX]
- */
-char **macro__act;
-
 
 /*
  * The array[ANGBAND_TERM_MAX] of window pointers
@@ -176,38 +139,136 @@ char angband_term_name[ANGBAND_TERM_MAX][16] =
 	"Term-7"
 };
 
-
-int max_macrotrigger = 0;
-char *macro_template = NULL;
-char *macro_modifier_chr;
-char *macro_modifier_name[MAX_MACRO_MOD];
-char *macro_trigger_name[MAX_MACRO_TRIGGER];
-char *macro_trigger_keycode[2][MAX_MACRO_TRIGGER];
-
-
 /*
  * Global table of color definitions (mostly zeros)
  */
 byte angband_color_table[MAX_COLORS][4] =
 {
-	{0x00, 0x00, 0x00, 0x00},	/* TERM_DARK */
-	{0x00, 0xFF, 0xFF, 0xFF},	/* TERM_WHITE */
-	{0x00, 0x80, 0x80, 0x80},	/* TERM_SLATE */
-	{0x00, 0xFF, 0x80, 0x00},	/* TERM_ORANGE */
-	{0x00, 0xC0, 0x00, 0x00},	/* TERM_RED */
-	{0x00, 0x00, 0x80, 0x40},	/* TERM_GREEN */
-	{0x00, 0x00, 0x40, 0xFF},	/* TERM_BLUE */
-	{0x00, 0x80, 0x40, 0x00},	/* TERM_UMBER */
-	{0x00, 0x60, 0x60, 0x60},	/* TERM_L_DARK */
-	{0x00, 0xC0, 0xC0, 0xC0},	/* TERM_L_WHITE */
-	{0x00, 0xFF, 0x00, 0xFF},	/* TERM_VIOLET */
-	{0x00, 0xFF, 0xFF, 0x00},	/* TERM_YELLOW */
-	{0x00, 0xFF, 0x40, 0x40},	/* TERM_L_RED */
-	{0x00, 0x00, 0xFF, 0x00},	/* TERM_L_GREEN */
-	{0x00, 0x00, 0xFF, 0xFF},	/* TERM_L_BLUE */
-	{0x00, 0xC0, 0x80, 0x40}	/* TERM_L_UMBER */
+	{0x00, 0x00, 0x00, 0x00}, /* 0  TERM_DARK */
+	{0x00, 0xff, 0xff, 0xff}, /* 1  TERM_WHITE */
+	{0x00, 0x80, 0x80, 0x80}, /* 2  TERM_SLATE */
+	{0x00, 0xff, 0x80, 0x00}, /* 3  TERM_ORANGE */
+	{0x00, 0xc0, 0x00, 0x00}, /* 4  TERM_RED */
+	{0x00, 0x00, 0x80, 0x40}, /* 5  TERM_GREEN */
+	{0x00, 0x00, 0x40, 0xff}, /* 6  TERM_BLUE */
+	{0x00, 0x80, 0x40, 0x00}, /* 7  TERM_UMBER */
+	{0x00, 0x60, 0x60, 0x60}, /* 8  TERM_L_DARK */
+	{0x00, 0xc0, 0xc0, 0xc0}, /* 9  TERM_L_WHITE */
+	{0x00, 0xff, 0x00, 0xff}, /* 10 TERM_L_PURPLE */
+	{0x00, 0xff, 0xff, 0x00}, /* 11 TERM_YELLOW */
+	{0x00, 0xff, 0x40, 0x40}, /* 12 TERM_L_RED */
+	{0x00, 0x00, 0xff, 0x00}, /* 13 TERM_L_GREEN */
+	{0x00, 0x00, 0xff, 0xff}, /* 14 TERM_L_BLUE */
+	{0x00, 0xc0, 0x80, 0x40}, /* 15 TERM_L_UMBER */
+	{0x00, 0x90, 0x00, 0x90}, /* 16 TERM_PURPLE */
+	{0x00, 0x90, 0x20, 0xff}, /* 17 TERM_VIOLET */
+	{0x00, 0x00, 0xa0, 0xa0}, /* 18 TERM_TEAL */
+	{0x00, 0x6c, 0x6c, 0x30}, /* 19 TERM_MUD */
+	{0x00, 0xff, 0xff, 0x90}, /* 20 TERM_L_YELLOW */
+	{0x00, 0xff, 0x00, 0xa0}, /* 21 TERM_MAGENTA */
+	{0x00, 0x20, 0xff, 0xdc}, /* 22 TERM_L_TEAL */
+	{0x00, 0xb8, 0xa8, 0xff}, /* 23 TERM_L_VIOLET */
+	{0x00, 0xff, 0x80, 0x80}, /* 24 TERM_L_PINK */
+	{0x00, 0xb4, 0xb4, 0x00}, /* 25 TERM_MUSTARD */
+	{0x00, 0xa0, 0xc0, 0xd0}, /* 26 TERM_BLUE_SLATE */
+	{0x00, 0x00, 0xb0, 0xff}, /* 27 TERM_DEEP_L_BLUE */
 };
 
+/*
+ * Global array of color names and translations.
+ */
+color_type color_table[MAX_COLORS] =
+{
+	/* full mono vga blind lighter darker highlight metallic misc */
+	{'d', "Dark", {0, 0, 0, TERM_DARK, TERM_L_DARK, TERM_DARK,
+				   TERM_L_DARK, TERM_L_DARK, TERM_DARK}},
+
+	{'w', "White", {1, 1, 1, TERM_WHITE, TERM_YELLOW, TERM_SLATE,
+					TERM_L_BLUE, TERM_YELLOW, TERM_WHITE}},
+
+	{'s', "Slate", {2, 1, 2, TERM_SLATE, TERM_L_WHITE, TERM_L_DARK,
+					TERM_L_WHITE, TERM_L_WHITE, TERM_SLATE}},
+
+	{'o', "Orange", {3, 1, 3, TERM_L_WHITE, TERM_YELLOW, TERM_SLATE,
+					 TERM_YELLOW, TERM_YELLOW, TERM_ORANGE}},
+
+	{'r', "Red", {4, 1, 4, TERM_SLATE, TERM_L_RED, TERM_SLATE,
+				  TERM_L_RED, TERM_L_RED, TERM_RED}},
+
+	{'g', "Green", {5, 1, 5, TERM_SLATE, TERM_L_GREEN, TERM_SLATE,
+					TERM_L_GREEN, TERM_L_GREEN, TERM_GREEN}},
+
+	{'b', "Blue", {6, 1, 6, TERM_SLATE, TERM_L_BLUE, TERM_SLATE,
+				   TERM_L_BLUE, TERM_L_BLUE, TERM_BLUE}},
+
+	{'u', "Umber", {7, 1, 7, TERM_L_DARK, TERM_L_UMBER, TERM_L_DARK,
+					TERM_L_UMBER, TERM_L_UMBER, TERM_UMBER}},
+
+	{'D', "Light Dark", {8, 1, 8, TERM_L_DARK, TERM_SLATE, TERM_L_DARK,
+						 TERM_SLATE, TERM_SLATE, TERM_L_DARK}},
+
+	{'W', "Light Slate", {9, 1, 9, TERM_L_WHITE, TERM_WHITE, TERM_SLATE,
+						  TERM_WHITE, TERM_WHITE, TERM_SLATE}},
+
+	{'P', "Light Purple", {10, 1, 10, TERM_SLATE, TERM_YELLOW, TERM_SLATE,
+						   TERM_YELLOW, TERM_YELLOW, TERM_L_PURPLE}},
+
+	{'y', "Yellow", {11, 1, 11, TERM_L_WHITE, TERM_L_YELLOW, TERM_L_WHITE,
+					 TERM_WHITE, TERM_WHITE, TERM_YELLOW}},
+
+	{'R', "Light Red", {12, 1, 12, TERM_L_WHITE, TERM_YELLOW, TERM_RED,
+						TERM_YELLOW, TERM_YELLOW, TERM_L_RED}},
+
+	{'G', "Light Green", {13, 1, 13, TERM_L_WHITE, TERM_YELLOW, TERM_GREEN,
+						  TERM_YELLOW, TERM_YELLOW, TERM_L_GREEN}},
+
+	{'B', "Light Blue", {14, 1, 14, TERM_L_WHITE, TERM_YELLOW, TERM_BLUE,
+						 TERM_YELLOW, TERM_YELLOW, TERM_L_BLUE}},
+
+	{'U', "Light Umber", {15, 1, 15, TERM_L_WHITE, TERM_YELLOW, TERM_UMBER,
+						  TERM_YELLOW, TERM_YELLOW, TERM_L_UMBER}},
+
+	/* "new" colors */
+	{'p', "Purple", {16, 1, 10,TERM_SLATE, TERM_L_PURPLE, TERM_SLATE,
+					 TERM_L_PURPLE, TERM_L_PURPLE, TERM_L_PURPLE}},
+
+	{'v', "Violet", {17, 1, 10,TERM_SLATE, TERM_L_PURPLE, TERM_SLATE,
+					 TERM_L_PURPLE, TERM_L_PURPLE, TERM_L_PURPLE}},
+
+	{'t', "Teal", {18, 1, 6, TERM_SLATE, TERM_L_TEAL, TERM_SLATE,
+				   TERM_L_TEAL, TERM_L_TEAL, TERM_L_BLUE}},
+
+	{'m', "Mud", {19, 1, 5, TERM_SLATE, TERM_MUSTARD, TERM_SLATE,
+				  TERM_MUSTARD, TERM_MUSTARD, TERM_UMBER}},
+
+	{'Y', "Light Yellow", {20, 1, 11, TERM_WHITE, TERM_WHITE, TERM_YELLOW,
+						   TERM_WHITE, TERM_WHITE, TERM_L_YELLOW}},
+
+	{'i', "Magenta-Pink", {21, 1, 12, TERM_SLATE, TERM_L_PINK, TERM_RED,
+						   TERM_L_PINK, TERM_L_PINK, TERM_L_PURPLE}},
+
+	{'T', "Light Teal", {22, 1, 14, TERM_L_WHITE, TERM_YELLOW, TERM_TEAL,
+						 TERM_YELLOW, TERM_YELLOW, TERM_L_BLUE}},
+
+	{'V', "Light Violet", {23, 1, 10, TERM_L_WHITE, TERM_YELLOW, TERM_VIOLET,
+						   TERM_YELLOW, TERM_YELLOW, TERM_L_PURPLE}},
+
+	{'I', "Light Pink", {24, 1, 12, TERM_L_WHITE, TERM_YELLOW, TERM_MAGENTA,
+						 TERM_YELLOW, TERM_YELLOW, TERM_L_PURPLE}},
+
+	{'M', "Mustard", {25, 1, 11, TERM_SLATE, TERM_YELLOW, TERM_SLATE,
+					  TERM_YELLOW, TERM_YELLOW, TERM_YELLOW}},
+
+	{'z', "Blue Slate",  {26, 1, 9, TERM_SLATE, TERM_DEEP_L_BLUE, TERM_SLATE,
+						  TERM_DEEP_L_BLUE, TERM_DEEP_L_BLUE, TERM_L_WHITE}},
+
+	{'Z', "Deep Light Blue", {27, 1, 14, TERM_L_WHITE, TERM_L_BLUE, TERM_BLUE_SLATE,
+							  TERM_L_BLUE, TERM_L_BLUE, TERM_L_BLUE}},
+
+	/* Rest to be filled in when the game loads */
+};
+
+
 
 /*
  * Standard sound (and message) names
@@ -469,10 +530,14 @@ quest *q_list;
 store_type *store;
 
 /*
- * Array[INVEN_TOTAL] of objects in the player's inventory
+ * Flag to override which store is selected if in a knowledge menu
  */
-object_type *inventory;
+int store_knowledge = STORE_NONE;
 
+/*
+ * Array[RANDNAME_NUM_TYPES][num_names] of random names
+ */
+cptr** name_sections;
 
 /*
  * The size of the "alloc_ego_table"
@@ -498,7 +563,7 @@ alloc_entry *alloc_race_table;
 
 /*
  * Specify attr/char pairs for visual special effects
- * Be sure to use "index & 0xFF" to avoid illegal access
+ * Be sure to use "index & 0xff" to avoid illegal access
  */
 byte misc_to_attr[256];
 char misc_to_char[256];
@@ -564,87 +629,50 @@ maxima *z_info;
  * The vault generation arrays
  */
 vault_type *v_info;
-char *v_name;
-char *v_text;
 
-/*
- * The terrain feature arrays
- */
 feature_type *f_info;
-char *f_name;
-char *f_text;
 
-/*
- * The object kind arrays
- */
 object_kind *k_info;
-char *k_name;
-char *k_text;
 
 /*
  * The artifact arrays
  */
 artifact_type *a_info;
-char *a_name;
-char *a_text;
 
 /*
  * The ego-item arrays
  */
 ego_item_type *e_info;
-char *e_name;
-char *e_text;
 flag_cache *slay_cache;
 
 /*
  * The monster race arrays
  */
 monster_race *r_info;
-char *r_name;
-char *r_text;
 
-
-/*
- * The player race arrays
- */
 player_race *p_info;
-char *p_name;
-char *p_text;
-
-/*
- * The player class arrays
- */
 player_class *c_info;
-char *c_name;
-char *c_text;
-
 /*
  * The player history arrays
  */
 hist_type *h_info;
-char *h_text;
 
-/*
- * The shop owner arrays
- */
 owner_type *b_info;
-char *b_name;
-char *b_text;
 
 /*
  * The object flavor arrays
  */
 flavor_type *flavor_info;
-char *flavor_name;
-char *flavor_text;
 
 /*
  * The spell arrays
  */
 spell_type *s_info;
-char *s_name;
-char *s_text;
 
+/*
+ * The hints array
+ */
+struct hint *hints;
 
 /*
  * The spell_list is built from s_info to facilitate a quick lookup
@@ -666,17 +694,9 @@ const char *ANGBAND_SYS = "xxx";
 const char *ANGBAND_GRAF = "old";
 
 /*
- * Path name: The main "lib" directory
- * This variable is not actually used anywhere in the code
- */
-char *ANGBAND_DIR;
-
-/*
- * Various lib/ sub-directories.
+ * Various directories. These are no longer necessarily all subdirs of "lib"
  */
 char *ANGBAND_DIR_APEX;
-char *ANGBAND_DIR_BONE;
-char *ANGBAND_DIR_DATA;
 char *ANGBAND_DIR_EDIT;
 char *ANGBAND_DIR_FILE;
 char *ANGBAND_DIR_HELP;
@@ -756,6 +776,11 @@ int text_out_wrap = 0;
  */
 int text_out_indent = 0;
 
+/*
+ * Hack -- Padding after wrapping
+ */
+int text_out_pad = 0;
+
 
 /*
  * Use transparent tiles
@@ -779,3 +804,8 @@ u16b inscriptions_count = 0;
 /* Delay in centiseconds before moving to allow another keypress */
 /* Zero means normal instant movement. */
 u16b lazymove_delay = 0;
+
+
+/* Number of days passed on the current dungeon trip -
+  - used for determining store updates on return to town */
+u16b daycount = 0;
