@@ -15,10 +15,12 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
+
 #include "angband.h"
+#include "cave.h"
 #include "effects.h"
 #include "object/tvalsval.h"
-
+#include "spells.h"
 
 /*
  * The defines below must match the spell numbers in spell.txt
@@ -97,8 +99,7 @@
 #define PRAYER_BLESS                    2
 #define PRAYER_REMOVE_FEAR              3
 #define PRAYER_CALL_LIGHT               4
-#define PRAYER_FIND_TRAPS               5
-#define PRAYER_DETECT_DOORS_STAIRS      6
+#define PRAYER_FIND_TRAPS_DOORS         5
 #define PRAYER_SLOW_POISON              7
 
 /* Words of Wisdom */
@@ -195,9 +196,9 @@ int get_spell_index(const object_type *o_ptr, int index)
 cptr get_spell_name(int tval, int spell)
 {
 	if (tval == TV_MAGIC_BOOK)
-		return s_name + s_info[spell].name;
+		return s_info[spell].name;
 	else
-		return s_name + s_info[spell + PY_MAX_SPELLS].name;
+		return s_info[spell + PY_MAX_SPELLS].name;
 }
 
 
@@ -341,7 +342,7 @@ void get_spell_info(int tval, int spell, char *p, size_t len)
 				break;
 			case PRAYER_ORB_OF_DRAINING:
 				strnfmt(p, len, " %d+3d6", plev +
-				        (plev / ((cp_ptr->flags & CF_BLESS_WEAPON) ? 2 : 4)));
+				        (plev / (player_has(PF_BLESS_WEAPON) ? 2 : 4)));
 				break;
 			case PRAYER_CURE_CRITICAL_WOUNDS:
 				my_strcpy(p, " heal 25%", len);
@@ -404,7 +405,7 @@ void get_spell_info(int tval, int spell, char *p, size_t len)
 static int beam_chance(void)
 {
 	int plev = p_ptr->lev;
-	return ((cp_ptr->flags & CF_BEAM) ? plev : (plev / 2));
+	return (player_has(PF_BEAM) ? plev : (plev / 2));
 }
 
 
@@ -510,7 +511,7 @@ static bool cast_mage_spell(int spell, int dir)
 
 		case SPELL_LIGHT_AREA:
 		{
-			(void)lite_area(damroll(2, (plev / 2)), (plev / 10) + 1);
+			(void)light_area(damroll(2, (plev / 2)), (plev / 10) + 1);
 			break;
 		}
 
@@ -545,7 +546,7 @@ static bool cast_mage_spell(int spell, int dir)
 
 		case SPELL_CONFUSE_MONSTER:
 		{
-			(void)confuse_monster(dir, plev);
+			(void)confuse_monster(dir, plev, TRUE);
 			break;
 		}
 
@@ -563,7 +564,7 @@ static bool cast_mage_spell(int spell, int dir)
 
 		case SPELL_SLEEP_MONSTER:
 		{
-			(void)sleep_monster(dir);
+			(void)sleep_monster(dir, TRUE);
 			break;
 		}
 
@@ -582,7 +583,7 @@ static bool cast_mage_spell(int spell, int dir)
 		case SPELL_SPEAR_OF_LIGHT:
 		{
 			msg_print("A line of blue shimmering light appears.");
-			lite_line(dir);
+			light_line(dir);
 			break;
 		}
 
@@ -629,7 +630,7 @@ static bool cast_mage_spell(int spell, int dir)
 
 		case SPELL_MASS_SLEEP:
 		{
-			(void)sleep_monsters();
+			(void)sleep_monsters(TRUE);
 			break;
 		}
 
@@ -754,6 +755,7 @@ static bool cast_mage_spell(int spell, int dir)
 		}
 		case SPELL_DETECT_INVISIBLE:
 		{
+			(void)detect_monsters_normal(TRUE);
 			(void)detect_monsters_invis(TRUE);
 			break;
 		}
@@ -932,18 +934,13 @@ static bool cast_priest_spell(int spell, int dir)
 
 		case PRAYER_CALL_LIGHT:
 		{
-			(void)lite_area(damroll(2, (plev / 2)), (plev / 10) + 1);
+			(void)light_area(damroll(2, (plev / 2)), (plev / 10) + 1);
 			break;
 		}
 
-		case PRAYER_FIND_TRAPS:
+		case PRAYER_FIND_TRAPS_DOORS:
 		{
 			(void)detect_traps(TRUE);
-			break;
-		}
-
-		case PRAYER_DETECT_DOORS_STAIRS:
-		{
 			(void)detect_doorstairs(TRUE);
 			break;
 		}
@@ -956,7 +953,7 @@ static bool cast_priest_spell(int spell, int dir)
 
 		case PRAYER_SCARE_MONSTER:
 		{
-			(void)fear_monster(dir, plev);
+			(void)fear_monster(dir, plev, TRUE);
 			break;
 		}
 
@@ -983,7 +980,7 @@ static bool cast_priest_spell(int spell, int dir)
 
 		case PRAYER_SANCTUARY:
 		{
-			(void)sleep_monsters_touch();
+			(void)sleep_monsters_touch(TRUE);
 			break;
 		}
 
@@ -1016,7 +1013,7 @@ static bool cast_priest_spell(int spell, int dir)
 		{
 			fire_ball(GF_HOLY_ORB, dir,
 			          (damroll(3, 6) + plev +
-			           (plev / ((cp_ptr->flags & CF_BLESS_WEAPON) ? 2 : 4))),
+			           (plev / (player_has(PF_BLESS_WEAPON) ? 2 : 4))),
 			          ((plev < 30) ? 2 : 3));
 			break;
 		}
@@ -1071,7 +1068,7 @@ static bool cast_priest_spell(int spell, int dir)
 
 		case PRAYER_TURN_UNDEAD:
 		{
-			(void)turn_undead();
+			(void)turn_undead(TRUE);
 			break;
 		}
 
@@ -1150,7 +1147,7 @@ static bool cast_priest_spell(int spell, int dir)
 
 		case PRAYER_CLAIRVOYANCE:
 		{
-			wiz_lite();
+			wiz_light();
 			break;
 		}
 
