@@ -17,16 +17,18 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-#include "angband.h"
 
+#include "angband.h"
+#include "button.h"
+#include "cave.h"
+#include "files.h"
 #include "game-event.h"
 #include "game-cmd.h"
-
+#include "monster/monster.h"
+#include "object/tvalsval.h"
 #include "textui.h"
 #include "ui-birth.h"
 
-#include "object/tvalsval.h"
-
 /* 
  * There are a few functions installed to be triggered by several 
  * of the basic player events.  For convenience, these have been grouped 
@@ -156,7 +158,7 @@ static void prt_title(int row, int col)
 	/* Normal */
 	else
 	{
-		p = c_text + cp_ptr->title[(p_ptr->lev - 1) / 5];
+		p = cp_ptr->title[(p_ptr->lev - 1) / 5];
 	}
 
 	prt_field(p, row, col);
@@ -243,13 +245,13 @@ static void prt_equippy(int row, int col)
 	object_type *o_ptr;
 
 	/* No equippy chars in bigtile mode */
-	if (use_bigtile) return;
+	if ((tile_width > 1) || (tile_height > 1)) return;
 
 	/* Dump equippy chars */
 	for (i = INVEN_WIELD; i < INVEN_TOTAL; i++)
 	{
 		/* Object */
-		o_ptr = &inventory[i];
+		o_ptr = &p_ptr->inventory[i];
 
 		a = object_attr(o_ptr);
 		c = object_char(o_ptr);
@@ -339,8 +341,8 @@ static void prt_sp(int row, int col)
 	char cur_sp[32], max_sp[32];
 	byte color = player_sp_attr();
 
-	/* Do not show mana unless it matters */
-	if (!cp_ptr->spell_book) return;
+	/* Do not show mana unless we have some */
+	if (!p_ptr->msp) return;
 
 	put_str("SP ", row, col);
 
@@ -366,8 +368,8 @@ byte monster_health_attr(void)
 	
 	/* Tracking an unseen, hallucinatory, or dead monster */
 	else if ((!mon_list[p_ptr->health_who].ml) ||
-		    (p_ptr->timed[TMD_IMAGE]) ||
-		    (!mon_list[p_ptr->health_who].hp < 0))
+			(p_ptr->timed[TMD_IMAGE]) ||
+			(mon_list[p_ptr->health_who].hp < 0))
 	{
 		/* The monster health is "unknown" */
 		attr = TERM_WHITE;
@@ -436,8 +438,8 @@ static void prt_health(int row, int col)
 
 	/* Tracking an unseen, hallucinatory, or dead monster */
 	else if ((!mon_list[p_ptr->health_who].ml) || /* Unseen */
-		    (p_ptr->timed[TMD_IMAGE]) || /* Hallucination */
-		    (!mon_list[p_ptr->health_who].hp < 0)) /* Dead (?) */
+			(p_ptr->timed[TMD_IMAGE]) || /* Hallucination */
+			(mon_list[p_ptr->health_who].hp < 0)) /* Dead (?) */
 	{
 		/* The monster health is "unknown" */
 		Term_putstr(col, row, 12, attr, "[----------]");
@@ -532,8 +534,8 @@ static void prt_wis(int row, int col) { prt_stat(A_WIS, row, col); }
 static void prt_int(int row, int col) { prt_stat(A_INT, row, col); }
 static void prt_con(int row, int col) { prt_stat(A_CON, row, col); }
 static void prt_chr(int row, int col) { prt_stat(A_CHR, row, col); }
-static void prt_race(int row, int col) { prt_field(p_name + rp_ptr->name, row, col); }
-static void prt_class(int row, int col) { prt_field(c_name + cp_ptr->name, row, col); }
+static void prt_race(int row, int col) { prt_field(rp_ptr->name, row, col); }
+static void prt_class(int row, int col) { prt_field(cp_ptr->name, row, col); }
 
 
 /*
@@ -633,7 +635,7 @@ static void hp_colour_change(game_event_type type, game_event_data *data, void *
 	 */
 	if ((OPT(hp_changes_color)) && (arg_graphics == GRAPHICS_NONE))
 	{
-		lite_spot(p_ptr->py, p_ptr->px);
+		light_spot(p_ptr->py, p_ptr->px);
 	}
 }
 
@@ -873,12 +875,14 @@ static size_t prt_state(int row, int col)
 	}
 
 	/* Repeating */
-	else if (p_ptr->command_rep)
+	else if (cmd_get_nrepeats())
 	{
-		if (p_ptr->command_rep > 999)
-			strnfmt(text, sizeof(text), "Rep. %3d00", p_ptr->command_rep / 100);
+		int nrepeats = cmd_get_nrepeats();
+
+		if (nrepeats > 999)
+			strnfmt(text, sizeof(text), "Rep. %3d00", nrepeats / 100);
 		else
-			strnfmt(text, sizeof(text), "Repeat %3d", p_ptr->command_rep);
+			strnfmt(text, sizeof(text), "Repeat %3d", nrepeats);
 	}
 
 	/* Searching */
@@ -1044,14 +1048,24 @@ static void update_maps(game_event_type type, game_event_data *data, void *user)
 			vy = ky + ROW_MAP;
 			vx = kx + COL_MAP;
 
-			if (use_bigtile) vx += kx;
+		      if (tile_width > 1)
+		      {
+			      vx += (tile_width - 1) * kx;
+		      }
+		      if (tile_height > 1)
+		      {
+			      vy += (tile_height - 1) * ky;
+		      }
 		}
 		else
 		{
-			if (use_bigtile)
+			if (tile_width > 1)
 			{
-				kx += kx;
-				if (kx + 1 >= t->wid) return;
+			        kx += (tile_width - 1) * kx;
+			}
+			if (tile_height > 1)
+			{
+			        ky += (tile_height - 1) * ky;
 			}
 			
 			/* Verify location */
@@ -1073,15 +1087,9 @@ static void update_maps(game_event_type type, game_event_data *data, void *user)
 		Term_queue_char(t, vx, vy, TERM_L_GREEN, c, ta, tc);
 #endif
 		
-		if (use_bigtile)
+		if ((tile_width > 1) || (tile_height > 1))
 		{
-			vx++;
-			
-			/* Mega-Hack : Queue dummy char */
-			if (a & 0x80)
-				Term_queue_char(t, vx, vy, 255, -1, 0, 0);
-			else
-				Term_queue_char(t, vx, vy, TERM_WHITE, ' ', TERM_WHITE, ' ');
+		        Term_big_queue_char(t, vx, vy, a, c, TERM_WHITE, ' ');
 		}
 	}
 }
@@ -1106,9 +1114,9 @@ static void update_inven_subwindow(game_event_type type, game_event_data *data,
 	Term_activate(inv_term);
 
 	if (!flip_inven)
-		display_inven();
+		show_inven(OLIST_WINDOW | OLIST_WEIGHT | OLIST_QUIVER);
 	else
-		display_equip();
+		show_equip(OLIST_WINDOW | OLIST_WEIGHT);
 
 	Term_fresh();
 	
@@ -1126,9 +1134,9 @@ static void update_equip_subwindow(game_event_type type, game_event_data *data,
 	Term_activate(inv_term);
 
 	if (!flip_inven)
-		display_equip();
+		show_equip(OLIST_WINDOW | OLIST_WEIGHT);
 	else
-		display_inven();
+		show_inven(OLIST_WINDOW | OLIST_WEIGHT | OLIST_QUIVER);
 
 	Term_fresh();
 	
@@ -1155,18 +1163,18 @@ void toggle_inven_equip(void)
 		if (op_ptr->window_flag[i] & PW_INVEN)
 		{
 			if (!flip_inven)
-				display_inven();
+				show_inven(OLIST_WINDOW | OLIST_WEIGHT | OLIST_QUIVER);
 			else
-				display_equip();
+				show_equip(OLIST_WINDOW | OLIST_WEIGHT);
 			
 			Term_fresh();
 		}
 		else if (op_ptr->window_flag[i] & PW_EQUIP)
 		{
 			if (!flip_inven)
-				display_equip();
+				show_equip(OLIST_WINDOW | OLIST_WEIGHT);
 			else
-				display_inven();
+				show_inven(OLIST_WINDOW | OLIST_WEIGHT | OLIST_QUIVER);
 			
 			Term_fresh();
 		}
@@ -1386,8 +1394,8 @@ static void update_player_compact_subwindow(game_event_type type, game_event_dat
 	Term_activate(inv_term);
 
 	/* Race and Class */
-	prt_field(p_name + rp_ptr->name, row++, col);
-	prt_field(c_name + cp_ptr->name, row++, col);
+	prt_field(rp_ptr->name, row++, col);
+	prt_field(cp_ptr->name, row++, col);
 
 	/* Title */
 	prt_title(row++, col);
@@ -1694,6 +1702,13 @@ static void show_splashscreen(game_event_type type, game_event_data *data, void
 		/* Dump the file to the screen */
 		while (file_getl(fp, buf, sizeof(buf)))
 		{
+			char *version_marker = strstr(buf, "$VERSION");
+			if (version_marker)
+			{
+				ptrdiff_t pos = version_marker - buf;
+				strnfmt(version_marker, sizeof(buf) - pos, "%-8s", VERSION_STRING);
+			}
+
 			text_out_e("%s", buf);
 			text_out("\n");
 		}
@@ -1803,3 +1818,15 @@ void init_display(void)
 
 	ui_init_birthstate_handlers();
 }
+
+
+/* Return a random hint from the global hints list */
+char* random_hint(void)
+{
+	struct hint *v, *r = NULL;
+	int n;
+	for (v = hints, n = 1; v; v = v->next, n++)
+		if (one_in_(n))
+			r = v;
+	return r->hint;
+}
