@@ -15,18 +15,14 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-#include "z-file.h"
-#include "z-virt.h"
-#include "z-util.h"
-#include "z-form.h"
+#include "angband.h"
 
-#ifndef RISCOS
-# include <sys/types.h>
-#endif
+#include <sys/types.h>
 
 #ifdef WINDOWS
 # include <windows.h>
 # include <io.h>
+# include <direct.h>
 #endif
 
 #ifdef MACH_O_CARBON
@@ -44,6 +40,15 @@
 
 #ifdef HAVE_STAT
 # include <sys/stat.h>
+# include <sys/types.h>
+#endif
+
+#ifdef WINDOWS
+# define my_mkdir(path, perms) mkdir(path)
+#elif HAVE_MKDIR || MACH_O_CARBON
+# define my_mkdir(path, perms) mkdir(path, perms)
+#else
+# define my_mkdir(path, perms) FALSE
 #endif
 
 /*
@@ -104,18 +109,8 @@ void safe_setuid_grab(void)
  */
 static void path_parse(char *buf, size_t max, cptr file)
 {
-#ifndef RISCOS
-
 	/* Accept the filename */
 	my_strcpy(buf, file, max);
-
-#else /* RISCOS */
-
-	/* Defined in main-ros.c */
-	char *riscosify_name(const char *path);
-	my_strcpy(buf, riscosify_name(path), max);
-
-#endif /* !RISCOS */
 }
 
 
@@ -150,11 +145,11 @@ static void path_process(char *buf, size_t len, size_t *cur_len, const char *pat
 #ifndef MACH_O_CARBON
 
 		/* Look up a user (or "current" user) */
-		if (username[0]) pw = getpwnam(username);
-		else             pw = getpwuid(getuid());
+		pw = username[0] ? getpwnam(username) : getpwuid(getuid());
 
 #else /* MACH_O_CARBON */
 
+		{
 		/* On Macs getlogin() can incorrectly return root, so get the username via system frameworks */
 		CFStringRef cfusername = CSCopyUserName(TRUE);
 		CFIndex cfbufferlength = CFStringGetMaximumSizeForEncoding(CFStringGetLength(cfusername), kCFStringEncodingUTF8) + 1;
@@ -165,6 +160,7 @@ static void path_process(char *buf, size_t len, size_t *cur_len, const char *pat
 		/* Look up the user */
 		pw = getpwnam(macusername);
 		mem_free(macusername);
+		}
 #endif /* !MACH_O_CARBON */
 
 		if (!pw) return;
@@ -184,7 +180,6 @@ static void path_process(char *buf, size_t len, size_t *cur_len, const char *pat
 
 
 
-
 /*
  * Create a new path string by appending a 'leaf' to 'base'.
  *
@@ -325,49 +320,34 @@ bool file_exists(const char *fname)
 
 #endif
 
-
-#ifndef RISCOS
-#ifdef HAVE_STAT
-
 /*
  * Return TRUE if first is newer than second, FALSE otherwise.
  */
 bool file_newer(const char *first, const char *second)
 {
-	struct stat first_stat, second_stat;
+#ifdef HAVE_STAT
+	struct stat stat1, stat2;
 
-	bool second_exists = stat(second, &second_stat) ? FALSE : TRUE;
-	bool first_exists = stat(first, &first_stat) ? FALSE : TRUE;
+	/* If the first doesn't exist, the first is not newer. */
+	if (stat(first, &stat1) != 0) return FALSE;
 
-	/*
-	 * If the first doesn't exist, the first is not newer;
-	 * If the second doesn't exist, the first is always newer.
-	 */
-	if (!first_exists)  return FALSE;
-	if (!second_exists) return TRUE;
-
-	if (first_stat.st_mtime >= second_stat.st_mtime)
-		return TRUE;
-
-	return FALSE;
-}
+	/* If the second doesn't exist, the first is always newer. */
+	if (stat(second, &stat2) != 0) return TRUE;
 
+	/* Compare modification times. */
+	return stat1.st_mtime > stat2.st_mtime ? TRUE : FALSE;
 #else /* HAVE_STAT */
-
-bool file_newer(const char *first, const char *second)
-{
-	/* Assume newer */
 	return FALSE;
+#endif /* !HAVE_STAT */
 }
 
-#endif /* HAVE_STAT */
-#endif /* RISCOS */
-
 
 
 
 /** File-handle functions **/
 
+void (*file_open_hook)(const char *path, file_type ftype);
+
 /*
  * Open file 'fname', in mode 'mode', with filetype 'ftype'.
  * Returns file handle or NULL.
@@ -375,7 +355,6 @@ bool file_newer(const char *first, const char *second)
 ang_file *file_open(const char *fname, file_mode mode, file_type ftype)
 {
 	ang_file *f = ZNEW(ang_file);
-	char modestr[3] = "__";
 	char buf[1024];
 
 	(void)ftype;
@@ -385,24 +364,12 @@ ang_file *file_open(const char *fname, file_mode mode, file_type ftype)
 
 	switch (mode)
 	{
-		case MODE_WRITE:
-			modestr[0] = 'w';
-			modestr[1] = 'b';
-			break;
-		case MODE_READ:
-			modestr[0] = 'r';
-			modestr[1] = 'b';
-			break;
-		case MODE_APPEND:
-			modestr[0] = 'a';
-			modestr[1] = '+';
-			break;
-		default:
-			break;
+		case MODE_WRITE:  f->fh = fopen(buf, "wb"); break;
+		case MODE_READ:   f->fh = fopen(buf, "rb"); break;
+		case MODE_APPEND: f->fh = fopen(buf, "a+"); break;
+		default:          f->fh = fopen(buf, "__");
 	}
 
-	f->fh = fopen(buf, modestr);
-
 	if (f->fh == NULL)
 	{
 		FREE(f);
@@ -412,30 +379,13 @@ ang_file *file_open(const char *fname, file_mode mode, file_type ftype)
 	f->fname = string_make(buf);
 	f->mode = mode;
 
-#ifdef MACH_O_CARBON
-	extern void fsetfileinfo(cptr path, u32b fcreator, u32b ftype);
-
-	/* OS X uses its own kind of filetypes */
-	if (mode != MODE_READ)
-	{
-		u32b mac_type = 'TEXT';
-
-		if (ftype == FTYPE_RAW) mac_type = 'DATA';
-		else if (ftype == FTYPE_SAVE) mac_type = 'SAVE';
-
-		fsetfileinfo(buf, 'A271', mac_type);
-	}
-#endif
-
-#if defined(RISCOS) && 0
-	/* do something for RISC OS here? */
-	if (mode != MODE_READ)
-		File_SetType(n, ftype);
-#endif
+	if (mode != MODE_READ && file_open_hook)
+		file_open_hook(buf, ftype);
 
 	return f;
 }
 
+
 /*
  * Close file handle 'f'.
  */
@@ -619,7 +569,6 @@ bool file_write(ang_file *f, const char *buf, size_t n)
 
 #endif 
 
-
 /** Line-based IO **/
 
 /*
@@ -636,6 +585,8 @@ bool file_getl(ang_file *f, char *buf, size_t len)
 	byte b;
 	size_t i = 0;
 
+	bool check_encodes = FALSE;
+
 	/* Leave a byte for the terminating 0 */
 	size_t max_len = len - 1;
 
@@ -685,15 +636,26 @@ bool file_getl(ang_file *f, char *buf, size_t len)
 		}
 
 		/* Ignore non-printables */
-		if (!isprint((unsigned char) c))
+		else if (my_isprint((unsigned char)c))
+  		{
+			buf[i++] = c;
+
+			/* Notice possible encode */
+ 			if (c == '[') check_encodes = TRUE;
+
+			continue;
+		}
+		else
 		{
 			buf[i++] = '?';
 			continue;
 		}
-
-		buf[i++] = c;
 	}
 
+	/* Translate encodes if necessary */
+ 	if (check_encodes) xstr_trans(buf, LATIN1);
+
+	buf[i] = '\0';
 	return TRUE;
 }
 
@@ -722,6 +684,94 @@ bool file_putf(ang_file *f, const char *fmt, ...)
 }
 
 
+/*
+ * Format and translate a string, then print it out to file.
+ */
+bool x_file_putf(ang_file *f, int encoding, const char *fmt, ...)
+{
+	va_list vp;
+
+ 	char buf[1024];
+
+ 	/* Begin the Varargs Stuff */
+ 	va_start(vp, fmt);
+
+ 	/* Format the args, save the length */
+ 	(void)vstrnfmt(buf, sizeof(buf), fmt, vp);
+
+ 	/* End the Varargs Stuff */
+ 	va_end(vp);
+
+ 	/* Translate*/
+ 	xstr_trans(buf, encoding);
+
+ 	return file_put(f, buf);
+}
+
+
+bool dir_exists(const char *path)
+{
+	#ifdef HAVE_STAT
+	struct stat buf;
+	if (stat(path, &buf) != 0)
+		return FALSE;
+	else if (buf.st_mode & S_IFDIR)
+		return TRUE;
+	else
+		return FALSE;
+	#else
+	return TRUE;
+	#endif
+}
+
+#ifdef HAVE_STAT
+bool dir_create(const char *path)
+{
+	const char *ptr;
+	char buf[512];
+
+	/* If the directory already exists then we're done */
+	if (dir_exists(path)) return TRUE;
+
+	#ifdef WINDOWS
+	/* If we're on windows, we need to skip past the "C:" part. */
+	if (isalpha(path[0]) && path[1] == ':') path += 2;
+	#endif
+
+	/* Iterate through the path looking for path segements. At each step,
+	 * create the path segment if it doesn't already exist. */
+	for (ptr = path; *ptr; ptr++)
+	{
+		if (*ptr == PATH_SEPC)
+		{
+			/* Find the length of the parent path string */
+			size_t len = (size_t)(ptr - path);
+
+			/* Skip the initial slash */
+			if (len == 0) continue;
+
+			/* If this is a duplicate path separator, continue */
+			if (*(ptr - 1) == PATH_SEPC) continue;
+
+			/* We can't handle really big filenames */
+			if (len - 1 > 512) return FALSE;
+
+			/* Create the parent path string, plus null-padding */
+			my_strcpy(buf, path, len + 1);
+
+			/* Skip if the parent exists */
+			if (dir_exists(buf)) continue;
+
+			/* The parent doesn't exist, so create it or fail */
+			if (my_mkdir(buf, 0755) != 0) return FALSE;
+		}
+	}
+	return my_mkdir(path, 0755) == 0 ? TRUE : FALSE;
+}
+
+#else /* HAVE_STAT */
+bool dir_create(const char *path) { return FALSE; }
+#endif /* !HAVE_STAT */
 
 /*** Directory scanning API ***/
 
