@@ -15,11 +15,7 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
-#include "z-form.h"
-#include "z-type.h"
-
-#include "z-util.h"
-#include "z-virt.h"
+#include "angband.h"
 
 
 /*
@@ -545,6 +541,12 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 				cptr arg;
 				char arg2[1024];
 
+				/* XXX There is a big bug here: if one
+				 * passes "%.0s" to strnfmt, then really we
+				 * should not dereference the arg at all.
+				 * But it does.  See bug #666.
+				 */
+
 				/* Get the next argument */
 				arg = tval.t == T_END ? va_arg(vp, cptr) : tval.u.s;
 
@@ -554,6 +556,9 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 				/* Prevent buffer overflows */
 				(void)my_strcpy(arg2, arg, sizeof(arg2));
 
+				/* Translate it to 8-bit (Latin-1) */
+ 				xstr_trans(arg2, LATIN1);
+
 				/* Format the argument */
 				sprintf(tmp, aux, arg2);
 
@@ -619,11 +624,11 @@ size_t vstrnfmt(char *buf, size_t max, cptr fmt, va_list vp)
 			for (q = 0; tmp[q]; q++)
 			{
 				/* Notice first non-space */
-				if (!isspace((unsigned char)tmp[q]))
+				if (!my_isspace((unsigned char)tmp[q]))
 				{
 					/* Capitalize if possible */
-					if (islower((unsigned char)tmp[q]))
-						tmp[q] = toupper((unsigned char)tmp[q]);
+					if (my_islower((unsigned char)tmp[q]))
+						tmp[q] = my_toupper((unsigned char)tmp[q]);
 
 					/* Done */
 					break;
@@ -691,7 +696,8 @@ char *vformat(cptr fmt, va_list vp)
 	if (!format_buf)
 	{
 		format_len = 1024;
-		C_MAKE(format_buf, format_len, char);
+		format_buf = mem_zalloc(format_len);
+		format_buf[0] = 0;
 	}
 
 	/* Null format yields last result */
@@ -700,18 +706,20 @@ char *vformat(cptr fmt, va_list vp)
 	/* Keep going until successful */
 	while (1)
 	{
+		va_list	args;
 		size_t len;
 
 		/* Build the string */
-		len = vstrnfmt(format_buf, format_len, fmt, vp);
+		VA_COPY(args, vp);
+		len = vstrnfmt(format_buf, format_len, fmt, args);
+		va_end(args);
 
 		/* Success */
 		if (len < format_len-1) break;
 
 		/* Grow the buffer */
-		KILL(format_buf);
 		format_len = format_len * 2;
-		C_MAKE(format_buf, format_len, char);
+		format_buf = mem_realloc(format_buf, format_len);
 	}
 
 	/* Return the new buffer */
@@ -720,7 +728,7 @@ char *vformat(cptr fmt, va_list vp)
 
 void vformat_kill(void)
 {
-	KILL(format_buf);
+	FREE(format_buf);
 }
 
 
