@@ -1,18 +1,21 @@
-/* File: z-term.c */
-
 /*
+ * File: z-term.c
+ * Purpose: a generic, efficient, terminal window package
+ *
  * Copyright (c) 1997 Ben Harrison
  *
- * This software may be copied and distributed for educational, research,
- * and not for profit purposes provided that this copyright and statement
- * are included in all such copies.
+ * This work is free software; you can redistribute it and/or modify it
+ * under the terms of either:
+ *
+ * a) the GNU General Public License as published by the Free Software
+ *    Foundation, version 2, or
+ *
+ * b) the "Angband licence":
+ *    This software may be copied and distributed for educational, research,
+ *    and not for profit purposes provided that this copyright and statement
+ *    are included in all such copies.  Other copyrights may also apply.
  */
-
-/* Purpose: a generic, efficient, terminal window package -BEN- */
-
-#include "z-term.h"
-
-#include "z-virt.h"
+#include "angband.h"
 
 
 /*
@@ -289,20 +292,20 @@ term *Term = NULL;
 static errr term_win_nuke(term_win *s)
 {
 	/* Free the window access arrays */
-	KILL(s->a);
-	KILL(s->c);
+	FREE(s->a);
+	FREE(s->c);
 
 	/* Free the window content arrays */
-	KILL(s->va);
-	KILL(s->vc);
+	FREE(s->va);
+	FREE(s->vc);
 
 	/* Free the terrain access arrays */
-	KILL(s->ta);
-	KILL(s->tc);
+	FREE(s->ta);
+	FREE(s->tc);
 
 	/* Free the terrain content arrays */
-	KILL(s->vta);
-	KILL(s->vtc);
+	FREE(s->vta);
+	FREE(s->vtc);
 
 	/* Success */
 	return (0);
@@ -317,20 +320,20 @@ static errr term_win_init(term_win *s, int w, int h)
 	int y;
 
 	/* Make the window access arrays */
-	C_MAKE(s->a, h, byte*);
-	C_MAKE(s->c, h, char*);
+	s->a = C_ZNEW(h, byte *);
+	s->c = C_ZNEW(h, char *);
 
 	/* Make the window content arrays */
-	C_MAKE(s->va, h * w, byte);
-	C_MAKE(s->vc, h * w, char);
+	s->va = C_ZNEW(h * w, byte);
+	s->vc = C_ZNEW(h * w, char);
 
 	/* Make the terrain access arrays */
-	C_MAKE(s->ta, h, byte*);
-	C_MAKE(s->tc, h, char*);
+	s->ta = C_ZNEW(h, byte *);
+	s->tc = C_ZNEW(h, char *);
 
 	/* Make the terrain content arrays */
-	C_MAKE(s->vta, h * w, byte);
-	C_MAKE(s->vtc, h * w, char);
+	s->vta = C_ZNEW(h * w, byte);
+	s->vtc = C_ZNEW(h * w, char);
 
 	/* Prepare the window access arrays */
 	for (y = 0; y < h; y++)
@@ -518,6 +521,54 @@ void Term_queue_char(term *t, int x, int y, byte a, char c, byte ta, char tc)
 	if (x > t->x2[y]) t->x2[y] = x;
 }
 
+/* Queue a large-sized tile */
+
+void Term_big_queue_char(term *t, int x, int y, byte a, char c, byte a1, char c1)
+{
+        int hor, vert;
+
+	/* Avoid warning */
+	(void)c;
+
+	/* No tall skinny tiles */
+	if (tile_width > 1)
+	{
+	        /* Horizontal first */
+	        for (hor = 0; hor <= tile_width; hor++)
+		{
+		        /* Queue dummy character */
+		        if (hor != 0)
+			{
+			        if (a & 0x80)
+				        Term_queue_char(t, x + hor, y, 255, -1, 0, 0);
+				else
+				        Term_queue_char(t, x + hor, y, TERM_WHITE, ' ', a1, c1);
+			}
+
+			/* Now vertical */
+			for (vert = 1; vert <= tile_height; vert++)
+			{
+			        /* Queue dummy character */
+			        if (a & 0x80)
+				        Term_queue_char(t, x + hor, y + vert, 255, -1, 0, 0);
+				else
+				        Term_queue_char(t, x + hor, y + vert, TERM_WHITE, ' ', a1, c1);
+			}
+		}
+	}
+	else
+	{
+	        /* Only vertical */
+	        for (vert = 1; vert <= tile_height; vert++)
+		{
+		        /* Queue dummy character */
+		        if (a & 0x80)
+			        Term_queue_char(t, x, y + vert, 255, -1, 0, 0);
+			else
+			        Term_queue_char(t, x, y + vert, TERM_WHITE, ' ', a1, c1);
+		}
+	}
+}
 
 /*
  * Mentally draw some attr/chars at a given location
@@ -551,7 +602,7 @@ void Term_queue_chars(int x, int y, int n, byte a, cptr s)
 
 		/* Save the "literal" information */
 		scr_aa[x] = a;
-		scr_cc[x] = *s;
+		scr_cc[x] = xchar_trans(*s);
 
 		scr_taa[x] = 0;
 		scr_tcc[x] = 0;
@@ -770,9 +821,6 @@ static void Term_fresh_row_both(int y, int x1, int x2)
 		/* Handle high-bit attr/chars */
 		if ((na & 0x80) && (nc & 0x80))
 		{
-			/* 2nd byte of bigtile */
-			if ((na == 255) && (nc == (char) -1)) continue;
-
 			/* Flush */
 			if (fn)
 			{
@@ -792,6 +840,9 @@ static void Term_fresh_row_both(int y, int x1, int x2)
 				fn = 0;
 			}
 
+			/* 2nd byte of bigtile */
+			if ((na == 255) && (nc == (char) -1)) continue;
+
 			/* Hack -- Draw the special attr/char pair */
 			(void)((*Term->pict_hook)(x, y, 1, &na, &nc, &nta, &ntc));
 
@@ -969,8 +1020,9 @@ static void Term_fresh_row_text(int y, int x1, int x2)
 	}
 }
 
-
-
+/* Helper variables for large cursor */
+bool bigcurs = FALSE;
+bool smlcurs = TRUE;
 
 
 /*
@@ -1302,7 +1354,9 @@ errr Term_fresh(void)
 		/* Draw the cursor */
 		if (!scr->cu && scr->cv)
 		{
-			if ((scr->cx + 1 < w) && (old->a[scr->cy][scr->cx + 1] == 255))
+		        if ((((tile_width > 1)||(tile_height > 1)) && 
+			     (!smlcurs) && (Term->saved == 0) && (scr->cy > 0))
+			    || bigcurs)
 			{
 				/* Double width cursor for the Bigtile mode */
 				(void)((*Term->bigcurs_hook)(scr->cx, scr->cy));
@@ -1499,7 +1553,7 @@ errr Term_addch(byte a, char c)
  * positive value, future calls to either function will
  * return negative ones.
  */
-errr Term_addstr(int n, byte a, cptr s)
+errr Term_addstr(int n, byte a, cptr buf)
 {
 	int k;
 
@@ -1507,6 +1561,14 @@ errr Term_addstr(int n, byte a, cptr s)
 
 	errr res = 0;
 
+	char s[1024];
+
+	/* Copy to a rewriteable string */
+ 	my_strcpy(s, buf, 1024);
+
+ 	/* Translate it to 7-bit ASCII or system-specific format */
+ 	xstr_trans(s, LATIN1);
+
 	/* Handle "unusable" cursor */
 	if (Term->scr->cu) return (-1);
 
@@ -1551,6 +1613,57 @@ errr Term_putch(int x, int y, byte a, char c)
 }
 
 
+/*
+ * Move to a location and, using an attr, add a big tile
+ */
+void Term_big_putch(int x, int y, byte a, char c)
+{
+        int hor, vert;
+
+	/* Avoid warning */
+	(void)c;
+
+	/* No tall skinny tiles */
+	if (tile_width > 1)
+	{
+	        /* Horizontal first */
+	        for (hor = 0; hor <= tile_width; hor++)
+		{
+		        /* Queue dummy character */
+		        if (hor != 0)
+			{	
+			        if (a & 0x80)
+				        Term_putch(x + hor, y, 255, -1);
+				else
+				        Term_putch(x + hor, y, TERM_WHITE, ' ');
+			}
+
+			/* Now vertical */
+			for (vert = 1; vert <= tile_height; vert++)
+			{
+			        /* Queue dummy character */
+			        if (a & 0x80)
+				        Term_putch(x + hor, y + vert, 255, -1);
+				else
+				        Term_putch(x + hor, y + vert, TERM_WHITE, ' ');
+			}
+		}
+	}
+	else
+	{
+	        /* Only vertical */
+	        for (vert = 1; vert <= tile_height; vert++)
+		{
+		        /* Queue dummy character */
+		        if (a & 0x80)
+			        Term_putch(x, y + vert, 255, -1);
+			else
+			        Term_putch(x, y + vert, TERM_WHITE, ' ');
+		}
+	}
+}
+
+
 /*
  * Move to a location and, using an attr, add a string
  */
@@ -1896,7 +2009,7 @@ errr Term_keypress(int k)
 errr Term_mousepress(int x, int y, char button)
 {
   /* Store the char, advance the queue */
-  Term->key_queue[Term->key_head].key = '\xff';
+  Term->key_queue[Term->key_head].key = 0;
   Term->key_queue[Term->key_head].mousex = x;
   Term->key_queue[Term->key_head].mousey = y;
   Term->key_queue[Term->key_head].index = button;
@@ -1924,7 +2037,7 @@ errr Term_mousepress(int x, int y, char button)
  */
 errr Term_key_push(int k)
 {
-	event_type ke;
+	ui_event_data ke;
 
 	if (!k) return (-1);
 
@@ -1935,7 +2048,7 @@ errr Term_key_push(int k)
 	return Term_event_push(&ke);
 }
 
-errr Term_event_push(const event_type *ke)
+errr Term_event_push(const ui_event_data *ke)
 {
 	/* Hack -- Refuse to enqueue non-keys */
 	if (!ke) return (-1);
@@ -1973,7 +2086,7 @@ errr Term_event_push(const event_type *ke)
  *
  * Remove the keypress if "take" is true.
  */
-errr Term_inkey(event_type *ch, bool wait, bool take)
+errr Term_inkey(ui_event_data *ch, bool wait, bool take)
 {
 	/* Assume no key */
 	ch->type = ch->key = 0;
@@ -2037,7 +2150,7 @@ errr Term_save(void)
 	term_win *mem;
 
 	/* Allocate window */
-	MAKE(mem, term_win);
+	mem = ZNEW(term_win);
 
 	/* Initialize window */
 	term_win_init(mem, w, h);
@@ -2049,6 +2162,9 @@ errr Term_save(void)
 	mem->next = Term->mem;
 	Term->mem = mem;
 
+	/* One more saved */
+	Term->saved++;
+
 	/* Success */
 	return (0);
 }
@@ -2096,6 +2212,9 @@ errr Term_load(void)
 	Term->y1 = 0;
 	Term->y2 = h - 1;
 
+	/* One less saved */
+	Term->saved--;
+
 	/* Success */
 	return (0);
 }
@@ -2119,7 +2238,7 @@ errr Term_resize(int w, int h)
 	term_win *hold_mem;
 	term_win *hold_tmp;
 
-	event_type evt = { EVT_RESIZE, 0, 0, 0, 0 };
+	ui_event_data evt = { EVT_RESIZE, 0, 0, 0, 0 };
 
 
 	/* Resizing is forbidden */
@@ -2155,11 +2274,11 @@ errr Term_resize(int w, int h)
 	hold_tmp = Term->tmp;
 
 	/* Create new scanners */
-	C_MAKE(Term->x1, h, byte);
-	C_MAKE(Term->x2, h, byte);
+	Term->x1 = C_ZNEW(h, byte);
+	Term->x2 = C_ZNEW(h, byte);
 
 	/* Create new window */
-	MAKE(Term->old, term_win);
+	Term->old = ZNEW(term_win);
 
 	/* Initialize new window */
 	term_win_init(Term->old, w, h);
@@ -2168,7 +2287,7 @@ errr Term_resize(int w, int h)
 	term_win_copy(Term->old, hold_old, wid, hgt);
 
 	/* Create new window */
-	MAKE(Term->scr, term_win);
+	Term->scr = ZNEW(term_win);
 
 	/* Initialize new window */
 	term_win_init(Term->scr, w, h);
@@ -2180,7 +2299,7 @@ errr Term_resize(int w, int h)
 	if (hold_mem)
 	{
 		/* Create new window */
-		MAKE(Term->mem, term_win);
+		Term->mem = ZNEW(term_win);
 
 		/* Initialize new window */
 		term_win_init(Term->mem, w, h);
@@ -2193,7 +2312,7 @@ errr Term_resize(int w, int h)
 	if (hold_tmp)
 	{
 		/* Create new window */
-		MAKE(Term->tmp, term_win);
+		Term->tmp = ZNEW(term_win);
 
 		/* Initialize new window */
 		term_win_init(Term->tmp, w, h);
@@ -2347,13 +2466,13 @@ errr term_nuke(term *t)
 	term_win_nuke(t->old);
 
 	/* Kill "displayed" */
-	KILL(t->old);
+	FREE(t->old);
 
 	/* Nuke "requested" */
 	term_win_nuke(t->scr);
 
 	/* Kill "requested" */
-	KILL(t->scr);
+	FREE(t->scr);
 
 	/* If needed */
 	if (t->mem)
@@ -2362,7 +2481,7 @@ errr term_nuke(term *t)
 		term_win_nuke(t->mem);
 
 		/* Kill "memorized" */
-		KILL(t->mem);
+		FREE(t->mem);
 	}
 
 	/* If needed */
@@ -2372,15 +2491,15 @@ errr term_nuke(term *t)
 		term_win_nuke(t->tmp);
 
 		/* Kill "temporary" */
-		KILL(t->tmp);
+		FREE(t->tmp);
 	}
 
 	/* Free some arrays */
-	KILL(t->x1);
-	KILL(t->x2);
+	FREE(t->x1);
+	FREE(t->x2);
 
 	/* Free the input queue */
-	KILL(t->key_queue);
+	FREE(t->key_queue);
 
 	/* Success */
 	return (0);
@@ -2409,7 +2528,7 @@ errr term_init(term *t, int w, int h, int k)
 	t->key_size = k;
 
 	/* Allocate the input queue */
-	C_MAKE(t->key_queue, t->key_size, event_type);
+	t->key_queue = C_ZNEW(t->key_size, ui_event_data);
 
 
 	/* Save the size */
@@ -2417,19 +2536,19 @@ errr term_init(term *t, int w, int h, int k)
 	t->hgt = h;
 
 	/* Allocate change arrays */
-	C_MAKE(t->x1, h, byte);
-	C_MAKE(t->x2, h, byte);
+	t->x1 = C_ZNEW(h, byte);
+	t->x2 = C_ZNEW(h, byte);
 
 
 	/* Allocate "displayed" */
-	MAKE(t->old, term_win);
+	t->old = ZNEW(term_win);
 
 	/* Initialize "displayed" */
 	term_win_init(t->old, w, h);
 
 
 	/* Allocate "requested" */
-	MAKE(t->scr, term_win);
+	t->scr = ZNEW(term_win);
 
 	/* Initialize "requested" */
 	term_win_init(t->scr, w, h);
@@ -2455,6 +2574,8 @@ errr term_init(term *t, int w, int h, int k)
 	t->attr_blank = 0;
 	t->char_blank = ' ';
 
+	/* No saves yet */
+	t->saved = 0;
 
 	/* Success */
 	return (0);
