@@ -211,6 +211,9 @@ struct term
 	term_win *tmp;
 	term_win *mem;
 
+        /* Number of times saved */
+        byte saved;
+
 	void (*init_hook)(term *t);
 	void (*nuke_hook)(term *t);
 
@@ -227,6 +230,8 @@ struct term
 	errr (*text_hook)(int x, int y, int n, byte a, cptr s);
 
 	errr (*pict_hook)(int x, int y, int n, const byte *ap, const char *cp, const byte *tap, const char *tcp);
+
+	byte (*xchar_hook)(byte c);
 };
 
 
@@ -252,18 +257,21 @@ struct term
  *
  * The other actions do not need a "v" code, so "zero" is used.
  */
-#define TERM_XTRA_EVENT	1	/* Process some pending events */
-#define TERM_XTRA_FLUSH 2	/* Flush all pending events */
-#define TERM_XTRA_CLEAR 3	/* Clear the entire window */
-#define TERM_XTRA_SHAPE 4	/* Set cursor shape (optional) */
-#define TERM_XTRA_FROSH 5	/* Flush one row (optional) */
-#define TERM_XTRA_FRESH 6	/* Flush all rows (optional) */
-#define TERM_XTRA_NOISE 7	/* Make a noise (optional) */
-#define TERM_XTRA_BORED 9	/* Handle stuff when bored (optional) */
-#define TERM_XTRA_REACT 10	/* React to global changes (optional) */
-#define TERM_XTRA_ALIVE 11	/* Change the "hard" level (optional) */
-#define TERM_XTRA_LEVEL 12	/* Change the "soft" level (optional) */
-#define TERM_XTRA_DELAY 13	/* Delay some milliseconds (optional) */
+#define TERM_XTRA_EVENT    1    /* Process some pending events */
+#define TERM_XTRA_FLUSH 2    /* Flush all pending events */
+#define TERM_XTRA_CLEAR 3    /* Clear the entire window */
+#define TERM_XTRA_SHAPE 4    /* Set cursor shape (optional) */
+#define TERM_XTRA_FROSH 5    /* Flush one row (optional) */
+#define TERM_XTRA_FRESH 6    /* Flush all rows (optional) */
+#define TERM_XTRA_NOISE 7    /* Make a noise (optional) */
+#define TERM_XTRA_BORED 9    /* Handle stuff when bored (optional) */
+#define TERM_XTRA_REACT 10    /* React to global changes (optional) */
+#define TERM_XTRA_ALIVE 11    /* Change the "hard" level (optional) */
+#define TERM_XTRA_LEVEL 12    /* Change the "soft" level (optional) */
+#define TERM_XTRA_DELAY 13    /* Delay some milliseconds (optional) */
+
+
+/*** Color constants ***/
 
 
 /*
@@ -272,35 +280,66 @@ struct term
  * The "(R,G,B)" codes are given in "fourths" of the "maximal" value,
  * and should "gamma corrected" on most (non-Macintosh) machines.
  */
-#define TERM_DARK       0   /* 'd' */   /* 0,0,0 */
-#define TERM_WHITE      1   /* 'w' */   /* 4,4,4 */
-#define TERM_SLATE      2   /* 's' */   /* 2,2,2 */
-#define TERM_ORANGE     3   /* 'o' */   /* 4,2,0 */
-#define TERM_RED        4   /* 'r' */   /* 3,0,0 */
-#define TERM_GREEN      5   /* 'g' */   /* 0,2,1 */
-#define TERM_BLUE       6   /* 'b' */   /* 0,0,4 */
-#define TERM_UMBER      7   /* 'u' */   /* 2,1,0 */
-#define TERM_L_DARK     8   /* 'D' */   /* 1,1,1 */
-#define TERM_L_WHITE    9   /* 'W' */   /* 3,3,3 */
-#define TERM_VIOLET     10  /* 'v' */   /* 4,0,4 */
-#define TERM_YELLOW     11  /* 'y' */   /* 4,4,0 */
-#define TERM_L_RED      12  /* 'R' */   /* 4,0,0 */
-#define TERM_L_GREEN    13  /* 'G' */   /* 0,4,0 */
-#define TERM_L_BLUE     14  /* 'B' */   /* 0,4,4 */
-#define TERM_L_UMBER    15  /* 'U' */   /* 3,2,1 */
+#define TERM_DARK     0  /* d */    /* 0 0 0 */
+#define TERM_WHITE    1  /* w */    /* 4 4 4 */
+#define TERM_SLATE    2  /* s */    /* 2 2 2 */
+#define TERM_ORANGE   3  /* o */    /* 4 2 0 */
+#define TERM_RED      4  /* r */    /* 3 0 0 */
+#define TERM_GREEN    5  /* g */    /* 0 2 1 */
+#define TERM_BLUE     6  /* b */    /* 0 0 4 */
+#define TERM_UMBER    7  /* u */    /* 2 1 0 */
+#define TERM_L_DARK   8  /* D */    /* 1 1 1 */
+#define TERM_L_WHITE  9  /* W */    /* 3 3 3 */
+#define TERM_L_PURPLE 10 /* P */    /* ? ? ? */
+#define TERM_YELLOW   11 /* y */    /* 4 4 0 */
+#define TERM_L_RED    12 /* R */    /* 4 0 0 */
+#define TERM_L_GREEN  13 /* G */    /* 0 4 0 */
+#define TERM_L_BLUE   14 /* B */    /* 0 4 4 */
+#define TERM_L_UMBER  15 /* U */    /* 3 2 1 */
+
+#define TERM_PURPLE      16    /* p */
+#define TERM_VIOLET      17    /* v */
+#define TERM_TEAL        18    /* t */
+#define TERM_MUD         19    /* m */
+#define TERM_L_YELLOW    20    /* Y */
+#define TERM_MAGENTA     21    /* i */
+#define TERM_L_TEAL      22    /* T */
+#define TERM_L_VIOLET    23    /* V */
+#define TERM_L_PINK      24    /* I */
+#define TERM_MUSTARD     25    /* M */
+#define TERM_BLUE_SLATE  26    /* z */
+#define TERM_DEEP_L_BLUE 27    /* Z */
+
+/* The following allow color 'translations' to support environments with a limited color depth
+ * as well as translate colours to alternates for e.g. menu highlighting. */
+
+#define ATTR_FULL        0    /* full color translation */
+#define ATTR_MONO        1    /* mono color translation */
+#define ATTR_VGA         2    /* 16 color translation */
+#define ATTR_BLIND       3    /* "Blind" color translation */
+#define ATTR_LIGHT       4    /* "Torchlit" color translation */
+#define ATTR_DARK        5    /* "Dark" color translation */
+#define ATTR_HIGH        6    /* "Highlight" color translation */
+#define ATTR_METAL       7    /* "Metallic" color translation */
+#define ATTR_MISC        8    /* "Miscellaneous" color translation - see misc_to_attr */
+
+#define MAX_ATTR        9
 
 /*
  * Maximum number of colours, and number of "basic" Angband colours
  */ 
-#define MAX_COLORS		256
-#define BASIC_COLORS	16
+#define MAX_COLORS        256
+#define BASIC_COLORS    28
 
 
 
 /**** Available Variables ****/
 
 extern term *Term;
-
+extern byte tile_width;
+extern byte tile_height;
+extern bool bigcurs;
+extern bool smlcurs;
 
 /**** Available Functions ****/
 
@@ -308,6 +347,7 @@ extern errr Term_user(int n);
 extern errr Term_xtra(int n, int v);
 
 extern void Term_queue_char(term *t, int x, int y, byte a, char c, byte ta, char tc);
+extern void Term_big_queue_char(term *t, int x, int y, byte a, char c, byte a1, char c1);
 extern void Term_queue_chars(int x, int y, int n, byte a, cptr s);
 
 extern errr Term_fresh(void);
@@ -317,6 +357,7 @@ extern errr Term_draw(int x, int y, byte a, char c);
 extern errr Term_addch(byte a, char c);
 extern errr Term_addstr(int n, byte a, cptr s);
 extern errr Term_putch(int x, int y, byte a, char c);
+extern void Term_big_putch(int x, int y, byte a, char c);
 extern errr Term_putstr(int x, int y, int n, byte a, cptr s);
 extern errr Term_erase(int x, int y, int n);
 extern errr Term_clear(void);
