@@ -15,15 +15,15 @@
  *    and not for profit purposes provided that this copyright and statement
  *    are included in all such copies.  Other copyrights may also apply.
  */
+
+#include <stdlib.h>
+
 #include "z-util.h"
 
-
-
 /*
  * Convenient storage of the program name
  */
-cptr argv0 = NULL;
-
+char *argv0 = NULL;
 
 /*
  * Case insensitive comparison between two strings
@@ -267,74 +267,34 @@ void quit(cptr str)
 	(void)(exit(EXIT_FAILURE));
 }
 
-
-
-
-/* Compare and swap hooks */
-bool (*ang_sort_comp)(const void *u, const void *v, int a, int b);
-void (*ang_sort_swap)(void *u, void *v, int a, int b);
-
-
-/*
- * Angband sorting algorithm -- quick sort in place
- *
- * Note that the details of the data we are sorting is hidden,
- * and we rely on the "ang_sort_comp()" and "ang_sort_swap()"
- * function hooks to interact with the data, which is given as
- * two pointers, and which may have any user-defined form.
- */
-void ang_sort_aux(void *u, void *v, int p, int q)
+/* Arithmetic mean of the first 'size' entries of the array 'nums' */
+int mean(int *nums, int size)
 {
-	int z, a, b;
+    	int i, total = 0;
 
-	/* Done sort */
-	if (p >= q) return;
+    	for(i = 0; i < size; i++) total += nums[i];
 
-	/* Pivot */
-	z = p;
+    	return total / size;
+}
 
-	/* Begin */
-	a = p;
-	b = q;
+/* Variance of the first 'size' entries of the array 'nums'  */
+int variance(int *nums, int size)
+{
+    	int i, avg, total = 0;
 
-	/* Partition */
-	while (TRUE)
+    	avg = mean(nums, size);
+
+    	for(i = 0; i < size; i++)
 	{
-		/* Slide i2 */
-		while (!(*ang_sort_comp)(u, v, b, z)) b--;
+        	int delta = nums[i] - avg;
+        	total += delta * delta;
+    	}
 
-		/* Slide i1 */
-		while (!(*ang_sort_comp)(u, v, z, a)) a++;
-
-		/* Done partition */
-		if (a >= b) break;
-
-		/* Swap */
-		(*ang_sort_swap)(u, v, a, b);
-
-		/* Advance */
-		a++, b--;
-	}
-
-	/* Recurse left side */
-	ang_sort_aux(u, v, p, b);
-
-	/* Recurse right side */
-	ang_sort_aux(u, v, b+1, q);
+    	return total / size;
 }
 
-
-/*
- * Angband sorting algorithm -- quick sort in place
- *
- * Note that the details of the data we are sorting is hidden,
- * and we rely on the "ang_sort_comp()" and "ang_sort_swap()"
- * function hooks to interact with the data, which is given as
- * two pointers, and which may have any user-defined form.
- */
-void ang_sort(void *u, void *v, int n)
+void sort(void *base, size_t nmemb, size_t smemb,
+          int (*comp)(const void *, const void *))
 {
-	/* Sort the array */
-	ang_sort_aux(u, v, 0, n-1);
+	qsort(base, nmemb, smemb, comp);
 }
-
