@@ -9,7 +9,7 @@
 /**
  * The name of the program.
  */
-extern const char *argv0;
+extern char *argv0;
 
 
 /* Aux functions */
@@ -82,5 +82,11 @@ void ang_sort_aux(void *u, void *v, int p, int q);
 extern bool (*ang_sort_comp)(const void *u, const void *v, int a, int b);
 extern void (*ang_sort_swap)(void *u, void *v, int a, int b);
 
+extern void sort(void *array, size_t nmemb, size_t smemb,
+                 int (*comp)(const void *a, const void *b));
+
+/* Mathematical functions */
+int mean(int *nums, int size);
+int variance(int *nums, int size);
 
 #endif /* INCLUDED_Z_UTIL_H */
